<?php
/**
 * Newsletter and Email marketing solution for SCHLIX CMS - Config
 *
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */

if (!defined('SCHLIX_VERSION')) die('No Access');
?>
<?php
$all_wysiwyg_profiles = \App\wysiwygEditors_Profiles::getAllWysiwygProfiles();
?>
<schlix-data-editor id="schlix-<?= $this->app_name ?>-admin" class="schlix-data-editor" data-app="<?= $this->app->getApplicationNameOnly() ?>" data-main-app-description="<?= $this->app->getApplicationDescription().' '.___(': Configuration') ?>"  data-schlix-controller="SCHLIX.CMS.NewslettersAdminController" data-model="<?= $this->getDataModelURL() ?>" data-field-id="<?= $this->app->getFieldID() ?>" data-cancel-warning="true">
<div class="content">
        <?php 
            ///////////////////////////////
            // Display save error, if any
            $this->displayLastConfigSaveResult();
            ///////////////////////////////
        ?>        

        <form method="post" action="<?= $this->createFriendlyAdminURL("action=saveconfig") ?>" autocomplete="off" >
            <div class="row">
                <div class="col-md-8">
                    <!-- -->
                </div>
                <div class="col-md-4 text-right" schlix-id="data-editor-toolbar">
                    <button type="submit" class="btn btn-success schlix-command-button" data-schlix-command="save-item" name="btn_save"><i class="fa fa-check"></i> <?= ___('Save'); ?></button>
                    <button type="submit" class="btn btn-danger schlix-command-button" data-schlix-command="cancel-item" name="btn_cancel"><i class="fa fa-times"></i> <?= ___('Cancel'); ?></button>
                </div>            
            </div>        

            <?= \INPUT::CSRF() ?>

            <!-- begin tabs -->
            <div class="schlixui-tabcontainer">
                <!-- tab navigation -->
                <ul class="schlixui-tabheader">
                    <li  class="selected"><a href="#tab1"><em><?= ___('Main page') ?></em></a></li>                
                    <li><a href="#tab2"><em><?= ___('Application Configuration') ?></em></a></li>
                    <li><a href="#tab_smtp_server"><em><?= ___('SMTP Server') ?></em></a></li>
                    <li><a href="#tab3"><em><?= ___('Default Main page Options') ?></em></a></li>
                    <li><a href="#tab4"><em><?= ___('Default Category Options') ?></em></a></li>
                    <li><a href="#tab5"><em><?= ___('Default Item Options') ?></em></a></li>                    
                    <li><a href="#tab6"><em><?= ___('Permissions') ?></em></a></li>
                </ul>
                <!-- end tab navigation -->
                <!-- tab ocntent -->
                <div class="schlixui-tabcontent">
                    <!-- tab -->
                    <div id ="tab1">
                        <schlix-config:textbox config-key='str_mainpage_title' label='<?= ___('Main page title') ?>' class='form-control' />                        
                        <schlix-config:textarea config-key='str_mainpage_text' label="<?= ___('Main page Text') ?>" class='wysiwyg' />             

                    </div>                
                    <!-- tab -->
                    <div id="tab2">

                        <schlix-config:app_alias class='form-control' />
                        <schlix-config:app_description class='form-control' />
                        
                        <schlix-config:integerbox config-key='int_activation_expiry_hours' class="form-control" label="<?= ___('New user activation expires in (hours)') ?>" config-default-value="24"  />
                        
                            <schlix-config:dropdownlist class="form-control" config-key='int_wysiwyg_profile_id' label='<?=  ___('Use this WYSIWYG Editor profile') ?>'>
                            <?php foreach ($all_wysiwyg_profiles as $profile): ?>
                                <schlix-config:option value="<?= $profile['id'] ?>"><?= SAFE_HTML('#'.$profile['id'].' '.$profile['editor_name'].': '.$profile['title'].' ('.$profile['virtual_filename'].')') ?> </schlix-config:option>
                            <?php endforeach ?>
                            </schlix-config:dropdownlist>
                        
                        <schlix-config:textbox config-key='str_rss_title' label='<?= ___('RSS Title') ?>' class='form-control' />
                        <schlix-config:textbox config-key='str_default_name' label='<?= ___('Default Newsletter Author Name') ?>' class='form-control' />
                        <schlix-config:textbox config-key='str_default_field_from' label='<?= ___('Default From E-mail Address') ?>' class='form-control' />
                        <schlix-config:textbox config-key='str_default_field_replyto' label='<?= ___('Default Reply-To E-mail Address') ?>' class='form-control' />
                        
                        <schlix-config:textbox config-key='str_description' label='<?= ___('Default Description') ?>' class='form-control' />

                        <schlix-config:textbox config-key='str_meta_keywords' label='<?= ___('Default Meta Keywords') ?>' class='form-control' />
                        <schlix-config:textbox config-key='str_meta_description' label='<?= ___('Default Meta Description') ?>' class='form-control' />

                        


                        <schlix-config:checkbox config-key='bool_convert_external_images' label='<?= ___('Fetch external images for copy and paste operation') ?>' />
                        <schlix-config:checkbox config-key='bool_enable_pageview_stats' label='<?= ___('Enable page impression stats') ?>' />


                    </div>
                    <div id ="tab_smtp_server">
                    <h3><?= ___('Outgoing SMTP server') ?></h3>
                    <schlix-config:integerbox min="1" config-key='int_max_email_per_batch' class="form-control" label="<?= ___('Max emails/batch send') ?>" config-default-value="10" />
                    <schlix-config:integerbox min="1" config-key='int_max_email_per_hour' class="form-control" label="<?= ___('Max emails/hour') ?>" config-default-value="200" />
                    <schlix-config:integerbox min="1" config-key='int_email_sleep_second' class="form-control" label="<?= ___('Pause for x second between messages') ?>" config-default-value="0" min="0" max="60" />
                    <em>
                        <?= (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN' && $configvar['SCHLIX_SMTP_HOST'] == '') ? ___('Please configure the SMTP server, otherwise registration/contact mail will not work') : ___('leave the SMTP settings below blank to use the default settings') ?></em>
                    <br />

                        <schlix-config:checkbox config-key='bool_use_smtp_server' label='<?= ___('Use SMTP server') ?>' />
                    
                        <schlix-config:textbox config-key="str_smtp_host" id='smtp_host' class="form-control" label="<?= ___('Host') ?>"  />

                        <schlix-config:integerbox config-key='int_smtp_port' class="form-control" label="<?= ___('Port') ?>"  />
                    <schlix-config:checkbox config-key='bool_smtp_use_auth' label='<?= ___('Use SMTP authentication') ?>' />
                    <schlix-config:textbox config-key="str_smtp_username" id='smtp_host' class="form-control" label="<?= ___('Username') ?>"  />
                    <schlix-config:textbox config-key="str_smtp_password" id='smtp_host' class="form-control" label="<?= ___('Password') ?>"  />
                    <schlix-config:dropdownlist config-key="str_smtp_secure_with" label="<?= ___('Security') ?>" class="form-control" >
                        <schlix-config:option value=""><?= ___('None') ?></schlix-config:option>
                        <schlix-config:option value="ssl"><?= ___('SSL') ?></schlix-config:option>
                        <schlix-config:option value="tls"><?= ___('TLS') ?></schlix-config:option>
                    </schlix-config:dropdownlist>


                    </div>                
                     
                    <!-- tab -->
                    <div id="tab3">
                    <schlix-config:integerbox config-key='int_mainpage_items_per_page' config-default-value="10" label='<?= ___('Default maximum mumber of items to be displayed per page') ?>' class='form-control' />
                    <schlix-config:meta_options config-key='array_mainpage_meta_options' config-options="<?= \SCHLIX\cmsConfigRegistry::encodeArray($this->app->getMainpageMetaOptionKeys()) ?>" config-default-value="<?= \SCHLIX\cmsConfigRegistry::encodeArray($this->app->getDefaultMainpageMetaOptionKeys()) ?>" item-per-column="1" column-count="3" />
                    
                    </div>
                    <!-- tab -->
                    <div id="tab4">
                        <schlix-config:integerbox config-key='int_category_items_per_page' config-default-value="10" label='<?= ___('Default maximum mumber of items to be displayed per page') ?>' class='form-control' />
                        <schlix-config:checkbox config-key='reset_category_items_per_page' label='<?=  ___('Reset the number of items per page for all categories') ?>' />
                        <schlix-config:meta_options config-key='array_default_category_meta_options' config-options="<?= \SCHLIX\cmsConfigRegistry::encodeArray($this->app->getCategoryMetaOptionKeys()) ?>" config-default-value="<?= \SCHLIX\cmsConfigRegistry::encodeArray($this->app->getDefaultCategoryMetaOptionKeys()) ?>" item-per-column="1" column-count="3" />
                        <schlix-config:checkbox config-key='reset_category_options' label='<?=  ___('Reset all categories options') ?>' />
                    </div>
                    <!-- tab -->
                    <div id="tab5">

                       <schlix-config:meta_options config-key='array_default_item_meta_options' config-options="<?= \SCHLIX\cmsConfigRegistry::encodeArray($this->app->getItemMetaOptionKeys()) ?>" config-default-value="<?= \SCHLIX\cmsConfigRegistry::encodeArray($this->app->getDefaultItemMetaOptionKeys()) ?>" item-per-column="1" column-count="3" />
                       <schlix-config:checkbox config-key='reset_item_options' label='<?=  ___('Reset all item options') ?>' />
                    </div>                
                    <!-- tab -->
                    <div id="tab6">

                        <schlix-config:permission-usergroups config-key='array_groups_allowed_for_backend_access' label="<?= ___('Enable backend access for the following user groups') ?>" />

                        <p><?= ___('Administrators will always be allowed to access to the backend, regardless of the settings above. If nothing is specified, all users who already have access to the backend can access this extension.'); ?></p>
                    </div>
                    <!-- tab -->                                    
                </div>
            </div>
            <br />
        </form>
</div>
</schlix-data-editor>
