<?php
/**
 * Newsletter and Email marketing solution for SCHLIX CMS - Edit Item Template
 *
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
if (!defined('SCHLIX_VERSION')) die('No Access');
?>
<?php
global $CurrentUser;
if ($id == 'new') {
    $item['permission_read'] = serialize('everyone');
    $header_title = ___('New Subscriber');
    $is_existing_subscriber = false;
} else {
    $header_title = ___('Edit Subscriber').' #'.$id;
    $is_existing_subscriber = true;
}

$available_field_types = [ 
    ['label' => ___('Text Box'), 'value' => 'textbox' ],
    ['label' => ___('Drop Down'), 'value' => 'dropdown' ],
    ['label' => ___('Radio Group'), 'value' => 'radiogroup'],
    ['label' => ___('Check Box'), 'value' => 'checkbox'],
    //['label' => ___('Check Box Group'), 'value' => 'checkboxgroup'],
];
?>
<schlix-data-editor id="schlix-<?= $this->app_name ?>-admin" class="schlix-data-editor" data-app="<?= $this->app->getApplicationNameOnly() ?>" data-main-app-description="<?= $this->app->getApplicationDescription() . ': ' . $header_title ?>" data-schlix-controller="SCHLIX.CMS.NewslettersCustomFieldsAdmin" data-model="<?= $this->getDataModelURL() ?>" data-field-id="<?= $this->app->getFieldID() ?>" data-cancel-warning="true" >
   <div class="content"> 
        <form id="frm_item_editor" method="post" action="<?= $this->createFriendlyAdminURL('action=saveitem') ?>" enctype="multipart/form-data" >
            <?= \INPUT::CSRF() ?>
            <input type="hidden" name="id" id="id" value="<?= $id; ?>" />
            <input type="hidden" name="guid" id="guid" value="<?= $item['guid']; ?>" />
            <div class="row">
                <div class="col-md-8 col-lg-8">
                    <!-- Page Title -->
                    <div class="form-group">
                        <label for="title"><?= ___("Field Name") ?></label>
                        <div class="input-group"
                             <!-- title -->
                             <input name="title" id="title" type="text" class="form-control"  size="49" maxlength="254"  required="required"   value="<?= SAFE_HTML($item['title']) ?>" placeholder="<?= ___('e.g. First Name') ?>" /> 
                            <!-- version -->
                            <span class="input-group-addon">
                                <schlix-cms-document-status id="status" data-input-name="status" data-value="<?= ($item['status'] == 1 || $item['id'] == 'new') ?>" 
                                                            data-icon-yes="fa fa-check" data-icon-no="fa fa-times" data-color-yes="green" data-color-no="red" 
                                                            data-text-yes="<?= ___('Shown in list view') ?>" data-text-no="<?= ___('Do not show this field in list view') ?>"></schlix-cms-document-status>
                            </span>
                            
                        </div>
                    </div>
                    <!-- End Page Title --> 
                    
                    <!-- Virtual Filename -->
                    <div class="form-group">
                        <label for="virtual_filename"><?= ___('Virtual field name in the database (alphanumeric only, lowercase preferred, no spaces)'); ?></label>                
                        <div class="input-group">
                            <span class="input-group-addon"><i class="fa fa-terminal"></i> </span>

                            <input name="virtual_filename" required="required"  maxlength="255" id="virtual_filename" class="form-control" type="text" value="<?= SAFE_HTML($item['virtual_filename']); ?>" placeholder="<?= ___('e.g. first_name') ?>"   />
                            
                        </div>
                    </div>
                    <!-- END Virtual Filename -->
                    <!-- field -->
                    <div class="form-group">
                        <label for="virtual_filename"><?= ___('Sort Order'); ?></label>                
                        <div class="input-group">
                            <input name="sort_order" required="required"  maxlength="255" id="sort_order" class="form-control" type="number" min="0" max="99999" value="<?= (int) ($item['sort_order']) ?>" />
                            
                        </div>
                    </div>
                    <!-- END field -->
                    <!-- field -->
                    <div class="form-group">
                        <label for="virtual_filename"><?= ___('Optional Default Value'); ?></label>                
                        <div class="input-group">
                            <input name="default_value" maxlength="255" id="sort_order" class="form-control" type="text" value="<?= ($item['default_value']) ?>" />
                            
                        </div>
                    </div>
                    <!-- END field -->
                    <!-- field -->
                    <div class="form-group">
                        <label for="virtual_filename"><?= ___('Optional Validation Regex Pattern'); ?></label>                
                        <div class="input-group">
                            <input name="regex_pattern" maxlength="255" id="regex_pattern" class="form-control" type="text" value="<?= ($item['regex_pattern']) ?>" />
                            
                        </div>
                    </div>
                    <!-- END field -->
                    
                    <fieldset>
                        <legend><?= ___('Field Type') ?></legend>
                        
                        <?php foreach ($available_field_types as $ft): ?>
                        <div class="radio">
                            <label><input required="required" class="field_type" type="radio" name="field_type" id="ft_<?= $ft['value'] ?>" value="<?= $ft['value'] ?>" <?= $ft['value'] == $item['field_type'] ? 'checked' : '' ?> /> <?= $ft['label'] ?></label>
                        </div>
                        <?php endforeach ?>
                    </fieldset>
                </div>
                <div class="col-md-4 col-lg-4 text-right" schlix-id="data-editor-toolbar">
                    <button type="submit" class="btn btn-success schlix-command-button" data-schlix-command="save-item" name="btn_save"><i class="fa fa-check"></i> <?= ___('Save'); ?></button>
                    <button type="submit" class="btn btn-info schlix-command-button" data-schlix-command="apply-item" name="btn_save"><i class="fa fa-caret-square-o-right"></i> <?= ___('Apply'); ?></button>
                    <button type="submit" class="btn btn-danger schlix-command-button" data-schlix-command="cancel-item" name="btn_save"><i class="fa fa-times"></i> <?= ___('Cancel'); ?></button>

                </div>            
            </div>

            <!-- main -->
            <div class="clearboth"></div>

            <?php
            ///////////////////////////////
            // Display save error, if any
            $this->displayLastSaveResult()
            ///////////////////////////////
            ?>

            <!-- end main section --> 
            <fieldset id="value_choices_container" ><legend><?= ___('Value Choices') ?></legend>
                <p><?= sprintf(___('Type each option value with the following format: %s, one item per line'), '<strong>value;label</strong>') ?></p>
            <textarea class="form-control" name="value_choices" style="min-height:400px" id="value_choices"><?= SAFE_HTML($item['value_choices']) ?></textarea>
            </fieldset>

        </form>
    </div>
</schlix-data-editor>