<?php
/**
 * Newsletter and Email marketing solution for SCHLIX CMS - Edit Item Template
 *
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
if (!defined('SCHLIX_VERSION')) die('No Access');
?>
<?php
global $CurrentUser;

$available_custom_fields = $this->app->createSubClassInstance('CustomFields')->getFieldsForNewslettersEditing();
if ($id == 'new') {
    $item['permission_read'] = serialize('everyone');
    $header_title = ___('New Newsletter Issue');
    
    $template_vfn = fget_string('fromtemplate');
    if (!is_postback() && $template_vfn)
    {
        $template = $this->app->createSubClassInstance('Templates')->getTemplateByVirtualFileName($template_vfn); 
        if ($template)
        {
            $item['title'] = $template['title'];
            $item['body_text'] = $template['body_text'];
            $item['body_html_original'] = $template['body_html'];
        }
    }
} else {
    $preview_link = $this->app->createFriendlyURL("action=viewitem&id={$id}");
    $header_title = ___('Edit Newsletter Issue').' #'.$id;
    
    $sending_to = ___('all subscribers');
    if ($item['subscribers_segment_id'] > 0)
    {
        $segment_item = $this->app->createSubClassInstance('SubscriberSegments')->getItemByID($item['subscribers_segment_id']);
        $sending_to = $segment_item['title'].' ('.___('segment')." #{$item['subscribers_segment_id']}".') ~ ('.number_format($segment_item['snapshot_count']).' '.___('subscribers'). ')';
    }
    
}
$category = $this->getCategoryByID($item['category_id']);
$item['email_from'] = ($id == 'new' || empty($item['email_from']))  ?  $category['email_from'] : $item['email_from'] ;
$item['email_sender'] = ($id == 'new' || empty($item['email_sender']))  ? $category['email_sender'] : $item['email_sender'] ;
$item['email_replyto'] = ($id == 'new' || empty($item['email_replyto']))  ? $category['email_replyto'] : $item['email_replyto'] ;

$parent_category_text = $this->getItemParentURLPathPreviewLink($item); 

if ($item['id'] == 'new') 
{
    $item['status'] = 0;
}
switch ($item['status'])
{
    case NEWSLETTER_ISSUE_CANCELLED_UNPUBLISHED: 
        $status_text = 'Terminated and unpublished';
        $status_icon = 'fa fa-times-circle';
        $status_icon_color = 'red';
        break;

    case NEWSLETTER_ISSUE_HIDDEN: 
        $status_text = 'Hidden';
        $status_icon = 'fa fa-file';
        $status_icon_color = 'orange';
        
        break;
    case NEWSLETTER_ISSUE_DRAFT: 
        $status_text = 'Draft';
        $status_icon = 'fa fa-file-text';
        $status_icon_color = 'gray';
        
        break;
    case NEWSLETTER_ISSUE_WAIT_TOBE_QUEUED: 
        $status_text = 'Published and waiting to be queued';
        $status_icon = 'fa fa-calendar-check-o';
        $status_icon_color = 'green';
        
        break;
    case NEWSLETTER_ISSUE_SENDING: 
        $status_text = 'Sending';
        $status_icon = 'fa fa-exchange';
        $status_icon_color = 'green';
        break;    
    case NEWSLETTER_ISSUE_SENT_AND_FINISHED: 
        $status_text = 'Sent and finished';
        $status_icon = 'fa fa-check-circle';
        $status_icon_color = 'green';
        break;
    case NEWSLETTER_ISSUE_PAUSED: 
        $status_text = 'Paused';
        $status_icon = 'fa fa-exclamation-triangle';
        $status_icon_color = 'orange';
        break;
    case  NEWSLETTER_ISSUE_CANCELLED_PARTIALLY_SENT: 
        $status_text = 'Cancelled, partially sent';
        $status_icon = 'fa fa-exclamation-triangle';
        $status_icon_color = 'orange';
        break;

}

$can_modify = ($item['status'] ==NEWSLETTER_ISSUE_DRAFT) || ($item['status'] ==NEWSLETTER_ISSUE_PAUSED);
                                
?>
<schlix-data-editor id="schlix-<?= $this->app_name ?>-admin" class="schlix-data-editor" data-app="<?= $this->app->getApplicationNameOnly() ?>" data-main-app-description="<?= $this->app->getApplicationDescription() . ': ' . $header_title ?>" data-schlix-controller="SCHLIX.CMS.NewslettersAdminController" data-model="<?= $this->getDataModelURL() ?>" data-field-id="<?= $this->app->getFieldID() ?>" data-cancel-warning="true" >
   <div class="content"> 
        <form id="frm_send_newsletter_issue" method="post" action="<?= $this->createFriendlyAdminURL('action=saveitem') ?>" enctype="multipart/form-data" >
            <?= \INPUT::CSRF() ?>
            <input type="hidden" name="id" id="id" value="<?= $id; ?>" />
            <input type="hidden" name="category_id" value="<?= $item['category_id']; ?>" />
            <input type="hidden" name="guid" id="guid" value="<?= $item['guid']; ?>" />
            <input type="hidden" name="send_test_mail" id="send_test_mail" value="0" />
            <input type="hidden" name="send_newsletter_issue_now" id="send_newsletter_issue_now" value="0" />
            <div class="row">
                <div class="col-md-4">
                    <!-- field -->
                    <div class="form-group">                            
                        <label for="email_name"><?= ___('Email Sender Name') ?></label>
                        <div class="input-group">
                            <span class="input-group-addon"><i class="fa fa-user"></i></span>
                         <input type="text" class="form-control" id="email_sender" name="email_sender" value="<?= SAFE_HTML($item['email_sender']); ?>" placeholder="<?= SCHLIX_MAIL_DEFAULT_SENDER ?>" />
                        </div>
                    </div>
                </div>
                <div class="col-md-4">
                    <!-- field -->
                    <div class="form-group">                            
                        <label for="email_from"><?= ___('Email From') ?></label>
                        <div class="input-group">
                            <span class="input-group-addon"><i class="fa fa-envelope-open-o"></i></span>
                         <input type="email" class="form-control" id="email_from" name="email_from" value="<?= SAFE_HTML($item['email_from']); ?>" placeholder="<?= SCHLIX_MAIL_DEFAULT_EMAIL ?>" />
                        </div>
                    </div>
                </div>
                <div class="col-md-4">                    
                    <!-- field -->
                    <div class="form-group">                            
                        <label for="email_replyto"><?= ___('Email Reply-To') ?></label>
                        <div class="input-group">
                            <span class="input-group-addon"><i class="fa fa-envelope-square"></i></span>
                         <input type="email" class="form-control" id="email_replyto" name="email_replyto" value="<?= SAFE_HTML($item['email_replyto']); ?>" placeholder="<?= SCHLIX_MAIL_DEFAULT_EMAIL ?>" />
                        </div>
                    </div>
                </div>

                    
                
            </div>
            
            <div class="row">
                <div class="col-md-8 col-lg-8">
                    <!-- Page Title -->
                    <div class="form-group">
                        <label for="title"><?= ___("Newsletter Issue Title/Email Subject") ?></label>
                        <div class="input-group"
                             <!-- title -->
                             <input name="title" id="title" type="text" class="form-control"  size="49" maxlength="254"  required="required"   value="<?= SAFE_HTML($item['title']) ?>" /> 
                            <!-- version -->
                            <?php if ($item['version'] > 0): ?>
                                <span class="input-group-addon">

                                    <?= ___('Version') . ': ' . $item['version']; ?>
                                </span>
                            <?php endif; ?>
                            <!-- end version -->

                            <!-- Status -->
                            <span class="input-group-addon">
                                <i class="<?= $status_icon ?>" style="color: <?= $status_icon_color ?>"></i> <?= $status_text ?>
                            </span>
                            <!-- Featured -->
                        </div>
                    </div>
                    <!-- End Page Title -->
                    <!-- Virtual Filename -->
                    <div class="form-group">
                        <label for="virtual_filename"><?= ___('Search Engine Friendly Shortcut'); ?></label>                
                        <div class="input-group">
                            <span class="input-group-addon"><?= $parent_category_text ?></span>

                            <input name="virtual_filename" size="49" maxlength="255" id="virtual_filename" class="form-control" required="required" pattern="[a-zA-Z0-9-_]+" type="text" value="<?= SAFE_HTML($item['virtual_filename']); ?>" />
                            <span class="input-group-addon">.html</span>
                            <?php if ($id !== 'new'): ?>
                                <span class="input-group-addon"> <a href="<?= $preview_link; ?>" target="_blank"><i class="fa fa-eye"></i>&nbsp;<?= ___('Preview') ?></a></span>
                            <?php endif ?>
                        </div>

                    </div>
                    <!-- END Virtual Filename -->
 
                    <!-- END field -->

                </div>
                <div class="col-md-4 col-lg-4 text-right" schlix-id="data-editor-toolbar">
                    <?php if ($can_modify): ?>
                    <button type="submit" class="btn btn-success schlix-command-button" data-schlix-command="save-item" name="btn_save"><i class="fa fa-check"></i> <?= ___('Save'); ?></button>
                    <button type="submit" class="btn btn-info schlix-command-button" data-schlix-command="apply-item" name="btn_save"><i class="fa fa-caret-square-o-right"></i> <?= ___('Apply'); ?></button>
                    <?php endif ?>
                    <button type="submit" class="btn btn-danger schlix-command-button" data-schlix-command="cancel-item" name="btn_save"><i class="fa fa-times"></i> <?= ___('Cancel'); ?></button>

                </div>            
            </div>

            <!-- main -->
            <div class="clearboth"></div>

            <?php
            ///////////////////////////////
            // Display save error, if any
            $this->displayLastSaveResult()
            ///////////////////////////////
            ?>

            <!-- end main section -->
            <!-- begin tabs -->
            <div id="schlix_editor_main_content" class="schlixui-tabcontainer">
                <!-- tab navigation -->
                <ul class="schlixui-tabheader">
                    <li class="selected"><a href="#tab1"><em><?= ___('Content') ?></em></a></li>
                    
                    <li><a href="#tab_logs"><em><?= ___('Logs') ?></em></a></li>
                    
                    <li><a href="#tab2"><em><?= ___('Meta') ?></em></a></li>
                    <li><a href="#tab3"><em><?= ___('Dates') ?></em></a></li>
                    <li><a href="#tab4"><em><?= ___('Options') ?></em></a></li>
                    <li><a href="#tab5"><em><?= ___('Permissions') ?></em></a></li>
                    <li><a href="#tab6"><em><?= ___('Versioning') ?></em></a></li>
                </ul>
                <!-- end tab navigation -->
                <!-- tab ocntent -->
                <div class="schlixui-tabcontent">
                    <!-- tab -->
                    <div id="tab1">
                        <!-- tab 1 content -->
                        <?= ___('Available variable fields') ?>: 
                        <ul>
                            <?php foreach ($available_custom_fields as $field): ?>
                            <li><strong><?= '{$'.$field['field'].'}' ?></strong> - <?= SAFE_HTML($field['description']) ?></li>
                            <?php endforeach ?>
                        </ul>
                        
                        <schlix-cms-document-status id="image_is_embedded" data-input-name="image_is_embedded" data-value="<?= ($item['image_is_embedded'] == 1) ?>" 
                                                            data-icon-yes="fa fa-frown-o" data-icon-no="fa fa-image" data-color-yes="red" data-color-no="green" 
                                                            data-text-yes="<?= ___('Images are embedded - this is not recommended (warning: heavy)') ?>" data-text-no="<?= ___('Images are linked (regular)') ?>"></schlix-cms-document-status>                        
                        <div style="clear:both"></div>
                        
                        <label for="body_html_original"><?= ___('HTML Body') ?></label>
                        <textarea class="wysiwyg form-control" name="body_html_original" id="body_html_original"><?= SAFE_HTML($item['body_html_original']) ?></textarea>

                        <div class="clearboth"></div>

                        <label for="body_text"><?= ___('Text Body') ?></label>
                        <textarea class="form-control" name="body_text" id="body_text"><?= SAFE_HTML($item['body_text']) ?></textarea>

                        <div class="clearboth"></div>
                        <p><?= ___('If you use variable templates for the email, you can display an alternate web version without the template') ?></p>
                        <label for="body_html_web"><?= ___('HTML Body (web view version)') ?></label>
                        <textarea class="wysiwyg form-control" name="body_html_web" id="body_html_web"><?= SAFE_HTML($item['body_html_web']) ?></textarea>

                        <div class="clearboth"></div>
                        
                        <!-- end tab 1 content -->
                    </div>
                    <!-- end tab -->
                    <?php if ((int) $id > 0): ?>
                    <!-- tab -->
                    <div id="tab_logs">
                        <!-- tab content -->
                <?php if ($id > 0 && $item['status'] != NEWSLETTER_ISSUE_DRAFT): ?>
                <h3><?= ___('Statistics') ?></h3>
                <div class="row">
                <div class="col-sm-4">
                    <table class="table table-striped">
                        <!-- row -->
                        <tr>
                            <td>
                                <?= ___('Total Queued') ?>
                            </td>
                            <td>
                                <?= \App\Newsletters_Queues::getTotalByQueueMarkerAndStatus($item['guid'], 0) ?>
                            </td>                            
                        </tr>
                        
                        <!-- row -->
                        <tr>
                            <td>
                                <?= ___('Total Sent') ?>
                            </td>
                            <td>
                                <?= \App\Newsletters_Queues::getTotalByQueueMarkerAndStatus($item['guid'], 1) ?>
                            </td>                            
                        </tr>
                        <!-- row -->
                        <tr>
                            <td>
                                <?= ___('Total Failed') ?>
                            </td>
                            <td>
                                <?= \App\Newsletters_Queues::getTotalByQueueMarkerAndStatus($item['guid'], -1) ?>
                            </td>                            
                        </tr>
                        <!-- row -->
                        <tr>
                            <td>
                                <?= ___('Total Cancelled') ?>
                            </td>
                            <td>
                                <?= \App\Newsletters_Queues::getTotalByQueueMarkerAndStatus($item['guid'], -2) ?>
                            </td>                            
                        </tr>
                    </table>
                </div>
                </div>
                <?php endif ?>
                        
                        <?php include ('edit.nlissuelogs.template.php') ?>
                        <!-- end tab content -->
                    </div>
                    <!-- end tab -->
                    <?php endif ?>
                    <!-- tab -->
                    <div id="tab2">
                        <!-- tab content -->
                        <label><?= ___('Meta Keywords') ?><br />
                            <input type="text" class="form-control" name="meta_key" value="<?= SAFE_HTML($item['meta_key']); ?>" />
                        </label>
                        <br />
                        <label><?= ___('Meta Description') ?><br />
                            <input type="text" class="form-control" name="meta_description" value="<?= SAFE_HTML($item['meta_description']); ?>" />
                        </label>
                        <br />
                        <label><?= ___('Tags') ?><br />
                            <input type="text" class="form-control" name="tags" value="<?= SAFE_HTML($item['tags']); ?>" />
                        </label>

                        <!-- end tab content -->
                    </div>
                    <!-- end tab -->
                    <!-- tab -->
                    <div id="tab3">
                        <!-- tab content -->
                        <!-- date created -->
                        <div class="row">
                            <div class="col-sm-2">
                                <label for="date_created"><?= ___('Created') ?></label>
                                <input class="form-control"  type="text" id="date_created" name="date_created" readonly="readonly" value="<?= $item['date_created'] ?>" />
                            </div>
                        </div>
                        <!-- date modified -->
                        <div class="row">
                            <div class="col-sm-2">
                                <label for="date_modified"><?= ___('Modified') ?></label>
                                <input class="form-control" type="text" id="date_modified" name="date_modified" readonly="readonly" value="<?= $item['date_modified'] ?>" />
                            </div>
                        </div>

                        <!-- date available -->
                        <div class="row">
                            <div class="col-sm-2">
                                <label for="date_available"><?= ___('Published on') ?></label>
                                <?= \INPUT::DateTimePicker('date_available', $item['date_available']); ?>
                            </div>
                        </div>

                        <!-- date expiry -->
                        <div class="row">
                            <div class="col-sm-2">
                                <label for="date_expiry"><?= ___('Keep sending to new subscribers until') ?></label>
                                <?= \INPUT::DateTimePicker('date_send_until', $item['date_send_until']); ?>
                            </div>
                        </div> 

                        <!-- end tab content -->
                    </div>
                    <!-- end tab -->
                    <!-- tab -->
                    <div id="tab4">
                        <!-- tab content -->
                        <?php if ($this->app->itemColumnExists('options')): ?>
                            <?= \INPUT::CMSObjectOptions('options', $this->app->getItemMetaOptionKeys(), $item['options'], $this->app->getDefaultItemMetaOptionKeys(), 1, 3);  ?>
                        <?php endif ?>

                        <!-- end tab content -->
                    </div>
                    <!-- end tab -->
                    <!-- tab -->
                    <div id="tab5">
                        <!-- tab content -->
                        <?= \INPUT::UserNameSelectionField('created_by_id', $item['created_by_id'], ___('Owner / Created By'), array('class'=>'form-control')); ?>
                        <?= \INPUT::UserNameSelectionField('modified_by_id', $item['modified_by_id'], ___('Last Modified By'), array('class'=>'form-control')); ?>

                        <fieldset id="fieldset_permission_read">
                            <legend><?= ___('Who can read this page?'); ?></legend>
                            <?= \INPUT::UserGroupReadPermissionCheckboxGroup('read', $item['permission_read']) ?>
                        </fieldset>
                        <fieldset id="fieldset_permission_write">
                            <legend><?= ___('Who can modify this page?') ?></legend>
                            <?= \INPUT::UserGroupWritePermissionCheckboxGroup('write', $item['permission_write'], $CurrentUser->getCurrentUserGroupIDs(), false) ?>
                        </fieldset>
                        <!-- end tab content -->
                    </div>
                    <!-- end tab -->
                    <!-- tab -->
                    <div id="tab6">
                        <!-- eh -->
                        <?php
                        App\Versioning_Admin::displayVersioningGrid($this->app_name, $item['guid']);
                        ?>
                    </div>
                    <!-- end tab -->
                </div>
                <!-- end tab content -->
                                        <div class="alert alert-warning">
                            <?= ___('You must include the following information to be CAN-SPAM and CASL-complianet') ?>
                            <ul>
    <li><?= ___('your name and/or company name') ?></li>
    <li><?= ___('physical mailing address and contact information') ?></li>
    <li><?= ___('an unsubscribe link') ?></li>
                                </ul>

                        </div>

                <!-- field -->
                <?php if ($id != 'new' && $item['status'] == NEWSLETTER_ISSUE_DRAFT): ?>
                <div class="row">
                    <div class="form-group col-sm-6">
                        <div class="input-group">

                            <!-- version -->
                            <span class="input-group-addon">
                                <i class="fa fa-envelope-o"></i> <?= ___('Send test email to the following subscriber'); ?>
                            </span>
                            <input name="test_email_address" id="test_email_address" type="email" class="form-control"  size="49" maxlength="254"  value="<?= SAFE_HTML($item['test_email_address']) ?>" placeholder="email@example.com" /> 

                        </div>

                    </div>
                    <div class="form-group col-sm-6">
                        <button type="submit" class="btn btn-warning schlix-send-button" data-schlix-command="send-testmail" name="btn_send_test" id="btn_send_test"><strong><i class="fa fa-paper-plane"></i> <?= ___('Save and Send Test Email'); ?></strong></button>
                    </div>
                </div>
                <!-- END field -->               
                <hr />
                <!-- field -->
                
                <div class="row">
                    <div class="form-group col-sm-6">
                        <div class="input-group">

                            <!-- version -->
                            <span class="input-group-addon">
                               <?= ___('Send this issue to the following subscribers'); ?> <i class="fa fa-users"></i>
                            </span>
                            <select class="form-control" name="subscribers_segment_id">
                                <option value="0"><?= ___('All Subscribers') ?></option>
                                <?php $segments = $this->app->createSubClassInstance('SubscriberSegments')->getAllItems('*'); ?>
                                <?php if ($segments): ?>
                                    <?php foreach ($segments as $segment): ?>
                                        <option value="<?= (int) $segment['id'] ?>" <?= ((int) $segment['id'] == (int) $item['subscribers_segment_id']) ? 'selected="selected"' : '' ?> ><?= SAFE_HTML($segment['title']) ?></option>
                                    <?php endforeach ?>
                                <?php endif ?>
                            </select>
                        </div>

                    </div>
                    <div class="form-group col-sm-6">
                        <button type="submit" class="btn btn-vk schlix-send-button" data-schlix-command="send-issue-now" name="btn_send_now" id="btn_send_now"><strong><i class="fa fa-paper-plane"></i> <?= ___('Send Now'); ?></strong></button>
                    </div>
                    
                </div>
                <!-- END field -->
                <?php elseif ($item['status'] == NEWSLETTER_ISSUE_WAIT_TOBE_QUEUED): ?>
                <div class="alert alert-info">
                    <p><?= sprintf(___("Newsletter issue is scheduled to be sent to %s on %s but it has not been processed yet"), $sending_to, $item['date_available']) ?></p>
                    <br />
                        <button type="submit" class="btn btn-vk schlix-send-button" data-schlix-command="send-reverttodraft" name="btn_revert_to_draft" id="btn_send_now"><strong><i class="fa fa-undo"></i> <?= ___('Set this issue back to draft so I can modify it'); ?></strong></button>
                </div>
                <?php elseif ($item['status'] == NEWSLETTER_ISSUE_SENDING): ?>
                
                <div class="alert alert-info">
                    <p><?= sprintf(___("Newsletter is currently being sent to %s"), $sending_to) ?></p>
                    <br />
                        <button type="submit" class="btn btn-warning schlix-send-button" data-schlix-command="send-pause" name="btn_pause" id="btn_pause"><strong><i class="fa fa-pause-circle"></i> <?= ___('Pause'); ?></strong></button> &nbsp;&nbsp;&nbsp;&nbsp;
                        <button type="submit" class="btn btn-danger schlix-send-button" data-schlix-command="send-cancel" name="btn_cancel_completely" id="btn_pause"><strong><i class="fa fa-times-rectangle"></i> <?= ___('Cancel completely'); ?></strong></button>
                </div>
                <?php elseif ($item['status'] == NEWSLETTER_ISSUE_PAUSED): ?>
                <div class="alert alert-warning">
                    <p><?= sprintf(___("Newsletter is currently being paused")) ?></p>
                    <br />
                        <button type="submit" class="btn btn-success schlix-send-button" data-schlix-command="send-resume" name="btn_resume" id="btn_resume"><strong><i class="fa fa-play-circle"></i> <?= ___('Resume'); ?></strong></button> &nbsp;&nbsp;&nbsp;&nbsp;
                        <button type="submit" class="btn btn-danger schlix-send-button" data-schlix-command="send-cancel" name="btn_cancel_completely" id="btn_pause"><strong><i class="fa fa-times-rectangle"></i> <?= ___('Cancel and unpublish completely'); ?></strong></button>
                </div>
                
                <?php elseif ($item['status'] == NEWSLETTER_ISSUE_CANCELLED_UNPUBLISHED): ?>
                <div class="alert alert-danger">
                    <p><?= sprintf(___("Newsletter has been unpublished completely and it is no longer editable")) ?></p>
                    
                </div>
                <?php elseif ($item['status'] == NEWSLETTER_ISSUE_SENT_AND_FINISHED): ?>
                <div class="alert alert-info">
                    <p><?= sprintf(___("Newsletter has been sent and it is no longer editable")) ?></p>
                </div>
                
                <?php endif ?>
            </div>
            <!-- end tabs -->
        </form>
    </div>
</schlix-data-editor>