<?php
namespace App;

/**
 * Newsletters - Admin class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 *
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @version 1.0
 * @package newsletters
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
class Newsletters_Admin extends \SCHLIX\cmsAdmin_CategorizedList {

    /**
     * The app
     * @var \App\Newsletters 
     */
    protected $app;
    //_________________________________________________________________________//
    public function __construct() {
        global $WYSIWYGEditor;
        
        $datatype = 'basicnestedcategory';
        $methods = array('standard_main_app' => 'Main Application Page',
            'standard_browse' => 'View a specific item/category'
        );
        parent::__construct($datatype, $methods);
        $this->setItemFieldNamesForAjaxListing('cid', 'id', 'title', 'virtual_filename', 'date_available', 'date_created', 'date_modified', 'sort_order', 'pageview', 'status', 'date_send_until');
        $this->setCategoryFieldNamesForAjaxListing('cid','title', 'virtual_filename', 'date_available', 'date_created', 'date_modified', 'sort_order', 'status');
        
    } 
    /**
     * Validation
     * @param type $datavalues
     */
    public function onGetAdminValidationErrorListBeforeSaveItem($datavalues) {

        $error_list = parent::onGetAdminValidationErrorListBeforeSaveItem($datavalues);
        // If it's a new user or password change flag is checked
        if (!str_contains($datavalues['body_html_original'],'{$unsubscribe_url}') && 
            !str_contains($datavalues['body_html_original'],'{$unsubscribe_generic_url}'))
        {
            $error_list[] = ___('Not compliant with CASL and CAN-SPAM act. You must have {$unsubscribe_url} or {$unsubscribe_generic_url} in the message body for the unsubscribe link');
        }
        $test_email_address = trim($datavalues['test_email_address']);
        if (!empty($test_email_address))
        {
            $subscriber = $this->app->createSubClassInstance('Subscribers')->getLimitedSubscriberInfoByEmail($test_email_address);
            if (!$subscriber)
                $error_list[] = sprintf(___('Cannot find a subscriber with the test email address: %s'), $test_email_address);
            else if ($subscriber['status'] != NEWSLETTER_SUBSCRIBER_ACTIVE)
                $error_list[] = sprintf(___('Subscriber ID# %d with email address %s is inactive'), $subscriber['id'] , $test_email_address);
        }
        
        return $error_list;
    }

            
    public function convertExternalImagesToAbsolute($html)
    {
        $http_prefix = SCHLIX_SITE_SSL_ENABLED && SCHLIX_SITE_HTTPS_URL ? SCHLIX_SITE_HTTPS_URL : SCHLIX_SITE_URL;
        
        $img_file_extensions = ['jpg','jpeg','png'];
        $replacement_table = [];
        preg_match_all('/<img[^>]+>/i',$html, $img_to_replace); 
        if ($img_to_replace[0])
        {
            foreach( $img_to_replace[0] as $img_tag)
            {
                preg_match_all('/(src)=("[^"]*")/i',$img_tag, $img_info);
                //$img_info[0]; // full tag src="/xxxx"
                //$img_info[1]; // src
                //$img_info[2]; // "/xxxx/image.jpg"
                $unquoted_img_src_file = substr($img_info[2][0],1,strlen($img_info[2][0])-2) ;
                if (str_starts_with($unquoted_img_src_file, '/'))
                {
                    $file_ext = pathinfo($unquoted_img_src_file, PATHINFO_EXTENSION);

                    if (in_array($file_ext,$img_file_extensions) )
                    {
                        $replacement_table[$img_info[2][0]] = '"'.$http_prefix.$unquoted_img_src_file.'"';
                    }
                }
            }
        }
        foreach ($replacement_table as $key => $replacement)
        {
            $html = str_replace($key, $replacement, $html);
        }
        return $html;
        
    }
    /**
     * Converts all images into inline images embedded in HTML
     * @param string $html
     * @return string
     */
    public function convertExternalImagestoEmbedded($html)
    {
        $img_file_extensions = ['jpg','jpeg','png'];
        $replacement_table = [];
        preg_match_all('/<img[^>]+>/i',$html, $img_to_replace); 
        if ($img_to_replace[0])
        {
            foreach( $img_to_replace[0] as $img_tag)
            {
                preg_match_all('/(src)=("[^"]*")/i',$img_tag, $img_info);
                //$img_info[0]; // full tag src="/xxxx"
                //$img_info[1]; // src
                //$img_info[2]; // "/xxxx/image.jpg"
                $unquoted_img_src_file = substr($img_info[2][0],1,strlen($img_info[2][0])-2) ;
                $is_remote_file = str_starts_with($unquoted_img_src_file, 'http://') || str_starts_with($unquoted_img_src_file, 'https://');
                if ($is_remote_file || str_starts_with($unquoted_img_src_file, '/'))
                {
                    if ($is_remote_file)
                    {
                        $f_content = get_remote_file_content($unquoted_img_src_file);
                    } else
                    {
                        $img_full_path = SCHLIX_SERVER_ROOT_PATH .$unquoted_img_src_file;                    
                        $f_content = file_get_contents($img_full_path);
                    }
                    $file_ext = pathinfo($unquoted_img_src_file, PATHINFO_EXTENSION);

                    if (in_array($file_ext,$img_file_extensions) && $f_content)
                    {
                        $mime_ext = ($file_ext == 'png') ? 'png' : 'jpg';                
                        $base64_img = "data:image/{$mime_ext};base64,".base64_encode($f_content);
                        $replacement_table[$unquoted_img_src_file] = $base64_img;
                    }
                }
            }
        }
        foreach ($replacement_table as $key => $replacement)
        {
            $html = str_replace($key, $replacement, $html);
        }
        return $html;
    }
    
    /**
     * Modify data before save item
     * @global \App\Users $CurrentUser
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues) {
        global $CurrentUser;
        
        $external_images_save_path = '/media/images/clippings/';
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        
        $current_admin_id = $CurrentUser->getCurrentUserID();
        
        $field_admin_id = ($datavalues['id'] == 'new') ? 'created_by_id' : 'modified_by_id';
        $datavalues[$field_admin_id] = intval($datavalues[$field_admin_id]);
        if (!($datavalues[$field_admin_id] > 0 && $datavalues[$field_admin_id] != $current_admin_id))
            $datavalues[$field_admin_id] = $current_admin_id;
        $datavalues['virtual_filename'] = convert_into_sef_friendly_title($datavalues['virtual_filename'], true);
        if (empty($datavalues['meta_description']))
            $datavalues['meta_description'] = $datavalues['title'];
        if (empty($datavalues['virtual_filename']) || ($datavalues['virtual_filename'] == 'index'))
            $datavalues['virtual_filename'] = 'item' . $datavalues[$this->field_id];
        $datavalues['virtual_filename'] = $this->app->preventDuplicateValueInItemTableUnderCategory('virtual_filename', $datavalues['virtual_filename'], $datavalues[$this->field_id], $datavalues['category_id']);
        if ($datavalues['permission_read_everyone'])
            $datavalues['permission_read'] = 'everyone';
        $datavalues['permission_read'] = serialize($datavalues['permission_read']);
        $datavalues['permission_write'] = serialize($datavalues['permission_write']);
        $datavalues['test_email_address'] = trim($datavalues['test_email_address']);
        $datavalues['options'] = serialize($datavalues['options']);
        $datavalues['image_is_embedded'] = (int) $datavalues['image_is_embedded'];
        if ($datavalues['image_is_embedded'])
        {
            $datavalues['body_html'] = $this->convertExternalImagestoEmbedded($datavalues['body_html_original']);
        } else
        {
            $datavalues['body_html'] = $this->convertExternalImagesToAbsolute($datavalues['body_html_original']);
        }
        if ($datavalues['send_newsletter_issue_now'])
        {
            $datavalues['status'] = 1;
            if (($datavalues['date_send_until'] == NULL) ||  ($datavalues['date_send_until'] == NULL_DATE))
                $datavalues['max_subscriber_id'] = $this->app->getLastSubscriberID();
            //$result = $this->app->emailNewsletterIssueToSubscriberSegment ($retval['id'], $datavalues['subscribers_segment_id']);
        } 
        return $datavalues;
    }

    /**
     * Before save category
     * @param array $datavalues
     */
    public function onModifyDataBeforeSaveCategory($datavalues) {
        
        global $CurrentUser;

        $external_images_save_path = '/media/images/clippings/';        
        $datavalues = parent::onModifyDataBeforeSaveCategory($datavalues);
        $current_admin_id = $CurrentUser->getCurrentUserID();

        $field_admin_id = ($datavalues[$this->app->getFieldCategoryID()]) ? 'created_by_id' : 'modified_by_id';

        if (!($datavalues[$field_admin_id] > 0 && $datavalues[$field_admin_id] != $current_admin_id))
            $datavalues[$field_admin_id] = $current_admin_id;
        $datavalues['virtual_filename'] = convert_into_sef_friendly_title($datavalues['virtual_filename']);
        if (empty($datavalues['meta_description']))
            $datavalues['meta_description'] = $datavalues['title'];
        if (empty($datavalues['virtual_filename']) || ($datavalues['virtual_filename'] == 'index'))
            $datavalues['virtual_filename'] = 'cat' . $datavalues['cid'];
        
        $datavalues['virtual_filename'] = $this->app->preventDuplicateValueInCategory('virtual_filename', $datavalues['virtual_filename'], $datavalues[$this->field_category_id]);
        
        if ($datavalues['permission_read_everyone'])
            $datavalues['permission_read'] = 'everyone';
        $datavalues['permission_read'] = serialize($datavalues['permission_read']);
        $datavalues['permission_write'] = serialize($datavalues['permission_write']);
        $datavalues['options'] = serialize($datavalues['options']);
        $datavalues['summary'] = convert_pasted_png_images_from_html_text($datavalues['summary'],  CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
        $datavalues['description'] = convert_pasted_png_images_from_html_text($datavalues['description'], CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
        if ($this->app->getConfig('bool_convert_external_images')) {
            $datavalues['summary'] = move_static_external_images_from_html_text($datavalues['summary'], CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
            $datavalues['description'] = move_static_external_images_from_html_text($datavalues['description'], CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
        }
        return $datavalues;        
    } 
    /**
     * You can customize the response schema field here
     * @param array $response_schema
     * @return array
     */
    public function modifyItemResponseSchemaFields(array $response_schema)
    {
        $response_schema = parent::modifyItemResponseSchemaFields($response_schema);
        // adda preview link
        $response_schema[] = array ('key' => 'preview_link', 'parser' =>'string');
        return $response_schema;
    }
    
    
    protected function getTotalRelatedRecordForNewsletterByID($newsletter_issue_id, $table)
    {
        global $SystemDB;

        $newsletter_issue_id = (int) $newsletter_issue_id;
        $sql = "SELECT count(id) AS total_count FROM `{$table}` WHERE newsletter_issue_id = {$newsletter_issue_id}";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result['total_count'];
    }
    
    public function ajaxGetRelatedRecordsForNewsletterIssueID($table, $newsletter_issue_id,  $start, $end, $sortby, $sortdirection) 
    {
        global $SystemDB;
         $newsletter_issue_id = (int) $newsletter_issue_id;
        $sortby = alpha_numeric_with_dash_underscore($sortby);
        $items_per_page = min(HARDCODE_MAX_ROWLIMIT, $end - $start);

        $total_item_count = $this->getTotalRelatedRecordForNewsletterByID($newsletter_issue_id, $table);

        $sql = $SystemDB->generateSelectSQLStatement($table, '*', "newsletter_issue_id = {$newsletter_issue_id}", $start, $end, $sortby, $sortdirection, true, SCHLIX_SQL_ENFORCE_ROW_LIMIT);

        $result = $SystemDB->getQueryResultArray($sql);
        
        return ajax_datasource_reply(200, $result, $start, $end, $items_per_page, $total_item_count, $sortby, $sortdirection);
    }

    /**
     * @global \App\Users $CurrentUser
     * @global \App\EmailQueue $SystemMail
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    public function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval) 
    {
        global $SystemMail, $CurrentUser;
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        if ($retval['status'] == SAVE_OK)
        {
            if ($original_datavalues['send_test_mail'])
            {
                $result = $this->app->emailTestNewsletterIssueToSubscriber ($retval['id']);
                if ($result['errors'])
                {
                    $str_error_list = implode(', ', $result['errors']);                    
                    $this->app->getEventLogger()->recordActivityForNewsletterIssueID($retval['id'], $str_error_list,'issue','error');                    
                }
            } 
        }
    }
    /*
    public function onRevertToDraft()
    {
        check_csrf_halt_on_error(true);
        $id = fpost_int('id');
        $item = $this->app->getItemByID($id);
        if ($item)
        {
            if ($item['status'] == NEWSLETTER_ISSUE_WAIT_TOBE_QUEUED)
            {
                $datavalues = ['status' => NEWSLETTER_ISSUE_DRAFT];
                $this->app->getItemTable()->quickUpdate($datavalues, "id = {$item['id']}");
                $this->app->getEventLogger()->recordAdminActivity("Reverted newsletter issue #{$item['id']}] to draft", 'send', 'info');
                return ajax_reply(200,'OK');
            } else
            return ajax_reply(400, 'You can only revert newsletter to draft if the status is still waiting to be queued');
        }
        return ajax_reply(400, 'Failed to revert item to draft because it cannot be found');
    }
    
    public function onPauseNewsletterIssue()
    {
        check_csrf_halt_on_error(true);
        $id = fpost_int('id');
        $item = $this->app->getItemByID($id);
        if ($item)
        {
            if ($item['status'] == NEWSLETTER_ISSUE_SENDING)
            {
                $datavalues = ['status' => NEWSLETTER_ISSUE_PAUSED];
                $this->app->getItemTable()->quickUpdate($datavalues, "id = {$item['id']}");
                $this->app->getEventLogger()->recordAdminActivity("Paused delivery of newsletter issue #{$item['id']}] to draft", 'send', 'info');
                
                return ajax_reply(200,'OK');
            } else
            return ajax_reply(400, 'You can only pause a newsletter if the status is still sending');
        }
        return ajax_reply(400, 'Failed to pause item because it cannot be found');
    }
    */

    protected function onHandleSendControl($command_name, $set_status_to, $current_status_1, $current_status_2 = NULL )
    {
        check_csrf_halt_on_error(true);
        $id = fpost_int('id');
        $item = $this->app->getItemByID($id);
        if ($item)
        {
            $b = true;
            if ($current_status_2 !== NULL)
                $b = $item['status'] == $current_status_2;
                
            if (($item['status'] == $current_status_1) || $b)
            {
                $datavalues = ['status' => $set_status_to];
                $this->app->getItemTable()->quickUpdate($datavalues, "id = {$item['id']}");
                $this->app->getEventLogger()->recordActivityForNewsletterIssueID($item['id'],"Sent command [{$command_name}] to control the delivery of newsletter issue #{$item['id']}]", 'send', 'info');
                
                return ajax_reply(200,'OK');
            } else
            {
                $cs = $current_status_2 ? $current_status_1.' or '.$current_status_2 : $current_status_1;
                return ajax_reply(400, "You can only send command [{$command_name}] if the status is {$cs}");
            }
        }
        return ajax_reply(400, 'Failed because the item cannot be found');
    }
    /**
     * Cancel remaining queue in newsletter issue
     * @param int $id
     */
    public function completelyCancelNewsletterIssue($id)
    {
        $item = $this->app->getItemByID($id);
        if ($item)
        {
            if ($item['status'] == NEWSLETTER_ISSUE_CANCELLED_UNPUBLISHED)
            {
                $total_unsent = \App\Newsletters_Queues::getTotalByQueueMarkerAndStatus($item['guid'], 0);
                $total_sent = \App\Newsletters_Queues::getTotalByQueueMarkerAndStatus($item['guid'], 1);
                $total_failed = \App\Newsletters_Queues::getTotalByQueueMarkerAndStatus($item['guid'], -1);
                if ($total_unsent > 0)
                {
                    \App\Newsletters_Queues::cancelDeliveryByQueueMarker($item['guid']);
                    $this->app->getEventLogger()->recordActivityForNewsletterIssueID($item['id'], "Cancelled newsletter issue ID #{$item['id']}. Total unsent = {$total_unsent}. Total sent = {$total_sent}. Total failed = {$total_failed}", 'send', 'info');
                }
            }
        }
    }
    public function editItem($id) {
        global $WYSIWYGEditor;
        
        $wid = $this->app->getConfig('int_wysiwyg_profile_id');
        if ($wid)
            $WYSIWYGEditor->overRideFullPrivilegeWysiwygEditor($wid);
        return parent::editItem($id);
    }

    
    public function viewImportFromPLPage()
    {        
        $this->setPageTitle('Import from PHPList');
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.importpl.admin', $local_variables);
        
        return true;
    } 
    
    public function ajaxPLAction($real_import)
    {
        require('newsletters.phplistimporter.class.php');
        $table_prefix = fpost_string('tableprefix');
        $db_host = fpost_string('host');
        $db_database = fpost_string('database');
        $db_username = fpost_string('username');
        $db_password = fpost_string('password');
        
        $this->app->setConfig('str_phplist_host', $db_host);
        $this->app->setConfig('str_phplist_db', $db_database);
        $this->app->setConfig('str_phplist_username', $db_username);
        $this->app->setConfig('str_phplist_table_prefix', $table_prefix);
        
        $phplist_importer = new \App\Newsletters_PHPListImporter($table_prefix, $db_host, $db_database, $db_username, $db_password);
        $result = $real_import ? $phplist_importer->performImport() : $phplist_importer->testDBConnection();
        return ajax_reply(200, $result);
    }
    //_________________________________________________________________________//
    public function Run() {
        switch (fget_alphanumeric('action')) {
            case 'geteventlogsresponseschema': 
                return ajax_echo($this->app->createSubClassInstance('EventLog_Admin')->ajaxGetAllDataResponseSchema());
                break;
            
            case 'getuserlogsresponseschema': 
                return ajax_echo($this->app->createSubClassInstance('SubscriberLog_Admin')->ajaxGetAllDataResponseSchema());
                
                break;
            case 'importplexecute':
                return ajax_echo($this->ajaxPLAction(true));
            case 'importpltestdb':
                return ajax_echo($this->ajaxPLAction(false));
            case 'geteventlogs':
                return ajax_echo($this->ajaxGetRelatedRecordsForNewsletterIssueID('scxnl_log_events',fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
            case 'getuserlogs':
                return ajax_echo($this->ajaxGetRelatedRecordsForNewsletterIssueID('scxnl_log_subscribers',fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
                break;
            case 'getitemsresponseschema' :
                return $this->getItemsResponseSchema();
                break; 
            case 'sendreverttodraft' : return ajax_echo ($this->onHandleSendControl('revert to draft', NEWSLETTER_ISSUE_DRAFT, NEWSLETTER_ISSUE_WAIT_TOBE_QUEUED));
            case 'sendpause' : return ajax_echo ($this->onHandleSendControl('pause', NEWSLETTER_ISSUE_PAUSED, NEWSLETTER_ISSUE_SENDING));
            case 'sendresume' : return ajax_echo ($this->onHandleSendControl('resume', NEWSLETTER_ISSUE_SENDING, NEWSLETTER_ISSUE_PAUSED));
            case 'sendcancel' : 
                $result = ajax_echo ($this->onHandleSendControl('cancel', NEWSLETTER_ISSUE_CANCELLED_UNPUBLISHED, NEWSLETTER_ISSUE_PAUSED, NEWSLETTER_ISSUE_SENDING));
                $this->completelyCancelNewsletterIssue(fpost_int('id'));
                return $result;
            case 'importpl':
                return $this->viewImportFromPLPage();
                break;
                
            default:return parent::Run();
        }
    }
    public function viewMainPage() {
        // insert it when admin browse the main admin page
        $system_cron = new cronScheduler();
        $system_cron->createDefaultSchedulerForMyApplicationStaticMethod('Run Newsletters Tasks', '\\App\\Newsletters::processRunNewsletterTasks', "*/5 * * * *");
        
        return parent::viewMainPage();
    }
    //_________________________________________________________________________//
    public function Uninstall() {
        return false;
    }

}

?>