/**
 * Newsletters - Javascript admin controller class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS 
 *
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
SCHLIX.CMS.NewslettersAdminController = class extends SCHLIX.CMS.BaseController  {  
    /**
     * Constructor
     */
    constructor ()
    {
        super("newsletters");
    };
    

    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_ViewLog (elCell, oRecord, oColumn, oData) {

        var odata_id = oRecord.getData("id");
        //  alert(this.parentControl.getTableFieldNameByRole('title'));
        var itemTitle = oData;// oRecord.getData("title");
        var itemLink = '';
        if (itemTitle && itemTitle.length > 60)
            itemTitle = itemTitle.substr(0, 60) + '...';
        if (!itemTitle) itemTitle = 'N/A';
        var app_name = this.parentControl.app_name;

        var the_id = oRecord.getData("id");
        var theValue = 'i' + the_id;
        if (itemTitle == '')
            itemTitle = '(Untitled)';
        //itemLink = _schlix_app_controller + oColumn.app_name + '&action=edititem&id=' + oRecord.getData("id");
        var the_icon;
        switch (oRecord.getData('type'))
        {
            case 'error' : 
                the_icon = 'fa-times-circle';
                break;
            case 'warn' : 
                the_icon = 'fa-exclamation-triangle';
                break;
                    
            case 'info' :
            default: the_icon = 'fa-info-circle';
        }
        var icon = "<i class =\"fa " + the_icon + " fa-2x\"></i>";
        // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            
        elCell.innerHTML = '<a class="open-viewlog" id="' + app_name + "-lnk-" + theValue + '" data-log-id="' + theValue + '" href="javascript:void(0)" data-log-message="' + oData + '">' + " " + icon + " " + itemTitle + '</a>';
    };
 

    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_ViewStatus (elCell, oRecord, oColumn, oData) {

        var status = oData ? oData : 0;
        var status_text = '';
        var status_icon = '';
        var status_icon_color;
        switch (status)
        {
            case -2: 
                status_text = 'Terminated and unpublished';
                status_icon = 'fa fa-times-circle';
                status_icon_color = 'red';
                break;

            case -1: 
                status_text = 'Hidden';
                status_icon = 'fa fa-file';
                status_icon_color = 'orange';

                break;
            case 0: 
                if (oRecord.getData('cid') > 0)                
                {
                status_text = 'Inactive';
                status_icon = 'fa fa-times';
                status_icon_color = 'red';                    
                } else
                {
                status_text = 'Draft';
                status_icon = 'fa fa-file-text';
                status_icon_color = 'gray';
                }
                break;
            case 1: 
                if (oRecord.getData('cid') > 0)                
                {
                status_text = 'Active';
                status_icon = 'fa fa-check';
                status_icon_color = 'green';                    
                } else
                {
                status_text = 'Published and waiting to be queued';
                status_icon = 'fa fa-calendar-check-o';
                status_icon_color = 'green';
                }
                break;
            case 2: 
                status_text = 'Sending';
                status_icon = 'fa fa-exchange';
                status_icon_color = 'green';
                break;    
            case 3: 
                status_text = 'Paused';
                status_icon = 'fa fa-exclamation-triangle';
                status_icon_color = 'orange';
                break;
            case 4: 
                status_text = 'Sent and finished';
                status_icon = 'fa fa-check-circle';
                status_icon_color = 'green';
                break;
            case  5: 
                status_text = 'Cancelled, partially sent';
                status_icon = 'fa fa-exclamation-triangle';
                status_icon_color = 'orange';
                break;
        }
        var icon = '<i class="fa ' + status_icon + ' " style="color:' + status_icon_color + '"></i> &nbsp;';
        // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            
        elCell.innerHTML = icon + status_text;
    };
     
    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_DefaultTitleColumn  (elCell, oRecord, oColumn, oData) {

        var field_title = SCHLIX.Util.isUndefined(this.parentControl.element_data["field-item_title"]) ? "title" : this.parentControl.element_data["field-item_title"];
        var odata_id = oRecord.getData("id");
        var odata_cid = oRecord.getData("cid");
        var itemTitle = oRecord.getData(field_title); // oRecord.getData("title");
        var itemLink = site_httpbase;
        if (itemTitle == null)
        {
            SCHLIX.log("formatDataTableCell_DefaultTitleColumn cannot process a null title", 'warn');
            return;
        }
        if (itemTitle.length > 40)
            itemTitle = itemTitle.substr(0, 40) + '...';
        var app_name = this.parentControl.app_name;
        if (odata_cid > 0)
        {
            SCHLIX.CMS.__default_formatFolderInDataTable(app_name, elCell, oRecord, itemTitle, site_httpbase + this.parentControl.schlix_application_url + 'action=editcategory&id=' + odata_cid);
        } else
        {
            // console.log(oRecord);
            var the_id = oRecord.getData("id");
            var theValue = 'i' + the_id;
            if (itemTitle === '')
                itemTitle = '(Untitled)';
            var icon = "<i class =\"fa fa-file-text-o fa-2x\"></i>";

            itemLink += this.parentControl.schlix_application_url + 'action=edititem&id=' + the_id;
            var item_class = (oRecord.getData('id')==1 || oRecord.getData('virtual_filename') == 'home') ? 'html_home' : 'dragdrop';
            var preview_link = oRecord.getData("preview_link");
            var display_preview_link = '';
            if (SCHLIX.Util.isString(preview_link))
            {
                var preview_icon = "<i class=\"fa fa-eye\"></i> ";
                display_preview_link = '<div style="float:right;margin-left:4em">' +  '<a target="_blank" href="' + preview_link + '">' + preview_icon +  '</a>' + '</div>';
            }

            elCell.innerHTML = '<a class="' + item_class + '" id="' + app_name + "-lnk-" + theValue + '" data-dragdrop-id="' + theValue + '" title="Click here to edit this item" href="' + itemLink + '">' + " " + icon + " " + itemTitle + '</a>' + display_preview_link;
        }
    };
    
    
    static formatDataTableCell_CheckBox (elCell, oRecord, oColumn, oData) {
                var theID = '';
                var theValue = '';

                var app_name = this.parentControl.app_name;
                if (oRecord.getData("cid") > 0)
                {
                    theID = app_name + '-select-cid' + oRecord.getData("cid");
                    theValue = 'c' + oRecord.getData("cid");
                }
                else
                {
                    theID = app_name + '-select-id' + oRecord.getData("id");
                    theValue = 'i' + oRecord.getData("id");
                }

                elCell.innerHTML = '<input type="checkbox" class="' + app_name + '-chkselections" name="' + app_name + '-chkselections[]" id="' + theID + '"  value="' + theValue + '" />';
    }; 
    onToolbarButtonClick(event)
    {
        var el = SCHLIX.Dom.get(event.target.id);
        // Stop the event's default behavior
        
        var command = el.getAttribute('data-schlix-command'); //  $(target).attr("data-category-id");
        this.runCommand(command, event);
        
        event.preventDefault();
        event.stopPropagation();
        return true;
    }
    onDOMReady ()
    {
        SCHLIX.Event.on("{.schlix-send-button}", 'click', this.onToolbarButtonClick, this, true);
        SCHLIX.Event.on("btn_test_database",'click',this.onTestPLImportClick,this,true);
        SCHLIX.Event.on("btn_execute_importpl",'click',this.onPerformImportPLClick,this,true);
    }; // end func
    
    saveItemWithAjax()
    {
            var main_component = SCHLIX.CMS.getObject('schlix-newsletters-admin');
            var form_el = main_component.__getFirstFoundForm();
            if (form_el)
            {
                if (form_el.checkValidity())
                {
                    try
                    {
                        main_component.onAjaxSaveItem(form_el.getAttribute('id'));
                    } catch (exc)
                    {
                        SCHLIX.Alert.error(exc);
                        return false;
                    }
                } else
                {
                    SCHLIX.Alert.error('Some form field values are invalid');
                    return true;
                }
            }
    }; 
    ///////////////////////////////////////////////////////////
    onRevertToDraftClick (e)
    {
        var el = SCHLIX.Dom.get('id');
        var id = el.value;

        if (id != "new")
        {
            var postData = "_csrftoken=" + _csrftoken + '&id=' + id ;
            //this.ajaxRequestPOST(this.reportPOSTOperation, this.getUpdateFieldURL(), postData, 'cronscheduler');   


            var request = SCHLIX.Ajax.POST( site_httpbase + "/admin/app/" + this.app_name + "?action=reverttodraft",
                    {success: this.onRevertToDraftResponse, failure: this.handleFailure, scope: this}, postData);
        }

    }; // end func
    onRespondChangeStatus(o)
    {
        var response = ajax_parse(o.responseText);
        if (response === 'OK')
        {
            SCHLIX.Alert.info('Status has been changed');
        }
        else
            alert(response);
        window.location.reload();
        
    }; // end func
    
    ///////////////////////////////////////////////////////////
    onPauseClick (e)
    {

    }; // end func
    onPauseResponse(o)
    {
        var response = ajax_parse(o.responseText);
        if (response === 'OK')
        {
            SCHLIX.Alert.info('Delivery has been paused');
        }
        else
            alert(response);
        window.location.reload();
        
    }; // end func
    
    onSendControlButtonClick (action, handler, confirmation_text)
    {
        if (confirm(confirmation_text))
        {
            var el = SCHLIX.Dom.get('id');
            var id = el.value;

            if (id != "new")
            {
                var postData = "_csrftoken=" + _csrftoken + '&id=' + id ;
                //this.ajaxRequestPOST(this.reportPOSTOperation, this.getUpdateFieldURL(), postData, 'cronscheduler');   


                var request = SCHLIX.Ajax.POST( site_httpbase + "/admin/app/" + this.app_name + "?action=" + action,
                        {success: handler, failure: this.handleFailure, scope: this}, postData);
            }
        }
    }
    
    onPerformImportPLClick(event)
    {
        if (confirm('Are you sure you want to reset the database and begin the import?'))
        {
            var db_username = SCHLIX.Dom.get('db_username').value;
            var db_password = SCHLIX.Dom.get('db_password').value;
            var db_host = SCHLIX.Dom.get('db_host').value;
            var db_database = SCHLIX.Dom.get('db_database').value;
            var db_tableprefix = SCHLIX.Dom.get('db_table_prefix').value;
            var postData = "_csrftoken=" + _csrftoken + '&username=' +db_username + '&password=' +db_password+ '&host=' + db_host+ '&database=' + db_database + '&tableprefix=' + db_tableprefix;
            SCHLIX.Dom.get('btn_execute_importpl').innerHTML = '<i class="fa fa-spinner fa-spin fa-fw"></i>Please wait while importing. This may take hours. DO NOT CLOSE THIS PAGE.';
            SCHLIX.Dom.get('btn_execute_importpl').disabled = true;
            var timeout_seconds = 2 * 24 * 60 * 1000; // 48 hours
            // for 15,000 records it took about 2 hours
            var request = SCHLIX.Ajax.POST( site_httpbase + "/admin/app/" + this.app_name + "?action=importplexecute&ajax=1",
                    {success: this.onGetImportResult, failure: this.onFailedDatabaseTest, scope: this, timeout: timeout_seconds}, postData
                    );
        }
        event.preventDefault();
        event.stopPropagation();
        
        return false;
    }; // end func
    onGetImportResult(o)
    {
        var response = ajax_parse(o.responseText);
        if (response.status == 'Executed')
        {
            SCHLIX.Dom.get('db_test_errors').innerHTML = '';
            SCHLIX.Dom.get('db_test_result').innerHTML = '<div class="alert alert-success">Completed</div>';
            SCHLIX.Dom.get('btn_test_database').style.display = 'none';
            SCHLIX.Dom.get('btn_execute_importpl').style.display = 'none';
            
        } else
        {
            SCHLIX.Alert.error('There seems to be an error during import');
        }
    }
    
    onTestPLImportClick(event)
    {
        var db_username = SCHLIX.Dom.get('db_username').value;
        var db_password = SCHLIX.Dom.get('db_password').value;
        var db_host = SCHLIX.Dom.get('db_host').value;
        var db_database = SCHLIX.Dom.get('db_database').value;
        var db_tableprefix = SCHLIX.Dom.get('db_table_prefix').value;
        var postData = "_csrftoken=" + _csrftoken + '&username=' +db_username + '&password=' +db_password+ '&host=' + db_host+ '&database=' + db_database + '&tableprefix=' + db_tableprefix;

        var request = SCHLIX.Ajax.POST( site_httpbase + "/admin/app/" + this.app_name + "?action=importpltestdb&ajax=1",
                {success: this.onGetTestDBResult, failure: this.onFailedDatabaseTest, scope: this}, postData
                );
    }; // end func
    
    onGetTestDBResult(o)
    {
        var response = ajax_parse(o.responseText);
        var errors = '';
        var result = '';
        if (response.status == false)
        {
            if (response.errors)
            {
                errors = '<ul>';
                for (var i = 0;i < response.errors.length;i++)
                {
                    errors += '<li>' + response.errors[i] + '</li>';
                }
                errors += '</ul>';
            }
        }
        
        if (response.result)
        {
            result = '<ul>';
            for (var i = 0;i < response.result.length;i++)
            {
                result += '<li>' + response.result[i] + '</li>';
            }
            result += '</ul>';
        }
        SCHLIX.Dom.get('db_test_errors').innerHTML = errors.length > 0 ? '<h3>Errors</h3><div class="alert alert-danger">' + errors + '</div>'  : '';
        SCHLIX.Dom.get('db_test_result').innerHTML = result.length > 0 ? '<h3>Import Tasks</h3><div class="alert alert-info">' + result + '</div>'  : '';
        if (errors.length == 0 && result.length > 0)
        {
        SCHLIX.Dom.get('db_username').readOnly=true;
        SCHLIX.Dom.get('db_password').readOnly=true;
        SCHLIX.Dom.get('db_host').readOnly=true;
        SCHLIX.Dom.get('db_database').readOnly=true;
        SCHLIX.Dom.get('db_table_prefix').readOnly=true;
            
            SCHLIX.Dom.get('btn_test_database').style.display = 'none';
            SCHLIX.Dom.get('btn_execute_importpl').style.display = 'inline-block';
        }
    };
    
    onFailedDatabaseTest(o)
    {
         var db_test_result = SCHLIX.Dom.get('db_tested');    
         db_test_result = '0';
    }; // end func
    
    /**
     * Run command
     * @param string command
     * @param Event evt
     * @returns bool
     */
    runCommand (command, evt)
    {
        //if (evt.type == 'contextmenu')
        var open_in_new_window = is_rightclick_event(evt);
        switch (command)
        {
            case 'new-item':
                this.redirectToCMSCommand("newitem",open_in_new_window);
                return true;
                break;
            case 'new-category':
                this.redirectToCMSCommand("newcategory",open_in_new_window);
                return true;
                break;
            case 'new-issue-from-template':
                var nl = evt.target.getAttribute('data-template-name');
                this.redirectToCMSCommand("newitem&fromtemplate=" + nl);
                break;
                
            case 'edit-current-category':
                var target = evt.target;                
                window.location = target.href;
                return true;
                break;                                                
            case 'config':
                this.redirectToCMSCommand("editconfig",open_in_new_window);
                return true;
                break;
            case 'send-reverttodraft':
                this.onSendControlButtonClick('sendreverttodraft', this.onRespondChangeStatus,'Are you sure you want to set this issue back to draft?');
                return true;
                break;
            case 'send-pause':
                this.onSendControlButtonClick('sendpause', this.onRespondChangeStatus,'Are you sure you want to pause the delivery of this issue?');
                return true;
                break;
            case 'send-resume':
                this.onSendControlButtonClick('sendresume', this.onRespondChangeStatus,'Are you sure you want to resume the delivery of this issue?');
                return true;
                break;
            case 'send-cancel':
                this.onSendControlButtonClick('sendcancel', this.onRespondChangeStatus,'Are you sure you want to completely cancel the delivery of this issue?');
                return true;
                break;                
            case 'save-item-ok':
                var send_newsletter_issue_now = SCHLIX.Dom.get('send_newsletter_issue_now');
                if (send_newsletter_issue_now.value == 1)
                {
                    window.location.reload();
                    return true;                    
                } 
                else 
                {
                    var eventlog = SCHLIX.CMS.getObject('schlix-nleventlogs-admin');
                    //eventlog.runCommand('refresh');
                    eventlog.refreshControls();
                    return super.runCommand(command,evt);
                }
                break;
            case "send-testmail":
                var send_newsletter_issue_now = SCHLIX.Dom.get('send_newsletter_issue_now');                
                send_newsletter_issue_now.value = 0; // make sure it's not this one selected
                var test_email_elem = SCHLIX.Dom.get('test_email_address');
                if (test_email_elem.value)
                {
                    var send_test_mail_elem = SCHLIX.Dom.get('send_test_mail');
                    send_test_mail_elem.value = 1;
                    this.saveItemWithAjax();
                    send_test_mail_elem.value = 0;
                    return true;
                } else
                {
                    SCHLIX.Alert.error('Invalid or empty test email address');
                }
                break;
            case "send-issue-now":
                var send_test_mail_elem = SCHLIX.Dom.get('send_test_mail');
                var send_newsletter_issue_now = SCHLIX.Dom.get('send_newsletter_issue_now');
                send_test_mail_elem.value = 0;
                send_newsletter_issue_now.value = 1;
                if (confirm('Are you sure you want to send this issue now?'))
                {
                    this.saveItemWithAjax();
                } else
                {
                    SCHLIX.Alert.error('Send process has been cancelled');
                }
                break;
            default:
                return super.runCommand(command,evt);
                break;
        }
    }
};