<?php
namespace App;
/**
 * Newsletters - Main Class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
class Newsletters_Blacklist extends \SCHLIX\cmsApplication_List {

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct("Newsletters Blacklist", 'scxnl_blacklist');
        // you can modify these to suit your needs
        $this->disable_frontend_runtime = false;
        $this->has_versioning = false;        
    }
            
    /**
     * 
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $email_address
     */
    public static function isEmailAddressBlacklisted($email_address)
    {
        global $SystemDB;
       
        $real_email_address = \App\Newsletters::getRealEmailAddress(trim($email_address));
        $row = $SystemDB->getQueryResultSingleRow("SELECT * FROM scxnl_blacklist WHERE real_email_address = :email_address",['email_address' => $real_email_address]);
        return $row != NULL;
    }
    /**
     * Ban an email address
     * @param string $email_address
     * @param string $reason
     * @return boolean
     */
    public function blacklistEmailAddress($email_address, $reason='')
    {
        $eventlog = $this->createSubClassInstance('EventLog');
        $newsletter_app = new \App\Newsletters();
        $real_email_address = \App\Newsletters::getRealEmailAddress($email_address);
        if (!static::isEmailAddressBlacklisted($real_email_address))
        {
            $datavalues = array();
            $datavalues['real_email_address'] = $real_email_address;
            $datavalues['date_created'] = get_current_datetime();
            $datavalues['notes'] = $reason;
            $this->table_items->quickInsert($datavalues);
            $eventlog->recordAdminActivity("Blacklisted {$email_address}. Reason: {$reason}", 'blacklist', 'info');
            $existing_subscriber = $newsletter_app->getSubscriberByRealEmailAddress($real_email_address);
            if ($existing_subscriber)
            {
                $newsletter_app->completeUnsubscribe($existing_subscriber['id'], $reason, NULL, true);
            }
            return true;
        }
        return false;	
    }    
    /**
     * Validates save item. If there's an error, it will return an array
     * with one or more error string, otherwise it will return a boolean true
     * @global \App\Users $CurrentUser
     * @param array $datavalues
     * @return bool|array String array
     */
    public function getValidationErrorListBeforeSaveItem($datavalues)
    {
        $parent_error_list = parent::getValidationErrorListBeforeSaveItem($datavalues);
        $error_list = array();
        // You can put custom item save validation here
        // e.g. 
        // if (str_contains($datavalues['title'],'a'))
        // $error_list[] = ___('Title may not contain letter a');
        return array_merge($parent_error_list, $error_list);
    }

    /**
     * Before save item
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues) {
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        // You can customize the data values before it's saved here
        // e.g.
        // $datavalues['title'] = real_strip_tags($datavalues['title']);
        //
        return $datavalues;
    }
    
    /**
     * Do something after save item
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    protected function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval)
    {
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
    }



    /**
     * Run Command
     * @param array $command
     * @return boolean
     */
    public function Run($command) {
        //$this->insertTest();
        switch ($command['action']) {
            case 'main': $this->viewMainPage($command['pg']);
                return true;
                break;
            default: return parent::Run($command);
        }
    }



}

?>