<?php
namespace App;
// June 15 - TODO: clean up messy code
/**
 * Newsletters - Main Class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */

define ('NEWSLETTER_SUBSCRIBER_INVALID', -4);
define ('NEWSLETTER_SUBSCRIBER_BLACKLISTED', -3);
define ('NEWSLETTER_SUBSCRIBER_NEW', -2);
define ('NEWSLETTER_SUBSCRIBER_UNSUBSCRIBED', 0);
define ('NEWSLETTER_SUBSCRIBER_ACTIVE', 1);

define ('NEWSLETTER_ACTIVATION_NEW', -2);
define ('NEWSLETTER_ACTIVATION_OK', 1);


define ('NEWSLETTER_EMAILCHANGE_DENIED', -4);
define ('NEWSLETTER_EMAILCHANGE_REQUEST_NEW', -2);
define ('NEWSLETTER_EMAILCHANGE_INVALID', 0);
define ('NEWSLETTER_EMAILCHANGE_OK', 1);

define ('NEWSLETTER_ISSUE_CANCELLED_UNPUBLISHED', -2);
define ('NEWSLETTER_ISSUE_HIDDEN', -1);
define ('NEWSLETTER_ISSUE_DRAFT', 0);
define ('NEWSLETTER_ISSUE_WAIT_TOBE_QUEUED', 1);
define ('NEWSLETTER_ISSUE_SENDING', 2);
define ('NEWSLETTER_ISSUE_SENT_AND_FINISHED', 4);
define ('NEWSLETTER_ISSUE_PAUSED', 3);
define ('NEWSLETTER_ISSUE_CANCELLED_PARTIALLY_SENT', 5);

class Newsletters extends \SCHLIX\cmsApplication_CategorizedList {
    /**
     * Logger
     * @var \App\Newsletters_SubscriberLog
     */
    protected $subscriberLog;
    
    /**
     * Logger
     * @var \App\Newsletters_EventLog
     */
    protected $eventLog;
    /**
     * Table: queue
     * @var \SCHLIX\cmsSQLTable 
     */
    protected $table_queues;    
    /**
     * Table: unsubscribe request
     * @var \SCHLIX\cmsSQLTable  
     */
    protected $table_unsubscribe_request;
    /**
     *
     * @var \SCHLIX\cmsSQLTable 
     */
    protected $table_subscribe_request;
    /**
     *
     * @var \SCHLIX\cmsSQLTable 
     */
    protected $table_subscribe_pages;
    
    /**
     * Subscribers
     * @var \SCHLIX\cmsSQLTable 
     */
    protected $table_subscribers;
    
    /**
     * Subscriptions
     * @var \SCHLIX\cmsSQLTable 
     */    
    protected $table_subscriptions;
    /**
     * Email Change Request
     * @var \SCHLIX\cmsSQLTable 
     */        
    protected $table_email_change_request;
    /**
     * Activation expiry in hours
     * @var $activation_expiry_in_hours 
     */
    protected $activation_expiry_in_hours;
    
    /**
     * Emailer
     * @var \App\Newsletters_Queues
     */
    protected $emailer;
    
    /**
     * Default name
     * @var string
     */
    protected $sender_default_name;

    /**
     * Default email
     * @var string
     */
    protected $sender_default_email;
    /**
     * Default reply-to
     * @var string
     */
    protected $sender_default_replyto;
    
    
    /**
     * Send request emails immediately, excluding newsletters email
     * @var bool 
     */
    protected $send_request_emails_immediately = true;

        
    
    public function __construct() {
        parent::__construct('Newsletters', 'scxnl_issues', 'scxnl_categories');
        
        $this->table_queues = new \SCHLIX\cmsSQLTable('scxnl_queues');
        $this->table_clicks = new \SCHLIX\cmsSQLTable('scxnl_clicks');
        $this->table_blacklist = new \SCHLIX\cmsSQLTable('scxnl_clicks');
        $this->table_subscribers = new \SCHLIX\cmsSQLTable('scxnl_subscribers');
        $this->table_subscribe_request = new \SCHLIX\cmsSQLTable('scxnl_rq_subscribe');
        $this->table_subscribe_pages = new \SCHLIX\cmsSQLTable('scxnl_subscribe_pages');
        $this->table_subscriptions = new \SCHLIX\cmsSQLTable('scxnl_subscriptions');
        $this->table_unsubscribe_request = new \SCHLIX\cmsSQLTable('scxnl_rq_unsubscribe');
        $this->table_email_change_request = new \SCHLIX\cmsSQLTable('scxnl_rq_email_change');
        $this->eventLog = $this->createSubClassInstance('EventLog');
        $this->subscriberLog = $this->createSubClassInstance('SubscriberLog');
        $this->schema_org_type_item = '';
        $this->has_versioning = true;
        
        $this->activation_expiry_in_hours = (int) $this->getConfig('int_activation_expiry_hours');
        if ($this->activation_expiry_in_hours == 0)
            $this->activation_expiry_in_hours = 24;
        $this->sender_default_name = $this->getConfig('str_default_name');
        if (empty($this->sender_default_name))
            $this->sender_default_name = SCHLIX_MAIL_DEFAULT_SENDER;
        $this->sender_default_email = $this->getConfig('str_default_field_from');
        if (empty($this->sender_default_email))
            $this->sender_default_email = SCHLIX_MAIL_DEFAULT_EMAIL;
        $this->sender_default_replyto = $this->getConfig('str_default_field_replyto');
        if (empty($this->sender_default_replyto))
            $this->sender_default_replyto = SCHLIX_MAIL_DEFAULT_EMAIL;
        
        $this->emailer = $this->createSubClassInstance('Queues');
        
        \App\Newsletters_Queues::setSMTPServerInfo($this->getConfig('bool_use_smtp_server'), $this->getConfig('str_smtp_secure_with'), $this->getConfig('str_smtp_host'), $this->getConfig('int_smtp_port'), $this->getConfig('bool_smtp_use_auth'), $this->getConfig('str_smtp_username'), $this->getConfig('str_smtp_password'));
    }
    
    public static function processRunNewsletterTasks() {
        $newsletter_app = new \App\Newsletters();
        
        // 1. Process the ones waiting to be queued
        $newsletter_app->markNewslettersReadyTobeQueued();
        // 2. Process the ones that is currently sending
        $newsletter_app->continueSendingNewsletterIssues();
        // 3. Change status for newsletter issues that should be terminated by date or reached max subscriber ID
        $newsletter_app->markStaleNewsletterIssuesAsSent();
        // 4. Process queued email
        $newsletter_app->processQueuedEmails();
    }    
    
    public function processQueuedEmails()
    {     
        
        \App\Newsletters_Queues::processRunEmailQueue();
    }
    
    
    public function markNewslettersReadyTobeQueued()
    {
        global $SystemDB;
        
        $table_item = $this->getItemTableName();
        $current_date_time = get_current_datetime();
        $str_current_date_time = sanitize_string($current_date_time);
        $status_waiting_tobe_sent = NEWSLETTER_ISSUE_WAIT_TOBE_QUEUED;
        $status_sending = NEWSLETTER_ISSUE_SENDING;
        $newsletter_issues_tobe_queued = $SystemDB->getQueryResultArray("SELECT * FROM {$table_item} WHERE status = {$status_waiting_tobe_sent} AND (time_to_sec(timediff({$str_current_date_time},`date_available`))/3600) > 0");
        if ($newsletter_issues_tobe_queued)
        {
            $arr_nlid_for_queue = array_column($newsletter_issues_tobe_queued,'id');
            $str_nlid_for_queue = implode(',', $arr_nlid_for_queue);
            $sql = "UPDATE {$table_item} SET status = {$status_sending} WHERE id IN ($str_nlid_for_queue)";
            $SystemDB->query($sql);
        }
        
    }
    public function markStaleNewsletterIssuesAsSent()
    {
        global $SystemDB;
        
        $status_sending = NEWSLETTER_ISSUE_SENDING;
       
        $status_sent_finished = NEWSLETTER_ISSUE_SENT_AND_FINISHED;
        $current_date_time = get_current_datetime();
        $str_current_date_time = sanitize_string($current_date_time);
        
        $sql = "SELECT id FROM {$this->table_items} WHERE (status = {$status_sending}) AND  ".
                "((last_sent_to_subscriber_id >= max_subscriber_id AND max_subscriber_id > 0) OR  ".
                "(date_send_until IS NOT NULL AND  date_send_until <> '0000-00-00 00:00:00' AND (time_to_sec(timediff({$str_current_date_time},`date_send_until`))/3600) > 0 )".
                ")";
                
        $unfinished_newsletters =$SystemDB->getQueryResultArray($sql);
        if ($unfinished_newsletters)
        {
            $col_ids = array_column($unfinished_newsletters, 'id');
            $str_col_ids = implode(',', $col_ids);
            $SystemDB->query("UPDATE {$this->table_items} SET status = {$status_sent_finished} WHERE id IN ({$str_col_ids})");
            $this->eventLog->recordGenericEvent("Marked newsletter issue ID ({$col_ids}) as finished", 'send', 'warning');
        }
        
        // Double check, just in case
        $sub = $this->createSubClassInstance('Subscribers');
        
        $sql = "SELECT id, last_sent_to_subscriber_id FROM {$this->table_items} WHERE status = {$status_sending} AND (date_send_until IS  NULL OR  date_send_until = '0000-00-00 00:00:00')";
        $sending_newsletters =$SystemDB->getQueryResultArray($sql);
        foreach ($sending_newsletters as $newsletter)
        {
            if ($newsletter['last_sent_to_subscriber_id'] > 0)
            {
                $count_after_this_id = $sub->getTotalSubscriberCountWithIDGreaterThan($newsletter['last_sent_to_subscriber_id']);
                if ($count_after_this_id == 0)
                {
                    $SystemDB->query("UPDATE {$this->table_items} SET status = {$status_sent_finished} WHERE id = {$newsletter['id']}");
                    $this->eventLog->recordGenericEvent("Marked newsletter issue ID ({$col_ids}) as finished, terminated automatically by subscriber count", 'send', 'warning');
                }
            }
        }
        
    }
    /**
     * Continue sending newsletter issues
     * @global \SCHLIX\cmsDatabase $SystemDB
     */
    public function continueSendingNewsletterIssues()
    {
        global $SystemDB;
        $status_sending = NEWSLETTER_ISSUE_SENDING;
        $current_date_time = get_current_datetime();
        $str_current_date_time = sanitize_string($current_date_time);
        
        $sql = "SELECT *,(time_to_sec(timediff({$str_current_date_time},`date_send_until`))/3600) FROM {$this->table_items} WHERE status = {$status_sending} ".
                "AND ((date_send_until = NULL OR date_send_until = '0000-00-00 00:00:00') ".
                  "OR (date_send_until IS NOT NULL AND  date_send_until <> '0000-00-00 00:00:00' AND (time_to_sec(timediff({$str_current_date_time},`date_send_until`))/3600) < 0))".
                "AND ((time_to_sec(timediff({$str_current_date_time},`date_available`))/3600) > 0)  ";
        $newsletter_issues = $SystemDB->getQueryResultArray($sql);
        foreach ($newsletter_issues as $issue)
        {
            /*$send_in_one_batch = ($issue['date_send_until'] == NULL_DATE) || ($issue['date_send_until'] == NULL);
            if ($send_in_one_batch)
            {
                $this->emailNewsletterIssue($issue['id'], $send_in_one_batch);
            }*/
             $this->emailNewsletterIssue($issue['id']);
        }
        
    }
    
    public function importFromPHPList()
    {
        $sub_app = new Newsletters_Subscribers();
        $sub_app->importFromPHPList();
    }
    public function truncateActivationRequests()
    {
        global $SystemDB;
        
        $SystemDB->query("TRUNCATE TABLE scxnl_log_events");
        $SystemDB->query("TRUNCATE TABLE scxnl_log_subscribers");
        $SystemDB->query("TRUNCATE TABLE scxnl_queues");
        $SystemDB->query("TRUNCATE TABLE scxnl_subscriber_attributes");
        $SystemDB->query("TRUNCATE TABLE scxnl_subscribers");
        $SystemDB->query("TRUNCATE TABLE scxnl_subscriptions");
        $SystemDB->query("TRUNCATE TABLE scxnl_rq_subscribe");
        $SystemDB->query("TRUNCATE TABLE scxnl_rq_unsubscribe");
        
    }

    public function truncateSubscribers()
    {
        global $SystemDB;
        
        $SystemDB->query("TRUNCATE TABLE scxnl_subscribers");
        $SystemDB->query("TRUNCATE TABLE scxnl_subscriber_attributes");
    }
                
    /**
     * Default subscribe page
     */
    public function viewSubscribePage($item)
    {
        $action = $this->createFriendlyURL('action=main'); 
        if (is_postback())
        {
            $result = $this->processSubscribeRequest();
            if ($result)
            {
                if (!$result['status'])
                    $error_list = $result['errors'];
                else 
                    $this->redirectSubscriberToMessage($result);
            }
        }
        $custom_fields = $this->createSubClassInstance('SubscribePages')->getCustomFieldPageAssocation($item['id']);        
        $local_variables = compact(array_keys(get_defined_vars()));        
        $this->loadTemplateFile('view.subscribe', $local_variables);
    }    
    
    /**
     * Default subscribe page
     */
    public function viewDefaultSubscribePage()
    {
        //$this->importFromPHPList();
        
        $subscribe_page = $this->createSubClassInstance('SubscribePages')->getDefaultSubscribePage();
        if ($subscribe_page)
        {
            $this->viewSubscribePage($subscribe_page);
        } else
        {
            $error_list = [___('Default subscribe page has not been specified. Please configure it in the backend admin first')];
            $local_variables = compact(array_keys(get_defined_vars()));        
            $this->loadTemplateFile('view.errors', $local_variables);
        }
    }
                
    /**
     * Return error list from the subscribe page
     * @return array
     */
    protected function getSubscribeRequestValidationErrorList()
    {
        $error_list = [];
        check_csrf_halt_on_error();
        $guid = fpost_string('ehid');
        $custom_fields = $this->createSubClassInstance('SubscribePages')->getCustomFieldFromPageGUID($guid);
        $required_fields = ['email_address'];
        foreach ($custom_fields as $cf)
        {
            if ($cf['required'])
                $required_fields[] = $cf['virtual_filename'];
        }
        
        foreach ($required_fields as $field)
        {
            if (empty(fpost_string($field)))
            {
                $error_list[] = sprintf(___('Field %s is required'), $field);
            }
        }
        $newsletters_to_subscribe = $this->sanitizeNewsletterIDsArray(fpost_array_int('newsletters_to_subscribe'));
        $email = fpost_string('email_address');
        if (filter_var($email, FILTER_VALIDATE_EMAIL) === false) {
            $error_list[] =___('Invalid email address');
        }
        if (\App\Newsletters_Blacklist::isEmailAddressBlacklisted($email))
        {
            $error_list[] = ___('You cannot subscribe to our mailing list');
        }
        if (empty($newsletters_to_subscribe))
        {
            $error_list[] = ___('Please select at least one newsletter to subscribe');
        }
        // check for active user
        if ($this->getSubscriberByRealEmailAddressWithStatus($email, true))
        {
            $error_list[] =___('Email address is already in use');
        }        
        $subscribe_page = $this->createSubClassInstance('SubscribePages')->getSubscribePageByGUID($guid);
        if (!$subscribe_page)
        {
            $error_list[] =___('Invalid subscribe page');
        } 
        
        return $error_list;
    }
    /**
     * Verify the user email address
     * @internal
     * @param string $email
     */
    public static function getRealEmailAddress($email)
    {
        $real_email = strtolower(trim($email));
        list($username, $domain) = explode('@', $email);
        if ($domain == 'gmail.com' || $domain == 'googlemail.com')
        {
            $username = str_replace('.','', $username);
            $pluspos = strpos($username, '+');
            if ($pluspos !== FALSE)
            {
                $username = substr($username, 0 , $pluspos);
            }
            return "{$username}@{$domain}";
        }
        return $real_email;
    }
    
    /**
     * Returns the subscriber info by field real_email_address
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $input_email
     * @return array|boolean
     */
    public function getSubscriberByRealEmailAddress($input_email) {
        global $SystemDB;

        if (empty($input_email))
            return false;
        $input_email = self::getRealEmailAddress($input_email);
        $email = sanitize_string($input_email);
        $sql = "SELECT * from {$this->table_subscribers} where real_email_address <> '' AND  LOWER(real_email_address) = LOWER({$email})";

        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    } 
    
    /**
     * Returns the subscriber info by field real_email_address
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $input_email
     * @param bool $active
     * @return array|boolean
     */
    public function getSubscriberByRealEmailAddressWithStatus($input_email, $active) {
        global $SystemDB;

        if (empty($input_email))
            return false;
        $input_email = self::getRealEmailAddress($input_email);
        $status = (int) NEWSLETTER_SUBSCRIBER_ACTIVE;
        $email = sanitize_string($input_email);
        $sign = $active ? '=' : '<>';
        $sql = "SELECT * from {$this->table_subscribers} where real_email_address <> '' AND  LOWER(real_email_address) = LOWER({$email}) AND status {$sign} {$status}";

        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    } 
    
    /**
     * Returns the subscriber info by field real_email_address
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $input_email
     * @return array|boolean
     */
    public function getUserByRealEmailAddress($input_email) {
        global $SystemDB;

        if (empty($input_email))
            return false;
        $input_email = self::getRealEmailAddress($input_email);
        $email = sanitize_string($input_email);
        $sql = "SELECT * from {$this->table_subscribers} where real_email_address <> '' AND  LOWER(real_email_address) = LOWER({$email})";

        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    }
    
    /**
     * Verify if an existing inactive subscriber wants to subscribe
     * @param int $subscriber_id
     * @param array $attr_values
     * @param int $initial_subscribe_page_id
     * @param array $newsletter_ids
     */
    public function verifyIfSubscriberWantsResubscription($subscriber_id, $attr_values,$initial_subscribe_page_id, $newsletter_ids)
    {
        if (!$initial_subscribe_page_id )
        {
            die ('Invalid subscribe page ID when adding a new unconfirmed user');
        }
        if (!is_array($newsletter_ids))
        {
            die ('Invalid newsletters - non existent');
        }
        
        $subscriber = $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id);
        if ($subscriber)
        {
            if ($subscriber['status'] != NEWSLETTER_SUBSCRIBER_BLACKLISTED)
            {
                $this->createSubClassInstance('Subscribers')->setSubscriberCustomFieldValues($subscriber_id, $attr_values);
                $this->createActivationRequest($subscriber_id, $initial_subscribe_page_id, $newsletter_ids);
                $this->subscriberLog->recordSubscriberActivity($subscriber_id, 'Existing user requested for reactivation');
            } else
            {
                $this->subscriberLog->recordSubscriberActivity($subscriber_id, 'Rejected - cannot request a manually blacklisted user');
            }
        } else {
            $this->eventLog->recordGenericEvent("Cannot find user ID # {$subscriber_id}",'resubscribe', 'error');
        }
    } 
    /**
     * Add new unconfirmed subscriber
     * @param string $email_address
     * @param array $attr_values
     * @param array $newsletter_ids
     * @param string $alternative_email_template_vfn
     */
    public function addUnconfirmedNewSubscriber($email_address, $attr_values,$initial_subscribe_page_id, $newsletter_ids, $alternative_email_template_vfn = '', $notes = 'Opt in')
    {
        global $SystemDB;
        
        $datavalues = [];
        if (!$initial_subscribe_page_id )
        {
            die ('Invalid subscribe page ID when adding a new unconfirmed user');
        }
        if (!is_array($newsletter_ids))
        {
            die ('Invalid newsletters - non existent');
        }
        $datavalues['guid'] = new_uuid_v4();
        $datavalues['email_address'] = trim($email_address);
        $datavalues['real_email_address'] = self::getRealEmailAddress($email_address);
        $datavalues['date_created'] = get_current_datetime();
        $datavalues['hash'] = md5($datavalues['guid'].$email_address.$datavalues['real_email_address'].$datavalues['date_created'].mt_rand(0,65536));
        
        $datavalues['status'] = NEWSLETTER_SUBSCRIBER_NEW;
        $this->table_subscribers->quickInsert($datavalues);
        $new_subscriber_id = $SystemDB->getLastInsertID();
        if (!$new_subscriber_id)
        {
            $this->eventLog->recordGenericEvent('Invalid new user ID for '.$email_address,'main','error');
        }
        else
        {
            $this->subscriberLog->recordSubscriberActivity($new_subscriber_id, "Created new subscriber data with email [{$email_address}]");
            $this->createSubClassInstance('Subscribers')->setSubscriberCustomFieldValues($new_subscriber_id, $attr_values);
            $this->createActivationRequest($new_subscriber_id, $initial_subscribe_page_id, $newsletter_ids, $alternative_email_template_vfn, $notes);
        }
        
    }
    
    /**
     * Get activation request by ID
     * @param int $id
     * @return array
     */
    public function getSubscribeRequestByID($id)
    {
        $id = (int) $id;
        return $this->table_subscribe_request->selectOne('*', "id = {$id}");
    }
    /**
     * Get activation request by ID
     * @param int $id
     * @return array
     */
    public function getUnsubscribeRequestByID($id)
    {
        $id = (int) $id;
        return $this->table_unsubscribe_request->selectOne('*', "id = {$id}");
    }
    
    
    /**
     * Create HTTPS URL if possible
     * @param string $url
     * @return string
     */
    public function createPossibleFriendlyHTTPSURL($url)
    {
        $prefix = SCHLIX_SITE_SSL_ENABLED && SCHLIX_SITE_HTTPS_URL ? SCHLIX_SITE_HTTPS_URL : SCHLIX_SITE_URL;
        return $prefix.$this->createFriendlyURL($url);
    }
    
    /**
     * Return subscribe page info
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @return array
     */
    public function getSubscribePageBySubscriberID($subscriber_id)
    {
        global $SystemDB;
        
        $subscriber_id = (int) $subscriber_id;
        $sql = "SELECT {$this->table_subscribe_pages}.* FROM {$this->table_subscribe_request} INNER JOIN {$this->table_subscribe_pages} ON {$this->table_subscribe_pages}.id = {$this->table_subscribe_request}.subscriber_id WHERE subscriber_id = {$subscriber_id} ORDER BY {$this->table_subscribe_request}.id LIMIT 0,1 ";
        return $SystemDB->getQueryResultSingleRow($sql); 
    }
        
    /**
     * Return trnasactional email template 
     * @param string $pg_prefix
     * @return array
     */
    protected function getEmailTemplate($pg_prefix)
    {
        $result = $this->createSubClassInstance('EmailTemplates')->getEmailTemplateByVirtualFileName($pg_prefix);
        
        if (!$result)
            die ("Cannot find email template {$pg_prefix}");
        return ['html' => $result['body_html'], 'text' => $result['body_text'], 'subject' => $result['title']];
    } 
    
    /**
     * Email Confirmation Request To new subscriber
     * @param int $new_subscriber_id
     * @param int $subscribe_page_id
     * @param string $alternative_email_template_vfn
     */
    public function emailSubscriptionVerification($activation_request_id, $alternative_email_template_vfn = '')
    {        
        $email_template = $alternative_email_template_vfn ? $alternative_email_template_vfn :  'subscription-verification';
        $this->emailSubscriberRequest('subscribe_verification', $this->getSubscribeRequestByID($activation_request_id), false, $email_template,'subscribe request verification');
    }
    /**
     * Email Confirmation Request To new subscriber
     * @param int $new_subscriber_id
     * @param int $subscribe_page_id
     */
    public function emailSubscriptionConfirmation($activation_request_id)
    {
        //$this->emailActivationRequestID($activation_request_id,'subscribe_confirmed','activation confirmation');
        $this->emailSubscriberRequest('subscribe_confirmation', $this->getSubscribeRequestByID($activation_request_id), false, 'subscription-confirmation','subscribe request confirmation');
        
    }    
    /**
     * Email unsubscribe verification to subscriber by I
     * @param int $subscriber_id
     */
    public function emailUnsubscribeVerification($subscriber_id)
    {

        $this->emailSubscriberRequest('unsubscribe_verification',  $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id), false,'unsubscribe-verification','unsubscribe request verification');
    }
    
    
    /**
     * Email Confirmation Request To new subscriber
     * @param int $unsubscribe_request_id
     */
    public function emailUnsubscribeConfirmation($unsubscribe_request_id)
    {
        $this->emailSubscriberRequest('unsubscribe_confirmation', $this->getUnsubscribeRequestByID($unsubscribe_request_id), false,'unsubscribe-confirmation','unsubscribe request confirmation');
    }    
    
    /**
     * Email email change verification
     * @param int $emailchange_request_id
     */
    public function emailEmailAddressChangeVerification($emailchange_request_id)
    {

        $this->emailSubscriberRequest('email_change_verification',  $this->getEmailChangeRequestByID($emailchange_request_id), false, 'email-change-verification',' email change');
    }
    
    /**
     * Email email change confirmation
     * @param int $emailchange_request_id
     */
    public function emailEmailAddressChangeConfirmation($emailchange_request_id)
    {

        $this->emailSubscriberRequest('email_change_confirmation',  $this->getEmailChangeRequestByID($emailchange_request_id), false, 'email-change-confirmation',' email change');
    }
    
    
    /**
     * Email email change cancellation
     * @param int $emailchange_request_id
     */
    public function emailEmailAddressChangeCancellation($emailchange_request_id)
    {

        $this->emailSubscriberRequest('email_change_cancelation',  $this->getEmailChangeRequestByID($emailchange_request_id), false, 'email-change-cancellation',' email change');
    }
    
    /**
     * Notify subscriber that unsubscribe cancelation has been processed and that the preferences have been updated
     * @param int $subscriber_id
     * @param array $old_subscriptions
     */
    public function emailCancelUnsubscribe($subscriber_id, $old_subscriptions)
    {
        $this->emailSubscriberRequest('cancel_unsubscribe',  $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id), true, 'unsubscribe-cancellation', 'cancel unsubscribe', $old_subscriptions);
    }
    
    /**
     * Notify subscriber that the preferences have been updated
     * @param int $subscriber_id
     * @param array $old_subscriptions
     */
    public function emailPreferencesUpdated($subscriber_id, $old_subscriptions)
    {
        $this->emailSubscriberRequest('preferences_updated',  $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id), true, 'preferences-confirmation', 'update preferences', $old_subscriptions);
    }
    
    /**
     * Send preferences link to subscriber
     * @param int $subscriber_id
     */
    public function emailPreferencesVerification($subscriber_id)
    {
        $this->emailSubscriberRequest('preferences_verification',  $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id), true, 'preferences-verification','send preferences link');
    }
    
    /**
     * Return Event Logger
     * @return \App\Newsletters_EventLog
     */
    public function getEventLogger()
    {
        return $this->eventLog;
    }
    
    /**
     * Return user Logger
     * @return \App\Newsletters_SubscriberLog
     */
    public function getSubscriberLogger()
    {
        return $this->subscriberLog;
    }
    
    public function hasNewsletterIssueGUIDQueuedToSubscriberID ($newsletter_guid, $subscriber_id)
    {
        global $SystemDB;
        
        $newsletter_guid = sanitize_string($newsletter_guid);
        $subscriber_id = (int) $subscriber_id;
        $sql = "SELECT * FROM {$this->table_queues} WHERE subscriber_id = {$subscriber_id} AND queue_marker = {$newsletter_guid}";
        $row = $SystemDB->getQueryResultSingleRow($sql);
        return ($row && ($row['id'] > 0));
    }
    
    /**
     * Returns the last subscriber ID - for sending batch
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @return int
     */
    public function getLastSubscriberID()
    {
        global $SystemDB;
        
        $row = $SystemDB->getQueryResultSingleRow("SELECT MAX(id) as max_subscriber_id FROM {$this->table_subscribers}");
        return $row ? $row['max_subscriber_id'] : 0;
    }
    
    
    /**
     * Set whether request emails should be send immediately
     * @param bool $value
     */
    public function setRequestEmailDeliveryInstantaneous($value)
    {
        $this->send_request_emails_immediately = $value;
    }
    /**
     * Email request. Do not access directly
     * @param string $request_type
     * @param bool $is_request_data_subscriber_data
     * @param string $prefix
     * @param string $log_request_type
     * @param array $request_data      
     * @return bool
     */
    protected function emailSubscriberRequest($request_type, $request_data, $is_request_data_subscriber_data, $prefix, $log_request_type, $extra_data = NULL)
    {        
        //$request_data = $this->getSubscribeRequestByID($request_data['id']);
        $request_data_id = $request_data['id'];
        $queue_marker = '';
        $subscriber_id = 0;
        if ($is_request_data_subscriber_data)
        {
            $subscribe_page = $this->getSubscribePageBySubscriberID($request_data['id']); 
            $subscriber_info = $request_data;
            $subscriber_id = $request_data['id'];
            $subscribe_page_id = $subscribe_page['id'];
        } else
        {
            //$queue_marker = $request_data['guid'];
            $subscriber_info = $this->getCompleteSubscriberInfoWithCustomFieldsByID($request_data['subscriber_id']);                
            $subscriber_id = $request_data['subscriber_id'];
            if ((int) $request_data['subscribe_page_id'] > 0)
            {
                $subscribe_page_id =  $request_data['subscribe_page_id'];
            } else
            {
                $subscribe_page = $this->getSubscribePageBySubscriberID($request_data['subscriber_id']);
                $subscribe_page_id =  $subscribe_page['id'];           
            }
        } 
        // subscriber exists?
        if (!$subscriber_info)
        {
            $this->eventLog->recordGenericEvent("Strange error - cannot find subscriber with {$request_type} #{$request_data_id}",$log_request_type,'error');
            return;
        }
         // request data exists?
        if ($request_data)
        {
            
///////////////////////
                $tmpl = $this->getEmailTemplate($prefix);
                $activation_url = $this->createPossibleFriendlyHTTPSURL("action=activation&id={$request_data['guid']}&token={$request_data['hash']}");
                $preferences_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe&id={$subscriber_info['guid']}&token={$subscriber_info['hash']}");
                $preferences_generic_url = $this->createPossibleFriendlyHTTPSURL("action=preferences");
                $unsubscribe_generic_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe");
                $unsubscribe_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe&id={$subscriber_info['guid']}&token={$subscriber_info['hash']}");
                if ($request_data['request_newsletter_ids'])
                    $newsletter_subscriptions = $this->getNewsletterCategoriesByIDs ($request_data['request_newsletter_ids']);
                else
                    $newsletter_subscriptions = $this->getExistingSubscriptionForSubscriberID($subscriber_info['id']);
                $subscribe_url = $this->createPossibleFriendlyHTTPSURL("");
                $datavalues = [
                    'preferences_url' => $preferences_url,   
                    'activation_url' => $activation_url,     
                    'subscribe_url' => $subscribe_url,
                    'unsubscribe_url' => $unsubscribe_url,
                    'preferences_generic_url' => $preferences_generic_url, // prevent auto unsubscribe
                    'unsubscribe_generic_url' => $unsubscribe_generic_url, // prevent auto unsubscribe
                    'ip_address' => get_user_real_ip_address()
                        ];
                $all_custom_fields = $this->createSubClassInstance('CustomFields')->getAllCustomFields();
                if ($subscriber_info && $all_custom_fields)
                foreach ($all_custom_fields as $field)
                {
                    $fieldn = $field['virtual_filename'];
                    $datavalues[$fieldn] = $subscriber_info[$fieldn];
                }
                if ($request_type == 'email_change_verification')
                {
                    $change_email_confirm_url = $this->createPossibleFriendlyHTTPSURL("action=emailchange&id={$request_data['guid']}&confirm=yes");
                    $change_email_cancel_url = $this->createPossibleFriendlyHTTPSURL("action=emailchange&id={$request_data['guid']}&confirm=no");               
                    $change_email_generic_url = $this->createPossibleFriendlyHTTPSURL("action=emailchange&id={$request_data['guid']}");               
                    $datavalues['change_email_confirm_url'] = $change_email_confirm_url;
                    $datavalues['change_email_cancel_url'] = $change_email_cancel_url;
                    $datavalues['change_email_generic_url'] = $change_email_generic_url;
                    
                    $datavalues['old_email_address'] = $request_data['email_address'];
                    $datavalues['new_email_address'] = $request_data['new_email_address'];
                }
                if ($newsletter_subscriptions)
                {
                    $tm = $this->getNewsletterListForEmail($newsletter_subscriptions);
                    $tmpl['html'] = str_replace('{$newsletter_subscriptions}', $tm['html'], $tmpl['html'] );
                    $tmpl['text'] = str_replace('{$newsletter_subscriptions}', $tm['text'], $tmpl['text'] );
                }
                if (is_array($extra_data) && !empty($extra_data))
                {
                    $oldtm = $this->getNewsletterListForEmail($extra_data);
                    $tmpl['html'] = str_replace('{$old_newsletter_subscriptions}', $oldtm['html'], $tmpl['html'] );
                    $tmpl['text'] = str_replace('{$old_newsletter_subscriptions}', $oldtm['text'], $tmpl['text'] );
                }
                $field_to_email = ($request_type == 'email_change_verification') ? $request_data['new_email_address'] : $subscriber_info['email_address'];
                
                $field_replyto_name = $this->sender_default_replyto ? $this->sender_default_name : '';
                $field_to_name = trim($subscriber_info['firstname'].' '.$subscriber_info['lastname']);
                $new_queue_id = $this->emailer->sendEmailWithCustomTemplate($subscriber_id, $queue_marker, $this->sender_default_name, $this->sender_default_email, $field_to_name, $field_to_email, $field_replyto_name, $this->sender_default_replyto, $tmpl['subject'], $tmpl['html'], $tmpl['text'], $datavalues); 
                if ($new_queue_id > 0) 
                {
                    if ($this->send_request_emails_immediately)                    
                        $this->emailer->processQueueByID($new_queue_id);
                    $str_result = ___('SUCCESS') ;
                } else
                    $str_result = ___('FAILED');
                $this->subscriberLog->recordSubscriberActivity($request_data['subscriber_id'], 
                        sprintf(___("Email {$request_type} to %s: %s"), $request_data['email_address'], $str_result));
                return $new_queue_id;
        } else
        {
            $this->eventLog->recordGenericEvent("Cannot find {$log_request_type} ID# {$request_data_id}", $log_request_type, 'error');
        }
        return false;
    }   
    /**
     * Send newsletter issue to subscriber segment ID
     * @param array $issue_id
     */
    protected function emailNewsletterIssue($issue_id)
    {
        global $SystemDB;
        
        $limit_per_batch = 40; // max SQL insert
        $item = $this->getItemByID($issue_id);
        $max_exec_time = ini_get('max_execution_time');                
        @set_time_limit(120 * 60); // max 60 minutes to queue 1000 emails
        // if your SQL server cannot handle 1 insert statement per 7.2 seconds, there's something wrong
        // Should be executed under 5 minutes
        if ($item)
        {
            if ($item['status'] == NEWSLETTER_ISSUE_SENDING)
            {
                
                $queue_marker = $item['guid'];
                if (empty($queue_marker))
                {
                    $this->eventLog->recordGenericEvent("Item GUID cannot be zero for newsletter issue ID #{$issue_id}", 'send', 'error');
                    return;
                }
                $preferences_generic_url = $this->createPossibleFriendlyHTTPSURL("action=preferences");
                $view_in_browser_generic_url = $this->createPossibleFriendlyHTTPSURL("action=viewitem&id={$issue_id}");
                $unsubscribe_generic_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe");
                $subscribers_segment_id = $item['subscribers_segment_id'];
                $segment_extra_criteria = '';
                if ($subscribers_segment_id > 0)
                {
                    $segment_app = new \App\Newsletters_SubscriberSegments();
                    $segment_info = $segment_app->getItemByID($subscribers_segment_id);
                    if ($segment_info)
                    {
                        $segment_extra_criteria = $segment_info['sql_query'];
                    } else
                    {
                        $this->eventLog->recordActivityForNewsletterIssueID($issue_id, "Subscriber segment ID# {$subscribers_segment_id} cannot be found or has been deleted", 'send','error');
                        
                        return;
                    }
                }
                $str_segment_extra_criteria = $segment_extra_criteria ? "AND ({$segment_extra_criteria})" : '';
                $subscriber_active_status = (int) NEWSLETTER_SUBSCRIBER_ACTIVE;
                $sql_subscriber = $this->createSubClassInstance('Subscribers')->generateGetSubscriberSQLWithAllCustomFields();
                $issue_last_sent_to = (int) $item['last_sent_to_subscriber_id'];
                $extra_criteria = ($item['max_subscriber_id'] ) ? "AND (id > {$issue_last_sent_to}) AND (id < {$item['max_subscriber_id']})" : "AND (id > {$issue_last_sent_to})";
                $limit =  "LIMIT {$limit_per_batch}";
                $sql = "SELECT * from ({$sql_subscriber}) tbl1 INNER JOIN {$this->table_subscriptions} ON {$this->table_subscriptions}.subscriber_id = tbl1.id WHERE tbl1.status = {$subscriber_active_status} AND newsletter_id = {$item['category_id']} {$extra_criteria} {$str_segment_extra_criteria} ORDER BY `id` ASC {$limit}";
                $query_result = $SystemDB->query($sql);
                $last_sent_to = 0;
                $total_count = 0;
                $total_subscribers_processed = 0;
                while ($subscriber = $query_result->fetch_assoc())
                {
                    // extra check - yes, this may be redundant and take more db usage,
                    //  but just in case something is wonky, leave the extra check as it is
                    if (!$this->hasNewsletterIssueGUIDQueuedToSubscriberID($queue_marker, $subscriber['id']))
                    {
                        //////////////////////////////////
                        // TODO: check if the user is subscribed to this newsletter?
                        $unsubscribe_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe&id={$subscriber['guid']}&token={$subscriber['hash']}");
                        $preferences_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe&id={$subscriber['guid']}&token={$subscriber['hash']}");
                        $view_in_browser_url = $view_in_browser_generic_url."?s={$subscriber['guid']}&a={$item['guid']}";
                        $track_url = $this->createPossibleFriendlyHTTPSURL("action=vm&?s={$subscriber['guid']}&a={$item['guid']}");
                        $datavalues = array_merge($subscriber, [
                            'view_in_browser_url' => $view_in_browser_url,
                            'webversion_url' => $view_in_browser_url,
                            'track_view_url' => $track_url,
                            'preferences_url' => $preferences_url,   
                            'unsubscribe_url' => $unsubscribe_url,
                            'preferences_generic_url' => $preferences_generic_url, // prevent auto unsubscribe
                            'unsubscribe_generic_url' => $unsubscribe_generic_url ]);

                        $field_to_name = trim($subscriber['firstname'].' '.$subscriber['lastname']);
                        if (!array_key_exists('email_address', $subscriber) )
                                $subscriber['email_address'] = $subscriber['real_email_address'];
                        $extra_headers = [];
                        $extra_headers[] = 'List-Unsubscribe: <'.$unsubscribe_url.'>';
                        $extra_headers[] = 'X-Schlix-Newsletter-Mark: '.$item['guid'];
                        $extra_headers[] = 'X-Schlix-Sub: '.$subscriber['guid'];
                        // don't use real_email_address, use email_address, because real_email_address is for validation only, not the user entered one
                        $insert_id = $this->emailer->sendEmailWithCustomTemplate($subscriber['id'], $queue_marker, $item['email_sender'], $item['email_from'], $field_to_name, $subscriber['email_address'], $item['email_sender'], $item['email_replyto'], $item['title'], $item['body_html'], $item['body_text'], $datavalues, NULL, $extra_headers); 
                        $str_result = ($insert_id > 0) ? sprintf(___('queue ID #%d'), $insert_id) : ___('FAILED');
                        $str_log = sprintf(___("Queued newsletter [issue #{$issue_id}] to %s: %s"), $subscriber['email_address'], $str_result);
                        $this->subscriberLog->recordNewsletterIssueIDForSubscriber($subscriber['id'], $issue_id, $str_log,'newsletters-issue','info');

                        $last_sent_to = $subscriber['id'];
                        $total_count++;
                    } else
                    {
                        $str_log = sprintf(___("Failed to queue newsletter [issue #{$issue_id}] to %s because of an existing queued newsletter issue"), $subscriber['email_address']);
                        $this->subscriberLog->recordNewsletterIssueIDForSubscriber($subscriber['id'], $issue_id,$str_log,'newsletters-issue','info');
                    }
                    ///////////////////////////////////
                    $total_subscribers_processed++;
                }
                @$query_result->close();     
                $this->eventLog->recordActivityForNewsletterIssueID($issue_id, "Queued {$total_count} emails for newsletter issue #{$issue_id}", 'newsletter-issue','info');
                $datavalues = array();
                if ( (($item['date_send_until'] == NULL) || ($item['date_send_until'] == NULL_DATE)) && ($total_subscribers_processed == 0))
                {
                    $datavalues['status'] = NEWSLETTER_ISSUE_SENT_AND_FINISHED;
                    $this->eventLog->recordGenericEvent("Finished sending Newsletter issue ID # {$issue_id} - terminated automatically by count");
                    
                } else                
                if ( (($item['date_send_until'] == NULL) || ($item['date_send_until'] == NULL_DATE)) && 
                    ($last_sent_to >= $item['max_subscriber_id']))
                {
                    $datavalues['status'] = NEWSLETTER_ISSUE_SENT_AND_FINISHED;
                    $this->eventLog->recordGenericEvent("Finished sending Newsletter issue ID # {$issue_id}");
                } else
                {
                    if (($item['date_send_until'] !== NULL) && ($item['date_send_until'] !== NULL_DATE))
                    {
                        if (time_difference(get_current_datetime(), $item['date_send_until']) < 0)
                        {
                            $datavalues['status'] = NEWSLETTER_ISSUE_SENT_AND_FINISHED;
                            $this->eventLog->recordGenericEvent("Finished sending Newsletter issue ID # {$issue_id}");
                        }
                    }                    
                }
                if (($last_sent_to > 0) || ($total_subscribers_processed == 0))
                {
                    if ($last_sent_to > 0)
                        $datavalues['last_sent_to_subscriber_id'] = $last_sent_to;
                    if (!empty($datavalues))
                        $this->table_items->quickUpdate($datavalues, "id = {$item['id']}");                
                }
            } else $this->eventLog->recordGenericEvent ("Newsletter issue ID # {$issue_id} status is not SENDING. Cancelled mailing this issue", 'send', 'error');
        }  else $this->eventLog->recordGenericEvent ("Newsletter issue ID # {$issue_id} cannot be found", 'send', 'error');
        //set_time_limit($max_exec_time); // restore server settings
        
    }
    /**
     * Send test newsletter to subscriber
     * @param int $issue_id
     * @return boolean
     */
    public function emailTestNewsletterIssueToSubscriber ($issue_id)
    {
        $result = array();
        $item = $this->getItemByID($issue_id);
        if ($item)
        {
            $test_email_address = $item['test_email_address'];
            $subscriber = $this->createSubClassInstance('Subscribers')->getLimitedSubscriberInfoByEmail($test_email_address);
            
            if ($subscriber)
            {
                if ($subscriber['status'] != NEWSLETTER_SUBSCRIBER_ACTIVE)
                {
                    $error_list[] = ___("Subscriber ID# {$subscriber['id']} status is not active. Test mail cannot be sent");
                } else
                if ($this->isSubscriberAssignedToNewsletterID($subscriber['id'], $item['category_id']))
                {
                    $queue_marker = "test-{$subscriber['id']}-".date('Y-m-d-H-i-s');
                    $preferences_generic_url = $this->createPossibleFriendlyHTTPSURL("action=preferences");
                    $unsubscribe_generic_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe");
                    $unsubscribe_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe&id={$subscriber['guid']}&token={$subscriber['hash']}");
                    $preferences_url = $this->createPossibleFriendlyHTTPSURL("action=unsubscribe&id={$subscriber['guid']}&token={$subscriber['hash']}");
                    // TODO: check if the user is subscribed to this newsletter?
                    
                    $datavalues = array_merge($subscriber, [
                        'preferences_url' => $preferences_url,   
                        'unsubscribe_url' => $unsubscribe_url,
                        'preferences_generic_url' => $preferences_generic_url, // prevent auto unsubscribe
                        'unsubscribe_generic_url' => $unsubscribe_generic_url ]);
                    $field_to_name = trim($subscriber['firstname'].' '.$subscriber['lastname']);
                    if (!array_key_exists('email_address', $subscriber) )
                            $subscriber['email_address'] = $subscriber['real_email_address'];
                   
                    $extra_headers = [];
                    $extra_headers[] = 'List-Unsubscribe: <'.$unsubscribe_url.'>';
                    $extra_headers[] = 'X-Schlix-Newsletter-Mark: '.$item['guid'];
                    $extra_headers[] = 'X-Schlix-Sub: '.$subscriber['guid'];
                    // don't use real_email_address, use email_address, because real_email_address is for validation only, not the user entered one
                    $new_queue_id = $this->emailer->sendEmailWithCustomTemplate($subscriber['id'], $queue_marker, $item['email_sender'], $item['email_from'], $field_to_name, $subscriber['email_address'], $item['email_sender'], $item['email_replyto'], $item['title'], $item['body_html'], $item['body_text'], $datavalues, null, $extra_headers); 
                    if ($new_queue_id)
                    {
                        $this->emailer->processQueueByID($new_queue_id);
                    }
                    $str_result = $new_queue_id ? ___('SUCCESS') : ___('FAILED');
                    $str_log = sprintf(___("Test newsletter [issue #{$issue_id}] to %s: %s"), $subscriber['email_address'], $str_result);
                    $this->subscriberLog->recordNewsletterIssueIDForSubscriber($subscriber['id'], $issue_id, $str_log,'newsletters-issue','test');
                    $result['status'] = $new_queue_id;
                    $this->eventLog->recordActivityForNewsletterIssueID($issue_id, $str_log,'newsletters-issue','test');
                    return $result;
                    // send test mail
                } else $error_list[] = ___("Subscriber ID# {$subscriber['id']} is not subscribed to this newsletter");
                    
            } else $error_list[] = ___("Test email cannot be sent because {$test_email_address} is not a recognized subscriber in our database");
        } else $error_list[] = ___("Newsletter Issue ID {$issue_id} does not exist");
        if ($error_list)
        {
            $result['status'] = FALSE;
            $result['errors'] = $error_list;
        }
        return $result;
    }
    
    /**
     * Given an array of newsletter, return its title only, in array ['text','html']
     * @param array $newsletter_subscriptions
     * @return array
     */
    protected function getNewsletterListForEmail($newsletter_subscriptions)
    {
            if ($newsletter_subscriptions)
            {
                $all_subs = array_column($newsletter_subscriptions,'title');
                $sub_text = '';
                $sub_html = '<ul>';
                for ($i = 0; $i < count($all_subs); $i++)
                {
                    $sub_html.= "<li>{$all_subs[$i]}</li>";
                    $sub_text.= "* {$all_subs[$i]}\n";
                }
                $sub_html.= '</ul>';
            }
            return ['text' => $sub_text, 'html' => $sub_html];
    }
                
    /**
     * Return the total count of confirmed subscriber in a newsletter
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $newsletter_id
     * @return int
     */
    public function getNewsletterConfirmedSubscriptionCount($newsletter_id)
    {
        global $SystemDB;
        
        $newsletter_id = (int) $newsletter_id;
        $sql = "SELECT COUNT(subscriber_id) AS total_subscription_count FROM {$this->table_subscriptions} INNER JOIN {$this->table_subscribers} ON {$this->table_subscribers}.id = {$this->table_subscriptions}.subscriber_id WHERE newsletter_id = {$newsletter_id} AND {$this->table_subscribers}.status = 1";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        
        return $result['total_subscription_count'];
    }
    
    protected function getNewsletterCategoriesByIDs($str_ids)
    {
        global $SystemDB;
        
        $sanitized_ids = [];
        $ids = explode(',',$str_ids);
        if ($ids)
        {
            foreach ($ids as $id)
                $sanitized_ids[] = (int) $id;
            $str_sanitized_ids = implode(',', $sanitized_ids);
            $sql = "SELECT * FROM {$this->table_categories} WHERE cid IN ({$str_sanitized_ids})";
            return $SystemDB->getQueryResultArray($sql);            
        }
        return NULL;
        
    }
    /**
     * Return the newsletter(s) that a subscriber is assigned to
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @return array
     */
    public function getExistingSubscriptionForSubscriberID($subscriber_id)
    {
        global $SystemDB;
        
        $subscriber_id = (int) $subscriber_id;
        $sql = "SELECT {$this->table_subscriptions}.*, title, status  FROM {$this->table_subscriptions} INNER JOIN {$this->table_categories} ON {$this->table_categories}.{$this->field_category_id} = {$this->table_subscriptions}.newsletter_id WHERE subscriber_id = {$subscriber_id} ORDER BY newsletter_id";
        return $SystemDB->getQueryResultArray($sql);
    }
    
    
    /**
     * Is subscriber already assigned to this newsletter ID?
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @param int $newsletter_id
     * @return bool
     */
    public function isSubscriberAssignedToNewsletterID($subscriber_id, $newsletter_id)
    {
        global $SystemDB;
        
        $subscriber_id = (int) $subscriber_id;
        $newsletter_id = (int) $newsletter_id;
        $sql = "SELECT * FROM {$this->table_subscriptions} WHERE subscriber_id = {$subscriber_id} AND newsletter_id = {$newsletter_id}";
        $row = $SystemDB->getQueryResultSingleRow($sql);
        return ($row['subscriber_id'] == $subscriber_id);
    }
    /**
     * Assign subscriber to newsletter ID
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @param int $newsletter_id
     * @param int $activation_request_id
     * @return type
     */
    protected function assignSubscriberToNewsletterID($subscriber_id, $newsletter_id, $activation_request_id)
    {
        
        if (!$this->isSubscriberAssignedToNewsletterID($subscriber_id, $newsletter_id))
        {
            $subscriber_id = (int) $subscriber_id;
            $newsletter_id = (int) $newsletter_id;
            $date_created = get_current_datetime();
            $datavalues = ['newsletter_id' => $newsletter_id,
                'subscriber_id' => $subscriber_id,
                'subscribe_request_id' => $activation_request_id,
                'date_created' => $date_created
                    ];
            $this->table_subscriptions->quickInsert($datavalues);
            $activation_str = $activation_request_id ? "Activation request ID# {$activation_request_id}" : '';
            $this->subscriberLog->recordSubscriberActivity($subscriber_id, "Subscriber is now assigned to newsletter ID #{$newsletter_id}. {$activation_str}",'error');            
            return true;
        } else
        {
            $this->subscriberLog->recordSubscriberActivity($subscriber_id, "Already assigned to newsletter ID #{$newsletter_id}. Skipping request for assignment",'subscription','error');
        }
        return false;
    }
    
    /**
     * Remove subscriber from newsletter subscription. This function should not be called directly.
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @param int $newsletter_id
     */
    protected function removeSubscriberFromNewsletterID($subscriber_id, $newsletter_id)
    {
        global $SystemDB;
        
        $subscriber_id = (int) $subscriber_id;
        $newsletter_id = (int) $newsletter_id;
        $sql = "DELETE FROM {$this->table_subscriptions} WHERE subscriber_id = {$subscriber_id} AND newsletter_id = {$newsletter_id}";
        $SystemDB->query($sql);
    }    
    /**
     * Subscribe user to newsletter ID
     * @param int $subscriber_id
     * @param array $newsletter_ids
     * @param int $activation_request_id
     * @return boolean
     */
    public function assignSubscriberToNewsletterIDArray($subscriber_id, $newsletter_ids, $activation_request_id)
    {
        $subscriber_id = (int) $subscriber_id;
        $user_info = $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id);
        if ($user_info)
        {
            $sanitized_newsletter_ids = $this->sanitizeNewsletterIDsArray($newsletter_ids);
            if ($sanitized_newsletter_ids)
            {
                $existing_subscriptions = $this->getExistingSubscriptionForSubscriberID($subscriber_id);
                $existing_newsletter_ids = !empty($existing_subscriptions) ? array_column($existing_subscriptions,'newsletter_id') : [];
                foreach ($sanitized_newsletter_ids as $newsletter_id )
                {
                    if (empty($existing_newsletter_ids) || !in_array($newsletter_id, $existing_newsletter_ids))
                        $this->assignSubscriberToNewsletterID($subscriber_id, $newsletter_id, $activation_request_id);
                }
                // Remove the ones not assigned
                if ($existing_newsletter_ids)
                {
                    foreach ($existing_newsletter_ids as $newsletter_id )
                    {
                        if (!in_array($newsletter_id, $sanitized_newsletter_ids))
                            $this->removeSubscriberFromNewsletterID($subscriber_id, $newsletter_id);
                    }
                }
                return true;                
            } else
            $this->subscriberLog->recordSubscriberActivity("Strange error - sanitized newsletter IDs are null",'subscribe','error');
        } else
            $this->eventLog->recordGenericEvent("Strange error - cannot find user ID #{$subscriber_id}",'subscribe','error');
        return false;
    }  
    /**
     * Return subscriber info by ID
     * @param int $subscriber_id
     * @return array
     */
    public function getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id)
    {
        return $this->createSubClassInstance('Subscribers')->getSubscriberByID($subscriber_id);                
    }
    
    /**
     * Return subscriber info by ID
     * @param string $guid
     * @param string $hash
     * @return array
     */
    public function getActiveSubscriberByGUIDAndHash($guid, $hash)
    {
        global $SystemDB;
        
        $sql = "SELECT * FROM {$this->table_subscribers} WHERE `guid` = :guid AND `hash` = :hash AND status = :status";
        $data = $SystemDB->getQueryResultSingleRow($sql, ['guid' => $guid,'hash' => $hash,'status' => NEWSLETTER_SUBSCRIBER_ACTIVE]);
        return $data;
    }
                
    /**
     * Confirm newsletter subscription
     * @param int $id
     */
    protected function confirmNewsletterSubscriptionForRequestID($id)
    {
        $id = (int) $id;
        $activation_request = $this->getSubscribeRequestByID($id);
        if ($activation_request)
        {
            $subscriber_id = $activation_request['subscriber_id'];
            $email = $activation_request['email_address'];
            
            $datavalues['date_activated'] = get_current_datetime();
            $datavalues['status'] = NEWSLETTER_SUBSCRIBER_ACTIVE;
            $datavalues['confirmed_ip_address'] = get_user_real_ip_address();
            $datavalues['confirmed_user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            $this->table_subscribe_request->quickUpdate($datavalues, "id = {$id}");         
            
            $datavalues = null;
            $datavalues = ['status' => NEWSLETTER_SUBSCRIBER_ACTIVE];
            $this->table_subscribers->quickUpdate($datavalues, "id = {$subscriber_id}");
            $this->emailSubscriptionConfirmation($id);
            $newsletter_ids_to_subscribe = explode(',', $activation_request['request_newsletter_ids']);
            $this->assignSubscriberToNewsletterIDArray($subscriber_id, $newsletter_ids_to_subscribe, $activation_request['id']);
            $real_email_address = $this->getRealEmailAddress($email);
            $this->invalidateEmailChangeRequestByRealEmailAddress($real_email_address, "activation request {$activation_request['id']}");
            $this->subscriberLog->recordSubscriberActivity($subscriber_id, "CONFIRMED: Newsletter subscription for request ID# {$id} for email [{$email}]", 'activation', 'info');
            
        } else
        {
            $this->eventLog->recordGenericEvent("Cannot confirm newsletter subscription for request ID# {$id}", 'activation', 'error');
        }
    }
    
    /**
     * Sanitize a given array of integer representing newsletter IDs
     * @param array $int_array
     * @return array
     */
    public function sanitizeNewsletterIDsArray($int_array)
    {
        $result = [];
        if ($int_array)
        foreach ($int_array as $int)
        {
            $i = (int) $int;
            if ($this->getCategoryByID($i))
                $result[] = $i;
        }
        return $result;
    }
    /**
     * Create activation request
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @param int $initial_subscribe_page_id
     * @param array $newsletter_ids
     * @param string $alternative_email_template_vfn
     * @param string $notes
     */
    public function createActivationRequest($subscriber_id, $initial_subscribe_page_id, $newsletter_ids, $alternative_email_template_vfn = '', $notes = 'Opt-in')
    { 
        global $SystemDB;
        
        $sanitized_newsletter_ids = $this->sanitizeNewsletterIDsArray($newsletter_ids);        
        if ($sanitized_newsletter_ids)
        {
            $datavalues = [];
            $datavalues['request_newsletter_ids'] = implode(',',$sanitized_newsletter_ids);
            $subscriber_info = $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id);
            $datavalues['subscriber_id'] = $subscriber_id;
            $datavalues['subscribe_page_id'] = $initial_subscribe_page_id;
            $datavalues['guid'] = new_uuid_v4();
            $hash = sha1($subscriber_info['real_email_address'].mt_rand(0,65536));
            $datavalues['hash'] = new_uuid_v4().'-'.$hash;
            $datavalues['notes'] = $notes;
            $datavalues['email_address'] = $subscriber_info['email_address'];
            $datavalues['real_email_address'] = $subscriber_info['real_email_address'];
            $datavalues['date_created'] = get_current_datetime();
            $datavalues['date_expiry'] = date(DEFAULT_DATE_FORMAT, strtotime("+{$this->activation_expiry_in_hours} hour"));
            $datavalues['status'] = NEWSLETTER_ACTIVATION_NEW;
            $datavalues['ip_address'] = get_user_real_ip_address();
            $datavalues['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            $datavalues['referrer'] = $_SERVER['HTTP_REFERER'];
  
            $this->table_subscribe_request->quickInsert($datavalues);
            $activation_request_id = $SystemDB->getLastInsertID();
            $this->subscriberLog->recordSubscriberActivity($subscriber_id, "Created activation request to email [{$subscriber_info['email_address']}]",'subscribe','info');
            $this->emailSubscriptionVerification($activation_request_id, $alternative_email_template_vfn);
        } else
        {
            $str_newsletter_ids = implode(',', $newsletter_ids);
            $this->subscriberLog->recordSubscriberActivity($subscriber_id, "Invalid attempt to register newsletter IDs [{$str_newsletter_ids}]",'subscribe','error');
        }
    }
    
    /**
     * Display message
     */
    public function viewMessageForUserPage()
    {
        $result = $_SESSION['newsletters_post_result'] ;
        if ($result)
        {
            $message = null;
            //$_SESSION['newsletters_post_result']  = NULL;
            $email = $result['email'];
            switch ($result['type'])
            {
                case 'emailchange_complete':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Your new email address %s has been verified. Thank you!'), $email);                  
                    break;              
                case 'emailchange_verify':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Please verify your new email address by clicking the link in the email: %s'), $email);                  
                    break;                                   
                case 'emailchange_cancelled':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Your request to modify your new email address to %s has been cancelled. Reason: %s'), $email, $result['reason']);                  
                    break;               
                case 'unsubscribe_verify':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('An email containg unsubscribe link has been send to %s. Please verify your unsubscription by clicking the link in that email'), $email);                  
                    break;               
                case 'preferences_update':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Your newsletter preferences have been updated. Thank you!'), $email);                  
                    break;                                        
                case 'preferences_verify':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Please check your inbox for an email message and click on the link in the email. Thank you!'), $email);                  
                    break;                                        
                
                
                case 'cancel_unsubscribe':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('You have decided to modify your preferences instead of unsubscribing. Thank you!'), $email);                  
                    break;                                        
                    
                case 'unsubscribe_complete':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('You have been unsubscribed from our newsletters. We are sorry to see you go!'), $email);                  
                    break;                                        
                case 'activation_complete':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Thank you for your subscription confirmation!'), $email);                  
                    break;         
                case 'activation_invalid':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Error: your request to activate %s has been denied. Reason: %s'), $email, $result['reason']); 
                    break;               
                
                case 'resend_activation':
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('An activation request email has been resent to %s. You will have to click the link to confirm the subscription'), $email);                  
                    break;
                case 'subscribe': 
                    $message_type = 'Info';
                    $message_icon = 'fa-info-circle';
                    
                    $message = sprintf(___('Thank you for your request to subscribe to our newsletter. An activation request email has been sent to %s. You will have to click the link to confirm the subscription'), $email); 
                    break;
                
            }
            
            $local_variables = compact(array_keys(get_defined_vars()));
            $this->loadTemplateFile('view.message', $local_variables);
            
            
        } else display_http_error(406);
    }
    
    /**
     * Return filtered page HTTP POST request
     * @param int $id
     * @return array
     */
    public function filterSubscribeSubmitRequestFromPageID($id)
    {
        $page_custom_fields = $this->createSubClassInstance('SubscribePages')->getCustomFieldPageAssocation($id);
        
        foreach ($page_custom_fields as $cf)
        {
          $xcf[] = $cf['virtual_filename'];
        }
        return select_http_post_variables($xcf);
        
    }
    
    /**
     * Get Activation request by email
     * @param string $email
     * @return array|boolean
     */
    public function getUnclickedActivationRequestByEmail($email)
    {
        $subscriber = $this->getSubscriberByRealEmailAddressWithStatus($email, false);
        if ($subscriber)
        {
            return $this->getUnclickedActivationRequestBySubscriberID($subscriber['id']);            
        }        
        return false;
    }
    
    /**
     * Get activation request by ID
     * @param int $id
     * @return array
     */
    public function getUnclickedActivationRequestByHash($guid, $hash)
    {
        $id = (int) $id;
        $current_date_time = sanitize_string(get_current_datetime());
        $guid = sanitize_string($guid);
        $hash=  sanitize_string($hash);
        $inactive = NEWSLETTER_ACTIVATION_NEW;
        return $this->table_subscribe_request->selectOne('*', "guid = {$guid} AND hash = {$hash} AND (time_to_sec(timediff(`date_expiry`,{$current_date_time})) / 3600) > 0 AND status = {$inactive}",'id','desc');        
    }    
    /**
     * Get activation request by GUID and hash, regardless of the status and date expiry
     * @param int $id
     * @return array
     */
    public function getResultFromActivationRequestByHash($guid, $hash)
    {
        $id = (int) $id;
        $guid = sanitize_string($guid);
        $hash=  sanitize_string($hash);
        $current_date_time = sanitize_string(get_current_datetime());        
        $result = $this->table_subscribe_request->selectOne("*,(time_to_sec(timediff(`date_expiry`,{$current_date_time})) / 3600) AS hours_before_expiry",  "guid = {$guid} AND hash = {$hash}", 'id','desc');        
        return $result;
    }    
    
    /**
     * Get activation request by ID that has not expired
     * @param int $subscriber_id
     * @return array
     */
    public function getUnclickedActivationRequestBySubscriberID($subscriber_id)
    {
        $subscriber_id = (int) $subscriber_id;
        $current_date_time = sanitize_string(get_current_datetime());
        $inactive = NEWSLETTER_ACTIVATION_NEW;
        return $this->table_subscribe_request->selectOne('*', "subscriber_id = {$subscriber_id} AND  (time_to_sec(timediff(`date_expiry`,{$current_date_time})) / 3600) > 0 AND status = {$inactive}",'id','desc');
    }    
         
    /**
     * Process POST from subscribe request
     * @return array|boolean
     */
    public function processSubscribeRequest()
    {
        if (is_postback())
        {
            $status = true;
            $error_list = $this->getSubscribeRequestValidationErrorList();
            $newsletters_to_subscribe = fpost_array_int('newsletters_to_subscribe');
            
            if (empty($error_list))
            {
                $subscribe_page = $this->createSubClassInstance('SubscribePages')->getSubscribePageByGUID (fpost_string('ehid'));
                $datavalues = $this->filterSubscribeSubmitRequestFromPageID($subscribe_page['id']);
                $email = fpost_string('email_address');
                $existing_inactive_subscriber = $this->getSubscriberByRealEmailAddressWithStatus($email, false);
                if ($existing_inactive_subscriber)
                {
                    // update
                    if ($existing_inactive_subscriber['status'] != NEWSLETTER_SUBSCRIBER_BLACKLISTED)
                    {
                        $unclicked_activation = $this->getUnclickedActivationRequestBySubscriberID($existing_inactive_subscriber['id']);
                        if ($unclicked_activation)
                        {
                            if ($existing_inactive_subscriber['status'] == NEWSLETTER_SUBSCRIBER_NEW || $existing_inactive_subscriber['status'] == NEWSLETTER_SUBSCRIBER_UNSUBSCRIBED )
                            {
                                    $_SESSION['reactivation_email'] = $email;
                                    $this->redirectToOtherAction('action=resendactivation');
                                    return false;
                            }
                        
                        } else
                        {
                            $this->verifyIfSubscriberWantsResubscription($existing_inactive_subscriber['id'], $datavalues, $subscribe_page['id'],$newsletters_to_subscribe);
                            $status = true;

                        }
                    } else
                    {
                        $status = false;
                        $error_list[] = ___('You can no longer subscribe to our newsletter. Please contact the site administrator');
                    }
                } else
                {
                    // add new
                    $this->addUnconfirmedNewSubscriber($email, $datavalues, $subscribe_page['id'], $newsletters_to_subscribe);
                    $status = true;
                    
                }
                
            } else
                $status = false;
            $result = [ 'type' => 'subscribe', 'status' => $status, 'errors' => $error_list, 'data' => $datavalues, 'email' => $email];
            return $result;
        }
        return false;
    }
    
    /**
     * Show info page. The parameter must contain type and email array key
     * @param array $result
     */
    public function redirectSubscriberToMessage($result)
    {
        $_SESSION['newsletters_post_result'] = $result;
        $this->redirectToOtherAction('action=info');
    }
    /**
     * Offer the user to resend the activation
     */
    public function viewResendActivationRequest()
    {
        if (is_postback())
        {
            $email_address = fpost_string('email_address');
            $activation = $this->getUnclickedActivationRequestByEmail($email_address);
            $this->emailSubscriptionVerification($activation['id'],'resend-subscription-verification');
            $_SESSION['reactivation_email'] = '';

            $this->redirectSubscriberToMessage([ 'type' => 'resend_activation', 'email' => $email_address]);
            
        } else
        {
            $email_address = $_SESSION['reactivation_email'];
        }
        if (empty($email_address))
            display_http_error(406);
        else 
        {
            $local_variables = compact(array_keys(get_defined_vars()));
            $this->loadTemplateFile('view.resend.activation', $local_variables);
        }        
    }
    
    /**
     * View Main Page
     * @param int $pg
     */
    public function viewMainPage($pg = 1)
    {
        
        return $this->viewDefaultSubscribePage();
        /*
        //$categories = $this->getAllCategories();
        //$this->viewDefaultSubscribePage
        
        $str_mainpage_title = $this->getConfig('str_mainpage_title', $this->getApplicationDescription());
        $standard_criteria = 'status > 0';
        $perpage = $this->getConfig('int_mainpage_items_per_page');
        $main_meta_options =  $this->translateMetaOptions($this->getConfig('array_mainpage_meta_options'));        
        if ($category_meta_options == NULL)
            $category_meta_options = array();
        if ($perpage == 0)
            $perpage = $this->getNumberOfListingsPerPage();
        $total_item_count = $this->getTotalItemCountByCategoryID(0, $standard_criteria);
        $total_category_count = $this->getTotalCategoryCount(0, $standard_criteria);
        $pagination = $this->getStartAndEndForItemPagination($pg, $perpage, $total_category_count + $total_item_count);
        $this->setPageTitle(($pg > 1) ? $str_mainpage_title . " - Page {$pg}" : $str_mainpage_title);
        $this->setPageMetaDescription($this->getConfig('meta_description'));
        $this->setPageMetaKeywords($this->getConfig('meta_key'));

       
        $category_sortby = $main_meta_options['child_categories_sortby'] ?  $main_meta_options['child_categories_sortby'] : $this->getFieldCategoryID();
        $category_sortdir = $main_meta_options['child_categories_sortdirection'] ?  $main_meta_options['child_categories_sortdirection'] : 'DESC';
        
        $item_sortby = $main_meta_options['items_sortby'] ?  $main_meta_options['items_sortby'] : $this->getFieldID();
        $item_sortdir = $main_meta_options['items_sortdirection'] ?  $main_meta_options['items_sortdirection'] : 'DESC';
        

        $total_start = ($pg - 1) * $perpage;
        if ($total_start <= $total_item_count + $total_category_count) {
            if ($total_category_count > 0 && $total_category_count > $total_start) {
                //$category_end = $pagination['start'] + min($total_child_category_count - $pagination['start'], $perpage);
                $categories = $this->getAllCategories('*', $standard_criteria, $pagination['start'], $pagination['end'], $category_sortby, $category_sortdir, $from_cache);
            }

            if ($total_item_count > 0 && $pagination['end'] - $total_category_count > 0) {
                $item_start = max(0, $pagination['start'] - $total_category_count);
                $item_end = $item_start + min($pagination['end'] - $total_category_count, $perpage);

                $items = $this->getItemsByCategoryID(0, '*', $standard_criteria, $item_start, $item_end, $item_sortby, $item_sortdir, $from_cache);
            }
        }
        else
            $error_message = ___("Page is outside of valid range");

        $main_meta_options = convert_array_values_to_keys($this->getConfig('array_mainpage_meta_options'));        

        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.main', $local_variables);*/
     } 
    /**
     * Creates a friendly URL
     * @param string $str
     * @return string
     */
    public function createFriendlyURL($str) {

        if (SCHLIX_SEF_ENABLED) {
            $command_array = array();
            parse_str($str, $command_array);
            if (array_key_exists('action', $command_array) && $command_array['action'] == 'main') {
                if ($command_array['pg'])
                    $final_url.="/pg{$command_array['pg']}.html";
                else
                    $final_url.='/';
            } else
                return parent::createFriendlyURL($str);
            $final_url = SCHLIX_SITE_HTTPBASE . '/' . $this->app_name . $final_url;
        } else
            return parent::createFriendlyURL($str);
        return remove_multiple_slashes($final_url);
    }

    /**
     * Interprets command from SEO-friendly URL
     * @param string $urlpath
     * @return array
     */
    public function interpretFriendlyURL($urlpath) {
        if (SCHLIX_SEF_ENABLED && !fget_string('app')) {
            $parsedurl = $this->probeFriendlyURLDestination($urlpath);
            $url = $parsedurl['url'];
            $url_array = $parsedurl['url_array'];
            if ($url_array[0] == 'rss')
                $command['action'] = 'rss';
            else
            if ($c = preg_match_all("/(pg)(\d+).*?(html)/is", $url_array[0], $x)) {
                $command = array();
                $folder_requestpage = $x[2][0];
                $command['pg'] = $folder_requestpage;
                $command['action'] = 'main';
            } else
                return parent::interpretFriendlyURL($urlpath);
        } else {
            return parent::interpretFriendlyURL($urlpath);
        }
        return $command;
    }

    /**
     * Returns the default category ID 
     * @return int
     */
    public function getDefaultCategoryID() {
        return 1;
    }  

    //_______________________________________________________________________________________________________________//
    /**
     * Returns an array containing on array of main page options. 
     * The values of the options will still be evaluated as a flat list array, 
     * however it is sectioned into array with the following keys:
     * header, value, type, and options.
     * - Label: section title (not used for any evaluation
     * - Type: checkboxgroup, dropdownlist, or none. If none, then it means there 
     *         are suboptions which contain another array of this
     * - Key: the key option. Please note that checkboxgroup doesn't have a key
     *        since the keys are in the options
     * - Options: an array with 2 keys: label and key
     * 
     * @return array
     */    
    
    public function getMainpageMetaOptionKeys() {
        return parent::getMainpageMetaOptionKeys();
    }
    //_______________________________________________________________________________________________________________//
    /**
     * Returns an array containing on array of main page options. In this base
     * class, the key is almost similar to getCategoryMetaOptionKeys
     * The values of the options will still be evaluated as a flat list array, 
     * however it is sectioned into array with the following keys:
     * header, value, type, and options.
     * - Label: section title (not used for any evaluation
     * - Type: checkboxgroup, dropdownlist, or none. If none, then it means there 
     *         are suboptions which contain another array of this
     * - Key: the key option. Please note that checkboxgroup doesn't have a key
     *        since the keys are in the options
     * - Options: an array with 2 keys: label and key
     * 
     * @return array
     */        
    public function getCategoryMetaOptionKeys() {
        return parent::getCategoryMetaOptionKeys();
    }
    
    //_______________________________________________________________________________________________________________//
    /**
     * Returns an array containing on array of item options.
     * The values of the options will still be evaluated as a flat list array, 
     * however it is sectioned into array with the following keys:
     * header, value, type, and options.
     * - Label: section title (not used for any evaluation
     * - Type: checkboxgroup, dropdownlist, or none. If none, then it means there 
     *         are suboptions which contain another array of this
     * - Key: the key option. Please note that checkboxgroup doesn't have a key
     *        since the keys are in the options
     * - Options: an array with 2 keys: label and key
     * 
     * @return array
     */
    public function getItemMetaOptionKeys() {
        return parent::getItemMetaOptionKeys();
    }
    
    /**
     * View activation request page
     * @param string $activation_id
     * @param string $token
     * @return boolean
     */
    public function viewActivationRequestPage($activation_id, $token) {
        $this->setPageTitle(___('Subscriber Activation'));
        $subscriber_verified = false;
        // display the form?
        $prompt_activation = $this->getConfig('bool_activation_requires_form');
        
        $is_posted = (is_postback() && $prompt_activation) || !$prompt_activation;
        // Process User Input
        if ($is_posted) {

            $error_list = [];
            //validation
            if (is_postback())
            {
            if (!is_valid_csrf())
                $error_list[]= ___('Invalid CSRF Verification Token') ;
            $token = fpost_string('activation_string');
            $activation_id = fpost_string('id');
            }
            if (empty($activation_id))
                $error_list[]=  ___('Subscriber ID is empty. Cannot proceed with activation');
            if (empty($token))
                $error_list[]= ___('Token is empty. Cannot proceed with activation') ;
            if (empty($error_list))
                $activation_info = $this->getResultFromActivationRequestByHash($activation_id, $token); // final check
                //print_r($_POST);
            if (empty($activation_info))
                $error_list[] = ___('Cannot find activation info with this token');
            else
            {
                if ($activation_info['hours_before_expiry'] <= 0)
                    $error_list[] = ___('This activation request has expired');
                // elseif for this
                if ($activation_info['status'] == NEWSLETTER_SUBSCRIBER_ACTIVE)
                    $error_list[] = ___('You have already activated this newsletter subscription');            
                else if ($activation_info['status'] != NEWSLETTER_SUBSCRIBER_NEW)
                    $error_list[] = ___('This activation request status is invalid');            

                if (empty($error_list)) {
                    $this->confirmNewsletterSubscriptionForRequestID($activation_info['id']);
                    $result = ['type' => 'activation_complete', 'email' => $subscriber_verified['email_address']];
                    $this->redirectSubscriberToMessage($result);
                    //exit here, all good
                    //$this->loadTemplateFile('view.activation.complete', get_defined_vars());
                    return true;
                }
            }
        } else {
            //////// No POST
            if (empty($activation_id))
                $error_list[] =  ___('ID is empty. Cannot proceed with activation') ;
            if (empty($token))
                $error_list[] = ___('Token is empty. Cannot proceed with activation') ;
        }
            $local_variables = compact(array_keys(get_defined_vars()));
        
        $this->loadTemplateFile('view.activation.request', $local_variables);
    }
    
    /**
     * Complete Unsubscribe
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @param string $unsubscribe_reason
     * @param int $newsletter_issue_id
     * @param bool $is_from_admin
     */
    public function completeUnsubscribe ($subscriber_id, $unsubscribe_reason, $newsletter_issue_id, $is_from_admin = false)
    {
        global $SystemDB, $CurrentUser;
        
        if ($newsletter_issue_id && $this->getItemByID($newsletter_issue_id))
        {
            $unsubscribe_method = 'issue-click';
            $newsletter_issue_id = $newsletter_issue_id;
        } else
        {
            $newsletter_issue_id = 0;
            $unsubscribe_method = $is_from_admin ? 'admin-request' : 'user-request';
        }
        $subscriber_info = $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id);
        if ($subscriber_info)
        {
            $datavalues = [];
            $email_address = $subscriber_info['email_address'];
            $datavalues['date_created'] = get_current_datetime();
            $datavalues['subscriber_id'] = $subscriber_id;
            $datavalues['email_address'] = $email_address;
            $datavalues['feedback'] = strip_tags($unsubscribe_reason);
            $datavalues['method'] = $unsubscribe_method;
            $datavalues['newsletter_issue_id'] = $newsletter_issue_id;
            $datavalues['ip_address'] = get_user_real_ip_address();
            $datavalues['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            $datavalues['referrer'] = $_SERVER['HTTP_REFERER'];
            
            if ($is_from_admin)
            {
                $datavalues['admin_id'] = $CurrentUser->getCurrentUserID();
            }
            $this->table_unsubscribe_request->quickInsert($datavalues);
            $unsubscribe_request_id = $SystemDB->getLastInsertID();
            $datavalues = null;
            $datavalues = ['status' => NEWSLETTER_SUBSCRIBER_UNSUBSCRIBED];
            $this->table_subscribers->quickUpdate($datavalues, "id = {$subscriber_id}");
            $subscriptions = $this->getExistingSubscriptionForSubscriberID($subscriber_id);
            if ($subscriptions)
            {
                foreach ($subscriptions as $sub)
                {
                    $this->removeSubscriberFromNewsletterID($newsletter_issue_id,$sub['newsletter_id']);
                    $this->subscriberLog->recordSubscriberActivity($subscriber_id, "NEWSLETTER_UNSUBSCRIBED: Unsubscribe newsletter ID #{$sub['newsletter_id']} for unsubscribe request ID# {$unsubscribe_request_id} for email [{$email_address}]", 'unsubscribe', 'info');
                }
            } else
            {
                $this->eventLog->recordGenericEvent("Warning - subscriber #{$subscriber_id} requested unsubscribed but not assigned to any newsletters",'unsubscribe', 'error');
            }
            //$newsletter_ids_to_subscribe = explode(',', $activation_request['request_newsletter_ids']);
            //$this->removeSubscriberFromNewsletterID($subscriber_id, $newsletter_ids_to_subscribe, $activation_request['id']);
            
            $this->emailUnsubscribeConfirmation($unsubscribe_request_id);
            $this->subscriberLog->recordSubscriberActivity($subscriber_id, "FULLY_UNSUBSCRIBED: Request ID# {$unsubscribe_request_id} for email [{$email_address}]", 'unsubscribe', 'info');
         } else
        {
            $this->eventLog->recordGenericEvent("Invalid unsubscribe request for subscriber #{$subscriber_id}",'unsubscribe', 'error');
        }
    }
    /**
     * Cancel unsubscribe
     * @param int $subscriber_id
     * @param array $modified_newsletter_ids
     * @param bool $from_cancel_unsubscribe
     */
    public function processRequestToModifySubscriptions($subscriber_id, $modified_newsletter_ids, $from_cancel_unsubscribe)
    {
            $old_subscriptions = $this->getExistingSubscriptionForSubscriberID($subscriber_id);            
            $this->assignSubscriberToNewsletterIDArray($subscriber_id, $modified_newsletter_ids,0);
            if ($from_cancel_unsubscribe)
            {
                $this->subscriberLog->recordSubscriberActivity($subscriber_id, 'Subscriber decided to keep subscribing', 'unsubscribe', 'info');
                $this->emailCancelUnsubscribe($subscriber_id, $old_subscriptions);
            }
            else
            {
                $subs_old = sort(array_values(array_column($old_subscriptions,'newsletter_id')));
                $subs_new = sort(array_values($modified_newsletter_ids));
                if ($subs_old != $subs_new)
                {                
                    $this->subscriberLog->recordSubscriberActivity($subscriber_id, 'Subscriber edited newsletter preferences', 'preferences', 'info');
                    $this->emailPreferencesUpdated($subscriber_id, $old_subscriptions);
                }
            }
    } 
    
    /**
     * Invalidate activation request by email address
     * @param string $real_email_address
     * @param string $reason
     */
    protected function invalidateActivationRequestByRealEmailAddress($real_email_address,$reason)
    {
        $current_date_time = sanitize_string(get_current_datetime());        
        $real_email_address = sanitize_string($real_email_address);
        
        $pending_activation_requests = $this->table_subscribe_request->selectAll("*,(time_to_sec(timediff(`date_expiry`,{$current_date_time})) / 3600) AS hours_before_expiry",  "real_email_address = {$real_email_address} AND hours_before_expiry > 0", 'id','desc');
        
        if (is_array($pending_activation_requests) && count($pending_activation_requests) > 0)
        {            
            $activation_request_ids = array_column($pending_activation_requests,'id');
            $str_activation_ids = implode(',', $activation_request_ids);            
            $this->table_subscribe_request->quickUpdate(['status'=> NEWSLETTER_SUBSCRIBER_INVALID, 'reason' => $reason ], "id IN ({$str_activation_ids})");
            $this->eventLog->recordGenericEvent("Invalidated activation request ID ({$str_activation_ids}) because of {$reason} ", 'activation','warning');
        }
    }
    
    /**
     * Return pending email change
     * @param int $subscriber_id
     * @return array
     */
    public function getPendingSubscriberEmailAddressChange($subscriber_id)
    {
        $subscriber_id = (int) $subscriber_id;
        $status = NEWSLETTER_EMAILCHANGE_REQUEST_NEW;
        $pending_email_change = $this->table_email_change_request->selectOne("*",  "subscriber_id = {$subscriber_id} AND status = {$status}", 'id','desc');
        return $pending_email_change;
    }
    /**
     * Invalidate email address change
     * @param string $real_email_address
     * @param string $reason
     */
    protected function invalidateEmailChangeRequestByRealEmailAddress($real_email_address,$reason)
    {
        $new_status = NEWSLETTER_SUBSCRIBER_NEW;
        $real_email_address = sanitize_string($real_email_address);
        $pending_email_change = $this->table_email_change_request->selectAll("*",  "new_real_email_address = {$real_email_address} AND status = {$new_status}", 'id','desc');
        
        if (is_array($pending_email_change) && count($pending_email_change) > 0)
        {            
            $email_request_ids = array_column($pending_email_change,'id');
            $str_email_request_ids = implode(',', $email_request_ids);            
            $this->table_email_change_request->quickUpdate(['status'=> NEWSLETTER_SUBSCRIBER_INVALID, 'reason' => $reason ], "id IN ({$str_email_request_ids})");
            $this->eventLog->recordGenericEvent("Invalidated email request ID ({$str_email_request_ids}) because of {$reason} ", 'emailchange','warning');
        }
    }
    /**
     * Confirm email address change request by GUID. Check subscriber status as well
     * $confirm_change is either TRUE or FALSE
     * @param string $guid
     * @param bool $confirm_change
     * @param string $reason
     * @return array
     */
    public function confirmEmailAddressChangeByGUID($guid, $confirm_change, $reason = '')
    {
        $emailchange_request = $this->getEmailChangeRequestByGUID($guid);        
        $error_list = [];
        if ($emailchange_request)
        {
            $subscriber_id = $emailchange_request['subscriber_id'];
            $subscriber = $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id);
            if (!$subscriber)
            {
                $error_list[] = ___('Cannot find the specified subscriber');
                $this->eventLog->recordGenericEvent("Cannot confirm email change request {$guid} because subscriber ID#{$subscriber_id} does not exist", 'email change', 'error');
            }
            else
            {
                if ($subscriber['status'] != NEWSLETTER_SUBSCRIBER_ACTIVE)
                    $error_list[] = ___('Subscriber not found, cannot modify email address');
            }
            switch ($emailchange_request['status'])
            {
                case NEWSLETTER_EMAILCHANGE_REQUEST_NEW:
                    $ok = 1; // do nothing
                    break;
                case NEWSLETTER_EMAILCHANGE_DENIED:
                    $error_list[] = ___('This email change request has already been denied');
                    break;

                case NEWSLETTER_EMAILCHANGE_INVALID:
                    $error_list[] = ___('This email change request is invalid');
                    break;
                case NEWSLETTER_EMAILCHANGE_OK:
                    $error_list[] = ___('This email change request has already been performed');
                    break;
                default:
                    $error_list[] = ___('Unknown email change request status');
            }
            if (empty($error_list))
            {
                if ($confirm_change)
                {
                    // modify subscriber's email
                    $new_email_address = trim ($emailchange_request['new_email_address']);
                    $real_email_address = $this->getRealEmailAddress($new_email_address);
                    $datavalues = ['email_address' => $new_email_address,'real_email_address' => $real_email_address];
                    $this->table_subscribers->quickUpdate($datavalues, "id = {$subscriber_id}");
                }
                // modify email change request table
                $datavalues = null;
                $datavalues = array();
                $datavalues['status']  = $confirm_change ? NEWSLETTER_EMAILCHANGE_OK : NEWSLETTER_EMAILCHANGE_DENIED;
                $datavalues['confirmed_ip_address'] = get_user_real_ip_address();
                $datavalues['confirmed_user_agent'] = $_SERVER['HTTP_USER_AGENT'];
                $datavalues['notes'] = str_limit($reason, 4096);
                $datavalues['date_activated'] = get_current_datetime();
                $this->table_email_change_request->quickUpdate($datavalues, "id = {$emailchange_request['id']}");
                $str_log = $confirm_change ? "Changed email address to {$emailchange_request['new_email_address']} confirmed" : "Canceled change email address to {$emailchange_request['new_email_address']}";
                $this->subscriberLog->recordSubscriberActivity($subscriber_id, $str_log, 'emailchange','info');
                
                return ['status' => true];
            }
        } else {
            $error_list[] = ___("Cannot find the specified email change request");
            $this->eventLog->recordGenericEvent("Cannot confirm email change request {$guid} because it is invalid", 'email change', 'error');            
        }
        return ['status' => false, 'errors' => $error_list];
    }
    
    /**
     * Process request to modify email addrses
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @param string $new_email_address
     */
    public function processRequestToModifyEmailAddress($subscriber_id, $new_email_address)
    {
        global $SystemDB;
        
        $result = array();
        $subscriber = $this->getCompleteSubscriberInfoWithCustomFieldsByID($subscriber_id);
        $new_email_address = trim($new_email_address);
        if ($subscriber)
        {
            if (filter_var($new_email_address, FILTER_VALIDATE_EMAIL) === false) {
                $error_list[] =___('Invalid email address');
            } else
            {
                //1. Check for existing email
                $other_user_with_this_email = $this->getSubscriberByRealEmailAddress($new_email_address);
                if (is_array($other_user_with_this_email) && ($other_user_with_this_email['id'] != $subscriber_id))
                {
                    $error_list[] = ___('This email is already registered to another user');
                }
            }
            if (empty($error_list))
            {
                        $real_email_address = $this->getRealEmailAddress($new_email_address);
                        if ( ($other_user_with_this_email['real_email_address'] == $real_email_address) &&
                             ($other_user_with_this_email['email_address'] != $new_email_address)
                                )
                            $this->changeSubscriberEmailAddressWithoutConfirmation($subscriber['id'], $new_email_address);
                        else
                        {
                
                            $datavalues = [];
                            $datavalues['email_address'] = $subscriber['email_address'];
                            $datavalues['new_email_address'] = $new_email_address;
                            $datavalues['subscriber_id'] = $subscriber_id;
                            $datavalues['guid'] = new_uuid_v4();
                            $datavalues['date_created'] = get_current_datetime();
                            $datavalues['status'] = NEWSLETTER_EMAILCHANGE_REQUEST_NEW;
                            $datavalues['ip_address'] = get_user_real_ip_address();
                            $datavalues['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
                            $datavalues['referrer'] = $_SERVER['HTTP_REFERER'];
                            $this->table_email_change_request->quickInsert($datavalues);
                            $new_request_id = $SystemDB->getLastInsertID();
                            $result['status'] = 1;
                            $this->emailEmailAddressChangeVerification($new_request_id);
                        }
            }
        } else
        {
            $this->eventLog->recordGenericEvent("Cannot find subscriber ID# {$subscriber_id} for email address change", 'emailchange', 'error');
        }
        if ($error_list)
        {
            $result['status'] = FALSE;
            $result['errors'] = $error_list;
        }
        return $result;
    }
    
    /**
     * View activation request page
     * @param string $emailchange_request_id
     * @param string $confirmyesno
     * @return boolean
     */
    public function viewConfirmEmailChangePage($emailchange_request_id, $confirmyesno) {
        $this->setPageTitle(___('Email change confirmation'));
        if ($confirmyesno) $confirmyesno = strtolower($confirmyesno);
        $subscriber_verified = false;
        
        $emailchange_request = $emailchange_request_id ? $this->getEmailChangeRequestByGUID($emailchange_request_id) : NULL;
        if (empty($emailchange_request_id))
            $error_list[]=  ___('Email change request ID is empty. Cannot proceed');        
        if (empty($emailchange_request))
            $error_list[] = ___('Cannot find email change request info with this ID');
        if (!empty($emailchange_request))
        switch ($emailchange_request['status'])
        {
            case NEWSLETTER_EMAILCHANGE_DENIED:
                $error_list[] = ___('This email change request has been denied');
                break;
            case NEWSLETTER_EMAILCHANGE_INVALID:
                $error_list[] = ___('This email change request is invalid');
                break;
            case NEWSLETTER_EMAILCHANGE_OK:
                $error_list[] = ___('This email change request has already been performed');
                break;
        }

        // display the form?
        $prompt_activation = $this->getConfig('bool_emailchange_requires_form');
        $is_posted = (is_postback() && $prompt_activation) || !$prompt_activation;
        // Process User Input
        if ($is_posted) {

            //validation
            if (is_postback())
            {
                if (!is_valid_csrf())
                    $error_list[]= ___('Invalid CSRF Verification Token') ;
                $confirmyesno = fpost_string('decision');
                $emailchange_request_id = fpost_string('id');
            }
            if ($confirmyesno != 'yes' && $confirmyesno != 'no')
                $error_list[]= ___('Invalid confirmation - either yes or no must be specified') ;
            else
            {
                if ($confirmyesno != 'yes' && $confirmyesno != 'no')
                    $error_list[] = ___('Invalid confirmation - either yes or no must be specified') ;
                if ($emailchange_request['status'] != NEWSLETTER_EMAILCHANGE_REQUEST_NEW)
                    $error_list[] = ___('This email change request status is invalid'.$emailchange_request['status'].NEWSLETTER_EMAILCHANGE_REQUEST_NEW);       
                if (empty($error_list)) {
                    if ($confirmyesno === 'yes')
                    {
                        $this->confirmEmailAddressChangeByGUID($emailchange_request_id, true);
                        $result = ['type' => 'emailchange_complete', 'email' => $emailchange_request['new_email_address']];
                        $this->redirectSubscriberToMessage($result);
                    } else
                    {
                        $reason = fpost_string_notags('reason');
                        $this->confirmEmailAddressChangeByGUID($emailchange_request_id, false, $reason);
                        $result = ['type' => 'emailchange_cancelled', 'email' => $emailchange_request['email_address'],'reason' => $reason];
                        $this->redirectSubscriberToMessage($result);
                    }
                    //exit here, all good
                    //$this->loadTemplateFile('view.activation.complete', get_defined_vars());
                    return true;
                }
            }
        } else {
            //////// No POST
            if (empty($emailchange_request_id))
                $error_list[] =  ___('ID is empty. Cannot proceed with activation') ;
        }
        $local_variables = compact(array_keys(get_defined_vars()));
        
        $this->loadTemplateFile('view.emailchange.request', $local_variables);
    }
    
    /**
     * View Unsubscribe page
     * @param string $subscriber_guid
     * @param string $subscriber_hash
     * @param int $newsletter_issue_id
     */
    public function viewUnsubscribePage($subscriber_guid, $subscriber_hash, $newsletter_issue_id)
    {
        $this->setPageTitle(___('Unsubscribe'));
        $active_subscriber = ($subscriber_guid && $subscriber_hash) ? $this->getActiveSubscriberByGUIDAndHash($subscriber_guid, $subscriber_hash) : NULL;
        
        // Process User Input
        if (is_postback() ) {

            $error_list = [];
            //validation
            if (!is_valid_csrf())
                $error_list[]= ___('Invalid CSRF Verification Token') ;
            if ($active_subscriber )
            {
                $unsubscribe_type = fpost_int('confirm_unsubscribe');
                if ($unsubscribe_type == 0)
                {
                    $result = ['type' => 'unsubscribe_complete', 'email' => $active_subscriber['email_address']];
                    $this->completeUnsubscribe( $active_subscriber['id'], fpost_string('unsubscribe_reason'), $newsletter_issue_id);
                    $this->redirectSubscriberToMessage($result);
                    return true;
                } else
                if ($unsubscribe_type == 1)
                {
                    $newsletters_to_keep = $this->sanitizeNewsletterIDsArray(fpost_array_int('newsletter_ids'));
                    
                    if (!empty($newsletters_to_keep))
                    {
                        $this->processRequestToModifySubscriptions ($active_subscriber['id'], $newsletters_to_keep, true);

                        $result = ['type' => 'cancel_unsubscribe', 'email' => $active_subscriber['email_address']];
                        $this->redirectSubscriberToMessage($result);
                        return true;
                    } else
                    {
                        $error_list[] = ___('You have to select at least one newsletter if you still want to subscribe');
                    }
                    
                } else
                {
                    $error_list[]= ___('Unknown action');
                }
            } else
            {
                $email_address = fpost_string('email_address');
                if ($email_address)
                {
                    if (filter_var($email_address, FILTER_VALIDATE_EMAIL) === false) {
                        $error_list[] =___('Invalid email address');
                    }
                    if (empty($error_list))
                    {
                        $subscriber = $this->getSubscriberByRealEmailAddress($email_address);
                        switch($subscriber['status'])
                        {
                            case NEWSLETTER_SUBSCRIBER_NEW:
                                $this->completeUnsubscribe($subscriber['id'], 'Warning - new unactivated user decides to unsubscribe before activation confirmation', NULL);
                                break;
                            case NEWSLETTER_SUBSCRIBER_ACTIVE:
                                $this->emailUnsubscribeVerification($subscriber['id']);
                                $result = ['type' => 'unsubscribe_verify', 'email' => $email_address];
                                $this->redirectSubscriberToMessage($result);
                                break;
                            default:
                                $error_list[] = sprintf(___('Unable to find a valid subscriber with the specified email address: %s'), $email_address);
                                break;
                        } 
                    }
                    // send unsubscribe link
                } else
                {
                    $error_list[] =___('Invalid postback');
                }
            }
        } 
        $local_variables = compact(array_keys(get_defined_vars()));        
        
        $this->loadTemplateFile('view.unsubscribe', $local_variables);
    }
    
    /**
     * Get email change request by ID
     * @param int $emailchange_request_id
     * @return array
     */
    public function getEmailChangeRequestByID($emailchange_request_id)
    {
        $id = (int) $emailchange_request_id;
        return $this->table_email_change_request->selectOne('*', "id = {$id}");
    }
    /**
     * Get email change request by ID
     * @param string $guid
     * @return array
     */
    public function getEmailChangeRequestByGUID($guid)
    {
        $guid = sanitize_string($guid);
        return $this->table_email_change_request->selectOne('*', "guid = {$guid}");
    }
    
    /**
     * Change subscriber's email address without confirmation
     * @param int $subscriber_id
     * @param string $new_email_address
     */
    protected function changeSubscriberEmailAddressWithoutConfirmation($subscriber_id, $new_email_address)
    {
        $new_email_address = trim ($new_email_address);
        $subscriber_id = (int) $subscriber_id;
        $real_email_address = $this->getRealEmailAddress($new_email_address);
        $datavalues = ['email_address' => $new_email_address,'real_email_address' => $real_email_address];
        $this->table_subscribers->quickUpdate($datavalues, "id = {$subscriber_id}");
        $this->subscriberLog->recordSubscriberActivity($subscriber_id, "Email address was changed to {$new_email_address} without requiring confirmation", 'emailchange','info');
    }
    
    /**
     * View Unsubscribe page
     * @param string $subscriber_guid
     * @param string $subscriber_hash
     * @param int $newsletter_issue_id
     */
    public function viewPreferencesPage($subscriber_guid, $subscriber_hash, $newsletter_issue_id)
    {
        $this->setPageTitle(___('Edit Preferences'));
        $active_subscriber = ($subscriber_guid && $subscriber_hash) ? $this->getActiveSubscriberByGUIDAndHash($subscriber_guid, $subscriber_hash) : NULL;
        // is there an active subscriber
        $pending_email_change = NULL;
        if ($active_subscriber)
        {
            $subscribe_page = $this->getSubscribePageBySubscriberID($active_subscriber['id']);
            $current_subscriber_email_address = $active_subscriber['email_address'];
            $full_subscriber_info = $this->getCompleteSubscriberInfoWithCustomFieldsByID($active_subscriber['id']);
            $custom_fields = $this->createSubClassInstance('SubscribePages')->getCustomFieldPageAssocation($subscribe_page['id']);
            $pending_email_change = $this->getPendingSubscriberEmailAddressChange($active_subscriber['id']);
        } else
        {
            $subscribe_page = $this->createSubClassInstance('SubscribePages')->getDefaultSubscribePage();
        }
        // Process User Input
        if (is_postback() ) {

            $error_list = [];
            //validation
            if (!is_valid_csrf())
                $error_list[]= ___('Invalid CSRF Verification Token') ;
            if ($active_subscriber )
            {
                // 1. Handle update email address
                $new_email_address = trim(fpost_string('new_email_address'));
                if (!empty($new_email_address))
                {
                    $change_result = $this->processRequestToModifyEmailAddress($active_subscriber['id'], $new_email_address);
                    if (!$change_result['status'])
                    {
                        foreach ($change_result['errors'] as $error)
                            $error_list[] = $error;
                    } else
                    {
                        if ($change_result['sent'])
                        {

                            $result = ['type' => 'emailchange_verify', 'email' => $new_email_address];
                            $this->redirectSubscriberToMessage($result);
                            return true;
                        }
                    } 
                }       
                
                // 2. Handle update field names
                $cf_fields_to_change = [];
                //print_r($custom_fields);die;
                if ($custom_fields)
                {
                    foreach ($custom_fields as $cf)
                    {   
                        $cf_custom_field_name = $cf['virtual_filename'];
                        $cf_new_value = trim(fpost_string($cf_custom_field_name));
                        if ($cf['required'] && empty($cf_new_value))
                        {
                            $error_list[] = sprintf(___('Field %s is required'), $cf['title']);
                        }
                    }
                }
                if (empty($error_list))
                {
                    if (fpost_bool('cancel_email_change') && is_array($pending_email_change) && count($pending_email_change) > 0)
                    {
                        $this->confirmEmailAddressChangeByGUID($pending_email_change['guid'], false, 'user clicked from preferences page');
                    }
                    // still in #2
                    $cf_fields_to_change = $this->filterSubscribeSubmitRequestFromPageID($subscribe_page['id']);
                    $this->createSubClassInstance('Subscribers')->setSubscriberCustomFieldValues($active_subscriber['id'], $cf_fields_to_change);
                    // 3. Handle change newsletter subscriptions
                    $edit_pref_confirm = fpost_int('confirm_edit_pref');
                    if ($edit_pref_confirm == 1)
                    {
                        $newsletters_to_keep = $this->sanitizeNewsletterIDsArray( fpost_array_int('newsletter_ids') );
                        if (!empty($newsletters_to_keep))
                        {
                            $this->processRequestToModifySubscriptions ($active_subscriber['id'], $newsletters_to_keep, false);

                            $result = ['type' => 'preferences_update', 'email' => $active_subscriber['email_address']];
                            $this->redirectSubscriberToMessage($result);
                            return true;
                        } else  $error_list[]= ___('You have to select at least one newsletter in this preferences page');
                    } 
                }
            } else
            {
                $email_address = fpost_string('email_address');
                if ($email_address)
                {
                    if (filter_var($email_address, FILTER_VALIDATE_EMAIL) === false) {
                        $error_list[] =___('Invalid email address');
                    }
                    if (empty($error_list))
                    {
                        $subscriber = $this->getSubscriberByRealEmailAddress($email_address);
                        switch($subscriber['status'])
                        {
                            case NEWSLETTER_SUBSCRIBER_NEW:
                                $this->completeUnsubscribe($subscriber['id'], 'Warning - new unactivated user decides to unsubscribe before activation confirmation');
                                break;
                            case NEWSLETTER_SUBSCRIBER_ACTIVE:
                                $this->emailPreferencesVerification($subscriber['id']);
                                $result = ['type' => 'preferences_verify', 'email' => $email_address];
                                $this->redirectSubscriberToMessage($result);
                                break;
                            default:
                                $error_list[] = sprintf(___('Unable to find a valid subscriber with the specified email address: %s'), $email_address);
                                break;
                        } 
                    }
                    // send unsubscribe link
                } else
                {
                    $error_list[] =___('Invalid postback');
                }
            }
        } 
        $local_variables = compact(array_keys(get_defined_vars()));        
        
        $this->loadTemplateFile('view.preferences', $local_variables);
    }
    
    public function viewSubscribePageByName($name)
    {
        $subscribe_page = $this->createSubClassInstance('SubscribePages')->getSubscribePageByVirtualFilename($name);
        if ($subscribe_page)
        {
            $this->viewSubscribePage($subscribe_page);
        } else
        {
            $error_list = [___('Unable to find the specified subscribe page')];
            $local_variables = compact(array_keys(get_defined_vars()));        
            $this->loadTemplateFile('view.errors', $local_variables);
        }        
    }
    /**
     * Run command passed by the main router
     * @param array $command
     * @return bool
     */
    public function Run($command) {
        switch ($command['action']) {
            case 'viewitem':
                $this->viewItemByID($command['id']);
                break;
            case 'viewcategory': $this->viewCategoryByID($command['cid'], $command['pg'], 'date_created', 'DESC', $this->cache);
                break;
            case 'main': $this->viewDefaultSubscribePage(); // $this->viewMainPage($command['pg']);
                break;
            case 'subscribe':
                $this->viewSubscribePageByName($command['to']);
                break;
            case 'info':
                $this->viewMessageForUserPage();
                break;
            case 'archives':
                $this->viewArchives();
                break;
            case 'resendactivation':
                $this->viewResendActivationRequest();
                break;
            case 'activation':
                $this->viewActivationRequestPage($command['id'], $command['token']);
                break;
            case 'unsubscribe':
                $this->viewUnsubscribePage($command['id'], $command['token'], $command['nlis']);
                break;
            case 'preferences':
                $this->viewPreferencesPage($command['id'], $command['token'], $command['nlis']);
                break;
            case 'emailchange':
                $this->viewConfirmEmailChangePage($command['id'], $command['confirm']);
                break;
            
            default: return parent::Run($command);
        }

        return RETURN_FULLPAGE;
    }
}
