<?php
namespace App;

/**
 * Newsletters - Admin class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 *
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @version 1.0
 * @package newsletters
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
class Newsletters_CustomFields_Admin extends \SCHLIX\cmsAdmin_List {

    //_________________________________________________________________________//
    public function __construct() {
        parent::__construct(null, null);
        $this->setItemFieldNamesForAjaxListing( 'id', 'title','virtual_filename','field_type','date_created','date_modified','status','sort_order');
        
    }
    
    /**
     * Modify data before save item
     * @global \App\Users $CurrentUser
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues) {
        global $CurrentUser;
        
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        
        $current_admin_id = $CurrentUser->getCurrentUserID();
        $field_admin_id = ($datavalues['id'] == 'new') ? 'created_by_id' : 'modified_by_id';
        $field_date_m = ($datavalues['id'] == 'new') ? 'date_created' : 'date_modified';
        $datavalues[$field_admin_id] = intval($datavalues[$field_admin_id]);
        $datavalues[$field_date_m] = get_current_datetime();
        if (!($datavalues[$field_admin_id] > 0 && $datavalues[$field_admin_id] != $current_admin_id))
            $datavalues[$field_admin_id] = $current_admin_id;
        
        $datavalues['virtual_filename'] = convert_into_sef_friendly_title($datavalues['virtual_filename'], true);
        $datavalues['virtual_filename'] = str_replace('-','_', $datavalues['virtual_filename']);
        if (empty($datavalues['meta_description']))
            $datavalues['meta_description'] = $datavalues['title'];
        if (empty($datavalues['virtual_filename']) || ($datavalues['virtual_filename'] == 'index'))
            $datavalues['virtual_filename'] = 'item' . $datavalues[$this->field_id];
        $datavalues['virtual_filename'] = $this->app->preventDuplicateValueInItemTable ('virtual_filename', $datavalues['virtual_filename'], $datavalues[$this->field_id]);
        $datavalues['virtual_filename'] = str_replace('-','_', $datavalues['virtual_filename'] );
        
        return $datavalues;
    }
    /**
     * Validation
     * @param type $datavalues
     */
    public function onGetAdminValidationErrorListBeforeSaveItem($datavalues) {

        $error_list = parent::onGetAdminValidationErrorListBeforeSaveItem($datavalues);
        
        $subscriber_app = $this->app->createSubClassInstance('Subscribers');
        $fieldnames = $subscriber_app->getItemFieldNames();
        if (empty($datavalues['virtual_filename']))
            $error_list[] = ___('Must specify field name');
        if (in_array($datavalues['virtual_filename'], $fieldnames))
            $error_list[] = ___('Custom field name cannot contain any existing field names in the subscribers table');
       
        return $error_list;
    }

} 