<?php
namespace App;
/**
 * Newsletters - Main Class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */

class Newsletters_CustomFields extends \SCHLIX\cmsApplication_List {

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct("Newsletters Custom Fields", 'scxnl_customfields');
        // you can modify these to suit your needs
        $this->disable_frontend_runtime = true;
        $this->has_versioning = true;
    }
    
    /**
     * Returns all available custom fields
     * @return array
     */
    public function getAllCustomFields($visible_only = false)
    {
        $criteria = $visible_only ? 'status > 0' : '';
        
        return $this->getAllItems('*', $criteria, 0,99, 'sort_order','ASC');        
    }
    
    public function getFieldsForNewslettersEditing()
    {
        $available_custom_fields = $this->getAllCustomFields();
        $available_fields = [];
        $available_fields = [
          ['field' => 'unsubscribe_url', 'description' => ___('Unsubscribe URL')],
          ['field' => 'subscribe_url', 'description' => ___('Subscribe URL')],
          ['field' => 'webversion_url', 'description' => ___('Web version URL')],
          ['field' => 'email_address', 'description' => ___('Email address')]    
        ];
        if ($available_custom_fields)
        {
            foreach ($available_custom_fields as $fi)
            {
                $available_fields[] = ['field' =>$fi['virtual_filename'], 'description' =>$fi['title']  ]; 
                //'<strong>${'.$fi['virtual_filename'].'}</strong>'.' - '.SAFE_HTML($fi['title']);
            }
        }
        return $available_fields;
    }
    /**
     * View Main Page
     * @param int $pg
     * @return boolean
     */
    //_______________________________________________________________________________________________________________//
    public function viewMainPage($pg = 1) {
        return false;
    }
    
    /**
     * Validates save item. If there's an error, it will return an array
     * with one or more error string, otherwise it will return a boolean true
     * @global \App\Users $CurrentUser
     * @param array $datavalues
     * @return bool|array String array
     */
    public function getValidationErrorListBeforeSaveItem($datavalues)
    {
        $parent_error_list = parent::getValidationErrorListBeforeSaveItem($datavalues);
        $error_list = array();
        // You can put custom item save validation here
        // e.g. 
        // if (str_contains($datavalues['title'],'a'))
        // $error_list[] = ___('Title may not contain letter a');
        return array_merge($parent_error_list, $error_list);
    }

    /**
     * Before save item
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues) {
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        // You can customize the data values before it's saved here
        // e.g.
        // $datavalues['title'] = real_strip_tags($datavalues['title']);
        //
        return $datavalues;
    }
    
    /**
     * Do something after save item
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    protected function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval)
    {
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
    }


    public static function drawField($cf, $default_value, $with_label)
    {
        
/*        
                    <label for="cf_<?= $cf['id'] ?>" class=" control-label"><?= SAFE_HTML(___($cf['title'])) ?></label>
                    <input type="text" class="form-control" name="<?= SAFE_HTML(___($cf['virtual_filename'])) ?>" id="cf_<?= ___($cf['id']) ?>"  placeholder="<?= SAFE_HTML(___($cf['title'])) ?>" value="<?= SAFE_HTML(fpost_string($cf['virtual_filename'])); ?>" <?= $cf['required'] ? ' required="required"' : '' ?> />

 */
        /*<label for="cf_<?= $cf['id'] ?>" class=" control-label"><?= SAFE_HTML(___($cf['title'])) ?></label>*/
        $label_for = \__HTML::LABEL_FOR($cf['title'], "cf_{$cf['id']}", ['class' => 'control-label']);
        $attributes =
                [
                    'id' =>  "cf_{$cf['id']}",
                    'name' =>  $cf['virtual_filename'],
                ];
        if ($cf['field_type'] == 'dropdown' || $cf['field_type'] == 'radiogroup')
        {
            $options = [];
            if ($cf['value_choices'])
            {
                $arr_choices = explode("\n", $cf['value_choices']);
                $options[] = ['value' => '', 'label' => ' -- '.___('Please select').' -- ' ];
                foreach ($arr_choices as $choice)
                {
                    list ($a, $b) = explode (';', $choice);
                    $a = trim($a);
                    $b = trim($b);
                    $options[] = ['value' => $a, 'label' => $b];
                }
            }
        }
        switch ($cf['field_type'])
        {
            case 'textbox':
                $attributes['class'] = 'form-control';
                $attributes['placeholder'] = $cf['title'];
                if ($cf['regex_pattern'])
                    $attributes['pattern'] = trim ($cf['regex_pattern']);
                if ($cf['required'])
                    $attributes['required'] = 'required';
                $input = \INPUT::TEXT($cf['virtual_filename'], $default_value, $attributes);
                if ($with_label)
                    $input = $label_for.$input;
                break;
            case 'dropdown':
                if ($cf['required'])
                    $attributes['required'] = 'required';
                $attributes['class'] = 'form-control';
                $input = \INPUT::SELECT($cf['virtual_filename'], $options, $default_value, $attributes, 'label','value');
                if ($with_label)
                    $input = $label_for.$input;
                
                break;
            case 'radiogroup':
                $legend = ($with_label) ? $cf['title'] : '';
                $input = \INPUT::RADIO_GROUP($cf['virtual_filename'], $options, $default_value, $legend, $attributes, 'label','value');
                break;
            case 'checkbox':
                $legend = ($with_label) ? $cf['title'] : '';
                $input = \INPUT::HIDDEN($cf['virtual_filename'], 0).
                        \INPUT::CHECKBOX($cf['virtual_filename'], 1, ($default_value == 1), $attributes);
                if ($with_label)
                    $input = $input.$label_for;
                
                break;                
        }
        return $input;
    }

    /**
     * Run Command
     * @param array $command
     * @return boolean
     */
    public function Run($command) {
        //$this->insertTest();
        switch ($command['action']) {
            case 'main': return false;
                break;
            default: return parent::Run($command);
        }
    }



}

?>