<?php
namespace App;
/**
 * Newsletters - Main Class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */

class Newsletters_PHPListImporter  {
    
    protected $table_prefix;
    protected $db_host;
    protected $db_database;
    protected $db_username;
    protected $db_password;
    
    protected $table_subscribers = 'scxnl_subscribers'; // in the future we may be able to change it so it can be associated with another db
    
    public function __construct($table_prefix, $db_host, $db_database, $db_username, $db_password) {
        
        $this->table_prefix = $table_prefix;
        $this->db_host = $db_host;
        $this->db_database = $db_database;
        $this->db_username = $db_username;
        $this->db_password = $db_password;
        $this->eventlog = new \App\Newsletters_EventLog();       
    }
    
    /**
     * Test database connection
     */
    public function testDBConnection()
    {
        $result_list =[];
        if (empty($this->db_host))
            $error_list[] = ___('Empty database host');
        if (empty($this->db_host))
            $error_list[] = ___('Empty database name');
        if (empty($this->db_host))
            $error_list[] = ___('Empty database username');
        if (empty($this->db_host))
            $error_list[] = ___('Empty database password');
        if (empty($error_list))
        {
            try
            {
                $mydb = new \Schlix\cmsDatabase($this->db_host,$this->db_database, $this->db_username,$this->db_password);    
                $check_table_exists = ['list', 'user_user','user_attribute', 'user_user_attribute', 'listuser','user_blacklist','user_blacklist_data'];
                $table_countries = $this->table_prefix.'listattr_country';
                $error_list = [];
                foreach ($check_table_exists as $tbl)
                {
                    $table_name = $this->table_prefix.$tbl;
                    if (!$mydb->tableExists($table_name))
                        $error_list[] = "SQL table {$table_name} does not exists. Invalid table prefix?";
                    else
                    {
                        $row = $mydb->getQueryResultSingleRow("SELECT COUNT(*) as total_count FROM `{$table_name}`");
                        $total_count = number_format($row['total_count'], 0);
                        $result_list[] = "{$table_name}: {$total_count} records";
                    }
                }
            } catch (Exception $exc)
            {
                $error_list[] = ___('Cannot connect to database. Reason:'). $exc->getMessage();
            }

        }
        $result = ['status' => (empty($error_list)),'errors' => $error_list,'result' =>$result_list];
        return $result;
    }
    
    
    
    /**
     * Import from PHPList
     * @param string $table_prefix
     * @param string $db_host
     * @param string $db_database
     * @param string $db_username
     * @param string $db_password
     */
    public function performImport ()
    {
        global $SystemDB;
        
        $truncate_first = true;
        
        
        //$check_table_exists = ['user_attribute','user_user','user_user_attribute'];
        $test = $this->testDBConnection();        
        if (empty($test['errors']))
        {
            $mydb = new \Schlix\cmsDatabase($this->db_host,$this->db_database, $this->db_username,$this->db_password);
            if ($truncate_first)
            {
                $SystemDB->query("TRUNCATE TABLE scxnl_categories");
        
                $SystemDB->query("TRUNCATE TABLE scxnl_log_events");
                $SystemDB->query("TRUNCATE TABLE scxnl_log_subscribers");
                $SystemDB->query("TRUNCATE TABLE scxnl_queues");
                $SystemDB->query("TRUNCATE TABLE scxnl_subscriber_attributes");
                $SystemDB->query("TRUNCATE TABLE scxnl_customfields");
                $SystemDB->query("TRUNCATE TABLE scxnl_subscribe_pages_fieldnames");
                
                $SystemDB->query("TRUNCATE TABLE scxnl_subscribers");
                $SystemDB->query("TRUNCATE TABLE scxnl_subscriptions");
                $SystemDB->query("TRUNCATE TABLE scxnl_rq_subscribe");
                $SystemDB->query("TRUNCATE TABLE scxnl_rq_unsubscribe");
                $SystemDB->query("TRUNCATE TABLE scxnl_blacklist");
                
            }
            set_time_limit(0);
            
            $table_countries = $this->table_prefix.'listattr_country';
            $table_users = $this->table_prefix.'user_user';
            $table_newsletters =  $this->table_prefix.'list';
            $table_attributes = $this->table_prefix.'user_attribute';
            $table_users_attributes = $this->table_prefix.'user_user_attribute';
            $table_users_subscriptions =$this->table_prefix.'listuser';
            $table_users_blacklist =$this->table_prefix.'user_blacklist';
            $table_users_blacklist_data = $this->table_prefix.'user_blacklist_data';
            
            $this->importPHPListNewsletters($mydb, $table_newsletters);
            $this->importPHPListBlackList($mydb, $table_users_blacklist, $table_users_blacklist_data);
            $this->importPHPListAttributes($mydb, $table_attributes);
            $this->importPHPListUsers($mydb, $table_users);
            $this->importPHPListUserAttributes($mydb, $table_users_attributes);
            $this->importPHPListUserSubscriptions($mydb, $table_users_subscriptions);
            $this->updateCountries($mydb, $table_countries);
            $this->updateBlackList();
           
            return ['status' => 'Executed'];
        } else return $test;
    }
    
    public function Run()
    {
        $this->this->performImportFromPHPList(false, $this->table_prefix, $this->db_host, $this->db_database, $this->db_username, $this->db_password);
    }
    /**
     * 
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param \SCHLIX\cmsDatabase $mydb
     * @param string $table_users
     */
    private function importPHPListUsers($mydb, $table_users)
    {
        global $SystemDB;
        
        //$txt_sql = 'SET SQL_MODE = "NO_AUTO_VALUE_ON_ZERO";';
        $query_result = $mydb->query("SELECT * FROM {$table_users}");
        $import_date = get_current_datetime();
        
        $userlog = new \App\Newsletters_SubscriberLog();
        $count = 0;
        $invalid_count = 0;
        while ($data = $mydb->getLastQueryResult()->fetch_assoc())
        {
            $existing_subscriber = $this->getSubscriberByRealEmailAddress($data['email']);
            if (!$existing_subscriber)
            {
                $subscriber_status = 0;
                if ($data['confirmed'] && $data['optedin'])
                    $subscriber_status = 1;
                else if ($data['optedin'] && !$data['confirmed'])
                    $subscriber_status = -2;
                if ($data['blacklisted'])
                    $subscriber_status = -3;
                $datavalues = null;
                $datavalues = [
                  'id' => $data['id'],
                  'guid' => new_uuid_v4(),
                  'hash' => $data['uniqid'],                       
                  'email_address' => trim($data['email']),
                  'real_email_address' => \App\Newsletters::getRealEmailAddress(trim($data['email'])),
                  'status' => $subscriber_status,
                  'bounce_count' => (int) $data['bouncecount'],
                  'date_created' => $data['entered'] ? $data['entered'] : $import_date,
                  'date_modified' => $data['modified'] ? $data['modified'] : NULL,
                  'notes' => "Imported from PHPLIST {$import_date}"
                ];
                $SystemDB->simpleInsertInto($this->table_subscribers, $datavalues);
                $count ++;
                $userlog->recordSubscriberActivity($data['id'], "Imported from PHPList", 'import-phplist','info');
            } else
            {
               $this->eventlog->recordGenericEvent("Cannot import PHPList user ID #{$data['id']} because of duplicate real email {$data['email']}", 'import-phplist');
               $invalid_count++;
            }
        }
        $mydb->getLastQueryResult()->close();      
        $this->eventlog->recordGenericEvent("Imported {$count} existing PHPList users. Skipped {$invalid_count} PHPList users because of duplicates");
        
    }
    /**
     * Import attributes
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param \SCHLIX\cmsDatabase $mydb
     * @param string $table_newsletters
     */
    private function importPHPListNewsletters($mydb, $table_newsletters)
    {
        global $SystemDB;
        
                
        $all_list = $mydb->getQueryResultArray("SELECT * FROM {$table_newsletters}");
        $count = count ($all_list);
        foreach ($all_list as $list)
        {
            $datavalues['cid'] = $list['id'];
            $datavalues['guid'] = new_uuid_v4();
            $datavalues['title'] = $list['name'];
            $datavalues['virtual_filename'] = convert_into_sef_friendly_title($list['name']);
            $datavalues['summary'] = $list['summary'];            
            $datavalues['date_created'] = $list['entered'];
            $datavalues['date_modified'] = $list['modified'];
            $datavalues['status'] = $list['active'];
            $datavalues['sort_order'] = $list['order'];
            $SystemDB->simpleInsertInto('scxnl_categories', $datavalues);
        }
        $this->eventlog->recordGenericEvent("Imported {$count} existing PHPList newsletters");
    }
    
    /**
     * Import attributes
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param \SCHLIX\cmsDatabase $mydb
     * @param string $table_attributes
     */
    private function importPHPListAttributes($mydb, $table_attributes)
    {
        global $SystemDB;
        
                
        $all_attributes = $mydb->getQueryResultArray("SELECT * FROM {$table_attributes}");
        $current_date_time = get_current_datetime();
        $count = count ($all_attributes);
        foreach ($all_attributes as $attr)
        {
            switch ($attr['type'])
            {
                case 'radio':
                    $attr_type = 'radiogroup';
                    break;
                case 'checkbox':
                    $attr_type = 'checkbox';
                    break;
                case 'select':
                    $attr_type = 'dropdown';
                    break;
                default: $attr_type = 'textbox';
            }
            $datavalues['id'] = $attr['id'];
            $datavalues['field_type'] = $attr_type;
            $datavalues['title'] = $attr['name'];
            $datavalues['virtual_filename'] = $attr['tablename'];
            $datavalues['default_value'] = $attr['default_value'];
            $datavalues['sort_order'] = $attr['listorder'];
            $datavalues['date_created'] = $current_date_time;
            $SystemDB->simpleInsertInto('scxnl_customfields', $datavalues);
        }
        $this->eventlog->recordGenericEvent("Imported {$count} existing PHPList attributes");
    }
    
    private function getImportSubscribeRequestID($subscriber_id)
    {
        global $SystemDB;
        
        $row = $SystemDB->getQueryResultSingleRow("SELECT MAX(id) FROM scxnl_rq_subscribe WHERE subscriber_id = :subscriber_id AND status = 1", ['subscriber_id' => $subscriber_id]);
        return $row ? $row['id'] : NULL;
    }
    

    private function getExistingBlackListEntryIDByEmailAddress($email_address)
    {
        global $SystemDB;
        
        $row = $SystemDB->getQueryResultSingleRow("SELECT id FROM scxnl_blacklist WHERE real_email_address = :email", ['email' => $email_address]);
        return $row ? $row['id'] : NULL;
    }    
    /**
     * Import attributes
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param \SCHLIX\cmsDatabase $mydb
     * @param string $table_black_list
     * @param string $table_black_list_data
     */
    private function importPHPListBlackList($mydb, $table_black_list, $table_black_list_data)
    {
        global $SystemDB;
        
        //$txt_sql = 'SET SQL_MODE = "NO_AUTO_VALUE_ON_ZERO";';
        $query_result = $mydb->query("SELECT * FROM {$table_black_list}");
        
        $current_date_time = get_current_datetime();
        
        /*$insert_partition = [];
        $partition_i = 0;
        $max_packet_info = $SystemDB->getQueryResultSingleRow( 'SELECT @@global.max_allowed_packet as max_packet'  );
        $max_packet_size = $max_packet_info['max_packet'] ? $max_packet_info['max_packet'] : 1024 * 1024;*/
        $count = 0;
        while ($data = $mydb->getLastQueryResult()->fetch_assoc())
        {
            $count++;
            $datavalues = null;
            $real_email = \App\Newsletters::getRealEmailAddress($data['email']);
            
            $datavalues = [
              'real_email_address' => $real_email,
              //'subscriber_id' =>  $data['userid'], //               
              'date_created' => $data['added']
            ];
            if (!$this->getExistingBlackListEntryIDByEmailAddress($real_email))
                $SystemDB->simpleInsertInto('scxnl_blacklist', $datavalues);
            else
                $this->eventlog->recordGenericEvent("Skipped blacklisting #{$data['email']} because it already exists", 'import-phplist');
        }
        $mydb->getLastQueryResult()->close();     
        // part 2
        //$txt_sql = 'SET SQL_MODE = "NO_AUTO_VALUE_ON_ZERO";';
        $query_result = $mydb->query("SELECT * FROM {$table_black_list_data}");
        /*$insert_partition = [];
        $partition_i = 0;
        $max_packet_info = $SystemDB->getQueryResultSingleRow( 'SELECT @@global.max_allowed_packet as max_packet'  );
        $max_packet_size = $max_packet_info['max_packet'] ? $max_packet_info['max_packet'] : 1024 * 1024;*/
        $count = 0;
        while ($data = $mydb->getLastQueryResult()->fetch_assoc())
        {
            $count++;
            $datavalues = null;
            $real_email = \App\Newsletters::getRealEmailAddress($data['email']);
            
            $datavalues = [
              'real_email_address' => $real_email,
              'notes' =>  $data['data'],
              'date_created' => $current_date_time
            ];
            $existing_id = $this->getExistingBlackListEntryIDByEmailAddress($real_email);
            if ($existing_id > 0)
            {
                
                $upd = null;
                $upd = array();
                $upd['notes'] = strip_tags($data['data']);
                $SystemDB->simpleUpdate('scxnl_blacklist', $upd, 'id', $existing_id);                
            }
            else
            {
                $SystemDB->simpleInsertInto('scxnl_blacklist', $datavalues);
            }
        }
        $mydb->getLastQueryResult()->close();             
        $this->eventlog->recordGenericEvent("Imported {$count} existing PHPList blacklist");
    }
    
    /**
     * Update user attribute field if it has country field
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param \SCHLIX\cmsDatabase $mydb
     * @param string $table_name
     * @return bool
     */
    public function updateCountries($mydb, $table_name)
    {
        global $SystemDB;
        
        $localfield = $SystemDB->getQueryResultSingleRow("SELECT id FROM scxnl_customfields WHERE virtual_filename = 'country'");
        $field_country_id = $localfield['id'];
        if ($field_country_id <= 0)
        {
            $this->eventlog->recordGenericEvent("No country table detected, exiting", 'import-phplist');
            return;
        }
        
        $real_country_list = [];
        $country_data = $mydb->getQueryResultArray ("SELECT * FROM {$table_name} ORDER BY id");
        $str_countries = '';
        foreach ($country_data as $country)
        {
            $real_country_list[$country['id']] = $country['name'];
            $str_countries = "{$country['name']};{$country['name']}\n";
        }
        $count = 0;
        $countryquery = $SystemDB->query("SELECT * FROM scxnl_subscriber_attributes WHERE fieldname_id = {$field_country_id}");
        while ($data = $countryquery->fetch_assoc())
        {
            if ($data['subscriber_id'] > 0)
            {
                $str_country = $real_country_list[$data['field_value']];
                if ($real_country_list[$data['field_value']])
                {
                    $cntry = sanitize_string($str_country);                    
                    $SystemDB->query("UPDATE scxnl_subscriber_attributes SET field_value = {$cntry} WHERE fieldname_id = {$field_country_id} AND subscriber_id = {$data['subscriber_id']}");
                }
            }
            $count++;
        }
        $countryquery->close();     
        $SystemDB->query("UPDATE scxnl_customfields SET value_choices = :country_list WHERE id = :field_country_id",
                ['country_list' => $str_countries,'field_country_id' => $field_country_id]);
        $this->eventlog->recordGenericEvent("Updated {$count} country data after PHPList import", 'import-phplist');
        
    } 
    
    
    public function updateBlackList()
    {
        global $SystemDB;
        $userlog = new \App\Newsletters_SubscriberLog();
        
        $query_result = $SystemDB->query("SELECT * FROM scxnl_blacklist");
        $current_date_time = get_current_datetime();
        
        /*$insert_partition = [];
        $partition_i = 0;
        $max_packet_info = $SystemDB->getQueryResultSingleRow( 'SELECT @@global.max_allowed_packet as max_packet'  );
        $max_packet_size = $max_packet_info['max_packet'] ? $max_packet_info['max_packet'] : 1024 * 1024;*/
        $count = 0;
        $import_timestamp_notes = "Import PHPList {$current_date_time}";
        while ($data = $SystemDB->getLastQueryResult()->fetch_assoc())
        {
            $count++;
            $datavalues = null;
            $real_email = \App\Newsletters::getRealEmailAddress($data['real_email_address']);
            $subscriber = $this->getSubscriberByRealEmailAddress($real_email);
            if ($subscriber)
            {
                // update subscriber
                $upd = array();
                $upd['status'] = NEWSLETTER_SUBSCRIBER_UNSUBSCRIBED;
                $SystemDB->simpleUpdate('scxnl_subscribers', $upd, 'id', $subscriber['id']);
                // update blacklist subscriber_id
                $upd = array();
                $upd['subscriber_id'] = $subscriber['id'];
                $SystemDB->simpleUpdate('scxnl_blacklist', $upd, 'id', $data['id']);                
                // add unsubscribe data
                $unsubscribe_data = array();
                $unsubscribe_data['subscriber_id'] = $subscriber['id'];
                $unsubscribe_data['date_created'] =  ($data['date_created']) ?  ($data['date_created']) : $current_date_time;
                $unsubscribe_data['feedback'] =  $data['notes'] ? $data['notes']." ~ $import_timestamp_notes" : $import_timestamp_notes;
                $unsubscribe_data['method'] = $data['notes'] ? 'phplist' : 'phplist-import';
                $unsubscribe_data['email_address'] = $real_email;
                $SystemDB->simpleInsertInto('scxnl_rq_unsubscribe', $unsubscribe_data);
                $userlog->recordSubscriberActivity($subscriber['id'], "Unsubscribed during import from PHPList",'unsubscribe','info');
                $count++;
            }
        }
        $SystemDB->getLastQueryResult()->close();     
        $this->eventlog->recordGenericEvent("Updated {$count} unsubscribe data after PHPList import", 'import-phplist');
        
    }
    /**
     * Import attributes
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param \SCHLIX\cmsDatabase $mydb
     * @param string $table_users_subscriptions
     */
    private function importPHPListUserSubscriptions($mydb, $table_users_subscriptions)
    {
        global $SystemDB;
        
        //$txt_sql = 'SET SQL_MODE = "NO_AUTO_VALUE_ON_ZERO";';
        $query_result = $mydb->query("SELECT * FROM {$table_users_subscriptions}");
        
        /*$insert_partition = [];
        $partition_i = 0;
        $max_packet_info = $SystemDB->getQueryResultSingleRow( 'SELECT @@global.max_allowed_packet as max_packet'  );
        $max_packet_size = $max_packet_info['max_packet'] ? $max_packet_info['max_packet'] : 1024 * 1024;*/
        $count = 0;
        while ($data = $mydb->getLastQueryResult()->fetch_assoc())
        {
            $count++;
            $datavalues = null;
            $datavalues = [
              'newsletter_id' => $data['listid'],
              'subscriber_id' => $data['userid'],
              'subscribe_request_id' => $this->getImportSubscribeRequestID($data['userid']),
              'date_created' => $data['entered'],
              'date_modified' => $data['modified'],
            ];
            $SystemDB->simpleInsertInto('scxnl_subscriptions', $datavalues);
        }
        $mydb->getLastQueryResult()->close();     
        $this->eventlog->recordGenericEvent("Imported {$count} existing PHPList user subscriptions");
    }
    
    /**
     * Import attributes
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param \SCHLIX\cmsDatabase $mydb
     * @param string $table_users_attributes
     */
    private function importPHPListUserAttributes($mydb, $table_users_attributes)
    {
        global $SystemDB;
        
        //$txt_sql = 'SET SQL_MODE = "NO_AUTO_VALUE_ON_ZERO";';
        $query_result = $mydb->query("SELECT * FROM {$table_users_attributes}");
        $import_date = get_current_datetime();
        
        /*$insert_partition = [];
        $partition_i = 0;
        $max_packet_info = $SystemDB->getQueryResultSingleRow( 'SELECT @@global.max_allowed_packet as max_packet'  );
        $max_packet_size = $max_packet_info['max_packet'] ? $max_packet_info['max_packet'] : 1024 * 1024;*/
        $count = 0;
        while ($data = $mydb->getLastQueryResult()->fetch_assoc())
        {
            $count++;
            $datavalues = null;
            $datavalues = [
              'fieldname_id' => $data['attributeid'],
              'subscriber_id' => $data['userid'],
              'field_value' => $data['value']
            ];
            $SystemDB->simpleInsertInto('scxnl_subscriber_attributes', $datavalues);
        }
        $mydb->getLastQueryResult()->close();     
        $this->eventlog->recordGenericEvent("Imported {$count} existing PHPList user attributes");
    }
    
    /**
     * Returns the subscriber info by field real_email_address
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $input_email
     * @return array|boolean
     */
    public function getSubscriberByRealEmailAddress($input_email) {
        global $SystemDB;

        if (empty($input_email))
            return false;
        $input_email = \App\Newsletters::getRealEmailAddress($input_email);
        $email = sanitize_string($input_email);
        $sql = "SELECT * from {$this->table_subscribers} where real_email_address <> '' AND  LOWER(real_email_address) = LOWER({$email})";

        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    } 
    
     

}
 