<?php
namespace App;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

class Newsletters_Queues extends \App\EmailQueue
{
    protected static $max_retry_count = 1;

    // can't use Trait yet to connect to basicApplication, wait for php 5.4
    protected static $queue_table_items = 'scxnl_queues';
    protected static $dir_storage_folder = NULL;
    protected static $dir_relative_storage_folder = '/data/private/newsletters';
    protected static $cfg_max_email_per_hour = 180;
    protected static $cfg_send_sleep = 0;
    
    protected static $default_sender = SCHLIX_MAIL_DEFAULT_SENDER;
    protected static $default_sender_address = SCHLIX_MAIL_DEFAULT_EMAIL;
    
    protected static $use_smtp_server;
    protected static $smtp_use_auth;
    protected static $smtp_host;
    protected static $smtp_port;
    protected static $smtp_username;
    protected static $smtp_password;
    protected static $smtp_security;
    
    protected static $xmailer = 'SCHLIX Newsletter (BETA)';
    /**
     * Constructor
     */
    public function __construct() {
        
        \SCHLIX\cmsApplication_List::__construct('Newsletter Queues', static::$queue_table_items);
        static::$queue_table_items =  'scxnl_queues';
        // Fix for PHP5.5 (ugly fix)
        if (static::$dir_storage_folder == NULL)
            static::$dir_storage_folder = SCHLIX_SITE_PATH . '/data/private/newsletters';
        // end ugly fix
        if (!is_dir(static::$dir_storage_folder)) {
            if (!mkdir(static::$dir_storage_folder, 0755, false))
                die('Fatal error: Mail storage folder cannot be created');
        }
        $max_hourly_email = $this->getConfig('int_max_email_per_hour');
        $max_perbatch_email = $this->getConfig('int_max_email_per_batch');
        $send_sleep_second = $this->getConfig('int_email_sleep_second');
        static::$cfg_max_email_per_hour = $max_hourly_email ? $max_hourly_email : 180;
        static::$cfg_max_email_per_batch = $max_perbatch_email ? $max_perbatch_email : 10;
        static::$cfg_send_sleep = $send_sleep_second ? $send_sleep_second : 0;
        $this->disable_frontend_runtime = true;        
    }
                    

    public static function replaceRelativeLinksToAbsolute($body_html)
    {
        return $body_html;
    }
    /**
     * Send email directly using PHPMailer. Don't set it to protected, it's a public method but users shouldn't access this method
     * unless it's really critical so all emails are logged in the database
     * @param type $unique_id
     * @param type $field_subject
     * @param type $field_from_name
     * @param type $field_from_email
     * @param type $field_to_name
     * @param type $field_to_email
     * @param type $field_cc_name
     * @param type $field_cc_email
     * @param type $field_replyto_name
     * @param type $field_replyto_email
     * @param type $field_bcc
     * @param type $field_priority
     * @param type $extra_headers
     * @param type $field_xmailer
     * @param type $field_return_receipt
     * @param type $body_html
     * @param type $body_text
     * @param type $attachments
     */

    public static function __sendEmailDirectly($timeout, $unique_id, $field_subject, $field_from_name, $field_from_email, $field_to_name, $field_to_email, $field_cc_name, $field_cc_email, $field_replyto_name, $field_replyto_email, $field_bcc, $field_priority, $extra_headers, $field_xmailer, $field_return_receipt, $body_html, $body_text, $attachments)
    {
        global $SystemLog;
        $mymailer = new \PHPMailer(true);
        try
        {
            /// Load Settings
            if (static::$use_smtp_server) {
                $mymailer->IsSMTP();
                $mymailer->SMTPAuth = static::$smtp_use_auth;
                $mymailer->Port = static::$smtp_port;
                $mymailer->Host = static::$smtp_host;
                if (static::$smtp_use_auth == true) {
                    $mymailer->Username = static::$smtp_username;
                    $mymailer->Password = static::$smtp_password;
                }
                if (static::$smtp_security)
                    $mymailer->SMTPSecure = strtolower(static::$smtp_security);
            }
            $mymailer->Timeout = ($timeout > 0) ? $timeout : 180;
            /// End load settings
            // Basic header


            if ($field_from_email)
                $mymailer->SetFrom($field_from_email,$field_from_name);
            if ($field_to_email)
                $mymailer->AddAddress($field_to_email, $field_to_name);        
            if ($field_replyto_email)
                $mymailer->AddReplyTo($field_replyto_email, $field_replyto_name);
            if ($field_cc_email)
                $mymailer->AddCC($field_cc_email, $field_cc_name);
            if ($field_bcc)
                $mymailer->AddBCC($field_bcc);
            $mymailer->Subject = $field_subject;        
            // end basic header
            /////////////////////////////////////////////////////////////////////
            // Advanced header
            $mymailer->XMailer = $field_xmailer ?  $field_xmailer : static::$xmailer;
            $mymailer->CharSet = 'UTF-8';        

            $mymailer->addCustomHeader('MIME-Version', '1.0');
            if ($field_priority != 0)
            {
                switch ($field_priority)
                {
                    case EMAIL_PRIORITY_LOW:
                        $mymailer->addCustomHeader('X-Priority', '5 (Lowest)');
                        $mymailer->addCustomHeader('X-MSMail-Priority', 'Low');
                        $mymailer->addCustomHeader('Importance', 'Low');    

                        break;
                    case EMAIL_PRIORITY_HIGH:
                        $mymailer->addCustomHeader('X-Priority', '1 (Highest)');
                        $mymailer->addCustomHeader('X-MSMail-Priority', 'High');
                        $mymailer->addCustomHeader('Importance', 'High');    
                        break;
                }
                //X-MSMail-Priority: High, Normal, or Low
                //Importance: High, Normal, or Low
            }
            if ($field_return_receipt) {
                $mymailer->addCustomHeader('Disposition-Notification-To', $field_return_receipt);
                $mymailer->addCustomHeader('X-Confirm-Reading-To', $field_return_receipt);
            }        
            if ($extra_headers)
            {
                $header_lines = explode('|', $extra_headers);
                if (is_array($header_lines) && count ($header_lines) > 0)
                {
                    foreach ($header_lines as $header)
                    {
                        $mymailer->addCustomHeader($header);
                    }
                }
            }
            if ($unique_id)
                $mymailer->addCustomHeader('X-Schlix-NLT-MsgID',$unique_id);
            // end Advanced header
            /////////////////////////////////////////////////////////////////////        
            // Body & Text
            $mymailer->WordWrap = 80;
            if (!empty($body_html)) {
                $mymailer->IsHTML(true); // send as HTML
                $converted_text = $mymailer->MsgHTML($body_html, SCHLIX_SITE_PATH);
                $mymailer->AltBody = (!empty($body_text)) ? $body_text : $converted_text;
            } else {
                $mymailer->IsHTML(false); // send as plaintext
                $mymailer->Body = $body_text;
            }
            if ($attachments) 
            {
                $attachment_array = explode('|', $attachments);
                foreach ($attachment_array as $file)
                {
                    $attachment_filename = SCHLIX_SITE_PATH.'/data/private/attachments/'.$file;
                    if (file_exists($attachment_filename))
                    {
                        $mymailer->AddAttachment($attachment_filename);
                    }
                }
            }

            $mymailer->Send();
            if ($use_smtp_server)
            {
                $mymailer->smtpClose();
            }
            return 'SENDMAIL_OK';
        }
        catch (\phpmailerException $e) {
            $error_msg = $e->errorMessage();
            //echo "Mail Exception: " . $error_msg;
            $SystemLog->error($error_msg);
            return $error_msg;
        }
        catch (\Exception $e) {
            $error_msg = $e->errorMessage();
            echo "EXCEPTION: ".$error_msg; //Boring error messages from anything else!
            $SystemLog->error($error_msg);
            return $error_msg;
        }
    }

    
    public static function setSMTPServerInfo($use_smtp, $security, $host, $port, $smtp_use_auth, $username, $password)
    {
        if ($use_smtp)
        {
            static::$use_smtp_server = $use_smtp;            
            static::$smtp_host = $host;
            static::$smtp_port = (int) $port ? $port : NULL;
            static::$smtp_security = trim(strtolower($security));
            if ($smtp_use_auth)
            {
                static::$smtp_username = $username;
                static::$smtp_password = $password;
                static::$smtp_use_auth = $smtp_use_auth;
            }
        }
    }

    //_______________________________________________________________________________________________________________//
    public function getEmailTemplate($virtual_filename) {
        //include_inc('emailtemplates.inc.php');
        $mailtemplate = new \App\Newsletters_Templates();
        $item = $mailtemplate->getItemsByVirtualFilename($virtual_filename);
        if ($item)
            return $item[0];
        else
            return false;
    }
    
    protected static function getNewsletterIssueStatus($guid)            
    {
        global $SystemDB;
        
        $row = $SystemDB->getQueryResultSingleRow("SELECT status FROM scxnl_issues WHERE guid = :guid", ['guid' => $guid]);
        return $row ? $row['status'] : NULL;
    }
 
    public static function getTotalByQueueMarkerAndStatus($guid, $status)
    {
        global $SystemDB;
        
        $row = $SystemDB->getQueryResultSingleRow("SELECT COUNT(id) AS total_count FROM scxnl_queues WHERE status = :status AND queue_marker = :guid",
                ['status' => (int) $status, 'guid' => $guid]);
        return $row['total_count'];        
    }
    
    /**
     * Cancel unsent newsletter issue
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $guid
     */
    public static function cancelDeliveryByQueueMarker($guid)
    {
        global $SystemDB;
        
        $SystemDB->query("UPDATE scxnl_queues SET status = -2 WHERE status = 0 AND queue_marker = :guid",['guid' => $guid]);        
    }
    
    //_______________________________________________________________________________________________________________//
    protected static function processSingleQueue(array $queue) {
        global $SystemLog, $SystemConfig;

        set_time_limit(300);
        if ($queue['status'] == 1)
        {
            $SystemLog->error("Queue #{$queue['id']} to {$queue['field_to_email']} status is already 1");
            return false;
        }
        //$mailstore_folder = static::getEmailStorageFolderForID($queue['id']);
        $mailstore_folder = SCHLIX_SITE_PATH.$queue['store_folder'];
        $body_html = file_get_contents("{$mailstore_folder}/{$queue['id']}.html");
        $body_text = file_get_contents("{$mailstore_folder}/{$queue['id']}.txt");
        
        if (empty($body_html) && empty($body_text))
        {
            $SystemLog->error("Cannot find either HTML or text content for email ID# {$queue['id']}");
            return false;
        }
        $unq = $queue['field_sha1'].'-'.$queue['id'];
        $dbconfig_timeout = $SystemConfig->get('emailqueue','int_smtp_timeout');
        // Custom mod
        if (!empty($queue['queue_marker']) && !\App\Newsletters_Subscribers::isSubscriberActive($queue['subscriber_id']) )
        {
            $result = 'SENDMAIL_FAILED_INACTIVE_SUBSCRIBER - Subscriber was inactive just before this email was about to be sent';
        } else
        {
        $result = static::__sendEmailDirectly($dbconfig_timeout, $unq, $queue['field_subject'], $queue['field_from_name'], $queue['field_from_email'], 
            $queue['field_to_name'], $queue['field_to_email'], $queue['field_cc_name'], $queue['field_cc_email'], 
            $queue['field_replyto_name'], $queue['field_replyto_email'], $queue['field_bcc'], $queue['field_priority'],
            $queue['extra_headers'], $queue['field_xmailer'], $queue['field_return_receipt'],
            $body_html, $body_text, $queue['attachments']);
        }

        if ($result == 'SENDMAIL_OK')
        {
            static::markEmailQueueAsSentByID($queue['id']);
            return true;
        } else
        {
            static::markEmailQueueAsFailedByID($queue['id'], $result);
            return false;
        }
        return false;
    }

    //_______________________________________________________________________________________________________________//
    
    public static function getListOfUnsentEMails($max_count) {
        global $SystemDB;

        $max_count = intval($max_count);
        $sql = "SELECT * FROM  " . static::$queue_table_items . "  WHERE status = 0 AND retry_count < 5 LIMIT 0, {$max_count}";
        
        $result = $SystemDB->getQueryResultArray($sql);
        return $result;
    }
    
                    
    public static function processRunEmailQueue() {
        global $SystemLog;
        
        set_time_limit(0);
        $str_q_ids= '';
        $id_failed = $id_success = [];
        $SystemLog->record("processRunEmailQueue called");
        $guidmarker_statuses = array();
        $max_rate_per_hour = static::getMaxEmailsPerHour();        
        $email_sent_last_hour = static::getCountOfEmailsSentWithinLastHour();
        $count_tobe_sent = $max_rate_per_hour - $email_sent_last_hour;
        
        if ($count_tobe_sent <= 0) {
            $output = "Would *NOT* process email queue because max emails/hour = {$max_rate_per_hour} and within the last hour the system sent {$email_sent_last_hour} emails";
            echo $output;
            $SystemLog->record($output);
            return false;
        } else {
            $max_perbatch = static::getMaxEmailsPerBatch();
            $count_tobe_sent = $max_rate_per_hour - $email_sent_last_hour;
            if ($count_tobe_sent > $max_perbatch)
                $count_tobe_sent = $max_perbatch;
            //echo $max_rate_per_hour; echo $email_sent_last_hour;die;
            // Now process the queue
            $unsent_emails = static::getListOfUnsentEMails($count_tobe_sent);
            $total_unsent_emails = count($unsent_emails);
            $count_success = 0;
            $count_failure = 0;
            $count_skipped = 0;
            if ($unsent_emails)
            {
                foreach ($unsent_emails as $unsent_email)
                {
                    set_time_limit(0);

                     //$SystemLog->record("Sending queue #{$unsent_email['id']}");
                    $is_nl_ok = true;
                    $qm = $unsent_email['queue_marker'];
                    $email_addr = $unsent_email['field_to_email'];
                        if (!empty($qm))
                        {
                            if (!array_key_exists($qm, $guidmarker_statuses))
                            {
                                $guidmarker_statuses[$qm] = static::getNewsletterIssueStatus($qm);
                            }
                            $is_nl_ok = ($guidmarker_statuses[$qm] == 2) || ($guidmarker_statuses[$qm] === NULL) ; // NEWSLETTER_ISSUE_SENDING
                        }
                        if ($is_nl_ok)
                        {
                            if (static::processSingleQueue($unsent_email))
                            {
                                $id_success[] = $unsent_email['id'];
                                $count_success++;
                            } else
                            {
                                $id_failed[] = $unsent_email['id'];
                                $count_failure++;
                            }
                            /*if (static::$cfg_max_email_per_hour > 0)
                                sleep(static::$cfg_max_email_per_hour);*/
                        } else $count_skipped++;
                    //$SystemLog->record("Done queue #{$unsent_email['id']}");
                }
                $q_ids = array_column($unsent_emails,'id');
                $str_q_ids = implode(', ', $q_ids);
            } else
            {
                $total_unsent_emails = "0";
            }
            $str_id_success = implode(', ', $id_success);
            $str_id_failed = implode (', ', $id_failed);
            $output = "Processed {$total_unsent_emails} queue items ($str_q_ids). Success: {$count_success} ({$str_id_success}). Failure: {$count_failure} ({$str_id_failed}). Skipped: {$count_skipped}. Max emails/hour = {$max_rate_per_hour}. Max email/batch = {$max_perbatch}. In this hour, {$email_sent_last_hour} emails have been sent";
            echo $output; // output to cron log
            // output to system log
            if ($total_unsent_emails > 0)
                $SystemLog->record($output);
            $SystemLog->record("processRunEmailQueue done");

            return true;
        }
    }
    
    //_______________________________________________________________________________________________________________//
    /**
     * Send email with custom template
     * @param int $subscriber_int
     * @param string $queue_marker     
     * @param string $field_from_name
     * @param string $field_from_email
     * @param string $field_to_name
     * @param string $field_to_email
     * @param string $field_cc_name
     * @param string $field_cc_email
     * @param string $field_replyto_name
     * @param string $field_replyto_email
     * @param string $tmpl_subject
     * @param string $tmpl_body_html
     * @param string $tmpl_body_text
     * @param array $vars
     * @param array $attachments
     * @param array $extra_headers
     * @return boolean
     */
    public function sendEmailWithCustomTemplate($subscriber_id, $queue_marker, $field_from_name, $field_from_email, $field_to_name, $field_to_email, $field_replyto_name, $field_replyto_email, $tmpl_subject, $tmpl_body_html, $tmpl_body_text,   $vars, $attachments = NULL, $extra_headers = NULL) {
        //$processed_email_template = $this->getEmailSubjectAndBodyFromTemplate($virtual_filename, $vars);
        $subject = $this->processMessageTemplate($tmpl_subject, $vars);
        $body_html = $this->processMessageTemplate($tmpl_body_html, $vars);
        $body_text = $this->processMessageTemplate($tmpl_body_text, $vars);
        $field_cc_name = $field_cc_email = '';
        if ($subject) {
            
           return $this->sendNLEmailToQueue($subscriber_id, $queue_marker, $subject, $field_from_name, $field_from_email, $field_to_name, $field_to_email, $field_cc_name, $field_cc_email, $field_replyto_name, $field_replyto_email, '', '', '', '', $body_html, $body_text, $attachments, $extra_headers);
        } else return FALSE;
    }

    
    /**
     * 
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_int
     * @param string $queue_marker
     * @param type $field_subject
     * @param type $field_from_name
     * @param type $field_from_email
     * @param type $field_to_name
     * @param type $field_to_email
     * @param type $field_cc_name
     * @param type $field_cc_email
     * @param type $field_replyto_name
     * @param type $field_replyto_email
     * @param type $field_bcc
     * @param type $field_priority
     * @param type $field_xmailer
     * @param type $field_return_receipt
     * @param type $body_html
     * @param type $body_text
     * @param type $attachments
     * @param type $extra_headers
     * @return boolean
     */
    public function sendNLEmailToQueue($subscriber_id, $queue_marker, $field_subject, $field_from_name, $field_from_email, $field_to_name, $field_to_email, $field_cc_name, $field_cc_email, $field_replyto_name, $field_replyto_email, $field_bcc, $field_priority, $field_xmailer, $field_return_receipt, $body_html, $body_text, $attachments, $extra_headers) {
        global $SystemDB;

        $body_html = trim ($body_html);
        $body_text = trim($body_text);
        $field_xmailer = trim($field_xmailer);
        $field_subject = trim($field_subject);
        if (empty($field_subject) || empty($field_to_email) || empty($field_from_email) || (empty($body_html) && empty($body_text)))
        {
            return FALSE;
        }
        
        $guid = new_uuid_v4();
        $the_attachment = NULL;
        $datavalues = array(
          'subscriber_id' => $subscriber_id,
          'queue_marker' => $queue_marker,
          'field_subject' => $field_subject,
          'field_from_name' => $field_from_name,
          'field_from_email' => $field_from_email,
          'field_to_name' => $field_to_name,
          'field_to_email' => $field_to_email,
          'field_cc_name' => $field_cc_name,
          'field_cc_email' => $field_cc_email,
          'field_replyto_name' => $field_replyto_name,
          'field_replyto_email' => $field_replyto_email,
          'field_bcc' => $field_bcc,
          'field_priority' => $field_priority,
          'field_xmailer' => $field_xmailer,
          'field_return_receipt' => $field_return_receipt,
          'guid' => $guid,
          'date_queued' => get_current_datetime()
        );
        
        if (is_array($attachments) && count($attachments) > 0)
        {
            $attachments = array_unique($attachments);
            foreach ($attachments as $file)
            {
                if (!file_exists(SCHLIX_SITE_PATH.'/data/private/attachments/'.$file))
                    return false;
            }
            $the_attachment = implode('|', $attachments);
        }        
        if ($the_attachment)
            $datavalues['attachments'] = $the_attachment;
        $extra_header_content = '';
        if ($extra_headers)
        {
            if (is_array($extra_headers))
            {
                if (count($extra_headers > 0))
                {
                    $extra_header_content = implode("|", $extra_headers);
                }
            } else
            {
                $extra_header_content = $extra_headers;
            }
        }
        $datavalues['extra_headers'] = $extra_header_content;
        $serialized_data = @serialize($datavalues).$body_html.$body_text;
        $datavalues['field_sha1'] = sha1($serialized_data); // to check authenticity
        
        $SystemDB->simpleInsertInto($this->table_items, $datavalues);
       
        $new_email_id = $SystemDB->getLastInsertID();

        $mailstore_folder = static::getEmailStorageFolderForID($new_email_id);
        $relative_mailstore_folder = static::getEmailStorageFolderForID($new_email_id, TRUE);
        
        $upd_store_folder = array('store_folder' => $relative_mailstore_folder);
        $SystemDB->simpleUpdate($this->table_items, $upd_store_folder,'id', $new_email_id);
        if ($this->setupMailStorageFolderForEmailID($new_email_id)) 
        {
            $body_html_filename = $body_html ? "{$mailstore_folder}/{$new_email_id}.html" : '';
            $body_text_filename = $body_text ? "{$mailstore_folder}/{$new_email_id}.txt" : '';
            $store_html_ok = $store_txt_ok = TRUE;
            if ($body_html_filename)
                $store_html_ok = file_put_contents($body_html_filename, $body_html);
            if ($body_text_filename)
                $store_txt_ok = file_put_contents($body_text_filename, $body_text);
            if ($store_html_ok === FALSE || $store_txt_ok === FALSE)
                die('Fatal error: Email storage system malfunction');
            
            return $new_email_id;
        }
        
        return FALSE;
    }
    
}
