<?php
namespace App;
/**
 * Newsletters - Main Class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
class Newsletters_SubscribePages extends \SCHLIX\cmsApplication_List {

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct("Newsletters Subscribe Pages", 'scxnl_subscribe_pages');
        // you can modify these to suit your needs
        $this->disable_frontend_runtime = true;
        $this->has_versioning = true;
    }
            
    /**
     * View Main Page
     * @param int $pg
     * @return boolean
     */
    //_______________________________________________________________________________________________________________//
    public function viewMainPage($pg = 1) {
            
    }
    

    /**
     * Get custom field associations
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $id
     * @param bool $use_complete_field_name
     * @return array
     */
    public function getCustomFieldPageAssocation($id)
    {
        global $SystemDB;
        
        $id = (int) $id;
        //$sql = "SELECT * FROM scxnl_subscribe_pages_fieldnames WHERE subscribe_page_id = {$id} ORDER BY sort_order";
        $sql = "SELECT scxnl_customfields.id, scxnl_customfields.title, scxnl_customfields.virtual_filename, default_value, value_choices, field_type, scxnl_subscribe_pages_fieldnames.* FROM scxnl_subscribe_pages_fieldnames INNER JOIN  scxnl_customfields ON scxnl_customfields.id = scxnl_subscribe_pages_fieldnames.fieldname_id WHERE subscribe_page_id = {$id} ORDER BY scxnl_subscribe_pages_fieldnames.sort_order";
        $data = $SystemDB->getQueryResultArray($sql);
        $result = [];
        if ($data)
        {
            foreach ($data as $dt)
            {
                $result[$dt['fieldname_id']] = $dt;
            }
        }
        return $result;
    }
       

    /**
     * Return subscribe page by I
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $id
     * @return array
     */
    public function getSubscribePageByID ($id)
    {
        global $SystemDB;
        
        return $SystemDB->getQueryResultSingleRow("SELECT * FROM {$this->table_items} WHERE id = :id",['id'=> $id]);
    }
    /**
     * Return subscribe page by GUID
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $guid
     * @return array
     */
    public function getSubscribePageByGUID ($guid)
    {
        global $SystemDB;
        
        return $SystemDB->getQueryResultSingleRow("SELECT * FROM {$this->table_items} WHERE guid = :guid",['guid'=> $guid]);
    } 
    
    /**
     * Get a list of active subscribe pages
     * @param string $virtual_filename
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @return array
     */
    public function getDefaultSubscribePage()
    {
        global $SystemDB;
        
        return $SystemDB->getQueryResultSingleRow("SELECT * FROM {$this->table_items} WHERE status > 0 AND is_default = 1");
    }
    
    /**
     * Given subscribe page's GUID, return the custom fields associated with it
     * TODO: optimize
     * @param string $guid
     * @return string
     */
    public function getCustomFieldFromPageGUID($guid)
    {
        $sub_page = $this->getSubscribePageByGUID($guid);
        if ($sub_page)
        {
            return $this->getCustomFieldPageAssocation($sub_page['id']);
        }
    }
            
    /**
     * Get a list of active subscribe pages
     * @param string $virtual_filename
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @return array
     */
    public function getSubscribePageByVirtualFileName($virtual_filename)
    {
        global $SystemDB;
        
        return $SystemDB->getQueryResultSingleRow("SELECT * FROM {$this->table_items} WHERE status > 0 AND virtual_filename = :vfn", ['vfn' => $virtual_filename]);
    }    
    /**
     * Validates save item. If there's an error, it will return an array
     * with one or more error string, otherwise it will return a boolean true
     * @global \App\Users $CurrentUser
     * @param array $datavalues
     * @return bool|array String array
     */
    public function getValidationErrorListBeforeSaveItem($datavalues)
    {
        $parent_error_list = parent::getValidationErrorListBeforeSaveItem($datavalues);
        $error_list = array();
        // You can put custom item save validation here
        // e.g. 
        // if (str_contains($datavalues['title'],'a'))
        // $error_list[] = ___('Title may not contain letter a');
        return array_merge($parent_error_list, $error_list);
    }

    /**
     * Before save item
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues) {
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        // You can customize the data values before it's saved here
        // e.g.
        // $datavalues['title'] = real_strip_tags($datavalues['title']);
        //
        return $datavalues;
    }
    /**
     * Do something after item has been saved
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    public function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval) 
    {
        
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        if ($retval['status'] == SAVE_OK)
        {
            
            // set the group
            $this->setItemCustomFields($retval['id'], array_values($original_datavalues['cf_id']),
                    $original_datavalues['cf_required'],
                    $original_datavalues['cf_sort_order']
                    );
        }
    }

    /**
     * Set the item to the specified category if the state is true, otherwise the 
     * item will be removed from the category. The content of cid_array must be
     * an array of a positive number greater than 1
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @global \SCHLIX\cmsLogger $SystemLog
     * @param int $id
     * @param array $cf_id_array
     * @param bool $state
     * @return boolean
     */
    public function setItemCustomFields($id, $cf_id_array, $cf_required_array, $cf_sort_order_array)
    {
        global $SystemDB, $SystemLog;
        $sqls = array();

        $id = (int) $id;
        if (!is_array($cf_id_array) || $id == 0)
            return false;
        $item = $this->getItemByID($id);  
        $custom_fields_app = $this->createSubClassInstance('CustomFields'); 
        $existing_page_assoc = $this->getCustomFieldPageAssocation($id);
        if (is_array($existing_page_assoc))
        {
            $existing_cid_array = array_column($existing_page_assoc,'fieldname_id');
            sort($existing_cid_array);
            sort($cf_id_array);
        }
        
        
        if ($item) {
            $cid_array_to_keep = array(); // separate it in case there's bad data with cid = 0 or less
            foreach ($cf_id_array as $cid) {
                $cid = (int) $cid;
                if ($cid > 0)
                {
                    $custom_field = $custom_fields_app->getItemByID($cid);
                    if ($custom_field)
                        $cid_array_to_keep[] = $cid;
                    
                    $required = (int) $cf_required_array[$cid] ? 1 : 0;
                    $sort_order = (int) $cf_sort_order_array[$cid];
                    
                    if (!in_array($cid, $existing_cid_array)) {
                        if ($custom_field)
                        {
                            $sqls[] = "INSERT IGNORE INTO scxnl_subscribe_pages_fieldnames (`subscribe_page_id`,`fieldname_id`,`sort_order`,`required`,`date_created`) VALUES ({$id}, {$cid}, {$sort_order},{$required}, NOW())";
                        } else {
                            $SystemLog->error("Trying to set a custom field name to a non-existing subscribe page ID# {$cid}", $this->app_name);
                        }
                    } else {
                        if ($custom_field)
                        {
                             $sqls[] = "UPDATE scxnl_subscribe_pages_fieldnames SET `sort_order` = {$sort_order}, `required` = {$required}, date_modified = NOW() WHERE `subscribe_page_id` = {$id} AND `fieldname_id` = {$cid}";
                        }else {
                            $SystemLog->error("Trying to update a custom field name to a non-existing subscribe page ID# {$cid}", $this->app_name);
                        }
                    
                    }
                }
            }
            
            if (count($sqls) > 0 || count($cid_array_to_keep) > 0) {
                // Only do this if the categories & the item have been verified
                $cid_str_to_keep = implode(',', $cid_array_to_keep);
                $sql = "DELETE FROM scxnl_subscribe_pages_fieldnames WHERE subscribe_page_id = {$id}";
                if ($cid_str_to_keep) // just in case there's still bad data ... 
                {
                    $sql.= " AND fieldname_id NOT IN ({$cid_str_to_keep})";
                }
                $SystemDB->query($sql);                    
                if ($sqls)
                {
                    foreach ($sqls as $sql) {
                        $SystemDB->query($sql);
                    }
                }
                return true;
            } else
            {
                $SystemLog->error('Failed to perform setItemCustomFields', $this->app_name);
            }
        } else {
            $SystemLog->error('Trying to set custom fields to a non-existing subscribe pages', $this->app_name);
        }
        return false;
    }
    
    /**
     * Run Command
     * @param array $command
     * @return boolean
     */
    public function Run($command) {
        //$this->insertTest();
        switch ($command['action']) {
            case 'main': $this->viewMainPage($command['pg']);
                return true;
                break;
            default: return parent::Run($command);
        }
    }



}

?>