<?php
namespace App;

/**
 * Newsletters - Admin class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 *
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @version 1.0
 * @package newsletters
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
class Newsletters_Subscribers_Admin extends \SCHLIX\cmsAdmin_List {

    /**
     * The app
     * @var \App\Newsletters_Subscribers 
     */
    protected $app;
    
    protected $extra_custom_fields;
    //protected $extra_custom_field_names = [];
    
    //_________________________________________________________________________//
    public function __construct() {
        parent::__construct(null, null);
        $this->setFieldsToBeListed();
    }
    
    protected function setFieldsToBeListed()
    {
        $custom_fields = $this->app->getAllCustomFieldNames(true);
        $main_fields = ['id', 'email_address', 'date_created',  'status'];
        $this->extra_custom_fields = $custom_fields;
        foreach ($custom_fields as $cf)
        {
            if ($cf['status'] > 0)
            {
                $main_fields[] = $cf['virtual_filename']; // show table name in ajax listing
                //$this->extra_custom_field_names[] = $cf['virtual_filename'];
            }
        }
        $this->setItemFieldNamesForAjaxListing( $main_fields);
        
    }
    
    protected function isThereAnyCustomFields()
    {
        global $SystemDB;
        
        $data = $SystemDB->getQueryResultSingleRow("SELECT COUNT(id) AS total_count FROM scxnl_customfields");
        return $data['total_count'] > 0;
    }
    //_________________________________________________________________________//
    /**
     * Get all items - with custom fields
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $start
     * @param int $end
     * @param string $sortby
     * @param string $sortdirection
     * @return array
     */
    public function ajaxGetAllItems($start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC')
    {
        global $SystemDB;
//return parent::ajaxGetAllItems ($start, $end, $sortby, $sortdirection);
        if ($this->isThereAnyCustomFields())
        {
            if (intval($start) > intval($end))
                return ajax_reply(400, 'Invalid Request');
            $sortby = $SystemDB->escapeString($sortby);
            $items_per_page = min(HARDCODE_MAX_ROWLIMIT, $end - $start);
            //$fields = implode(',', quote_array_of_field_names_for_query($this->getItemFieldNamesForAjaxListing()));
            $total_item_count = $this->app->getTotalItemCount('');
            $result_array_files = $this->app->getAllSubscribersWithCustomFields([],'', 'id DESC', $items_per_page, $start);
            //$result_array_files = $this->app->getAllItems($fields, '', $start, $end, $sortby, $sortdirection, false);
            return ajax_datasource_reply(200, $result_array_files, $start, $end, $items_per_page, $total_item_count, $sortby, $sortdirection);
        } else
            return parent::ajaxGetAllItems ($start, $end, $sortby, $sortdirection);
    }
    
    public function getSubscriberCountArray()
    {
        global $SystemDB;
        
        $info_active = $SystemDB->getQueryResultSingleRow("SELECT COUNT(*) AS total_count FROM {$this->table_items} WHERE status = ".NEWSLETTER_SUBSCRIBER_ACTIVE);
        $info_pending = $SystemDB->getQueryResultSingleRow("SELECT COUNT(*) AS total_count FROM {$this->table_items} WHERE status = ".NEWSLETTER_SUBSCRIBER_NEW);
        $info_unsubscribed = $SystemDB->getQueryResultSingleRow("SELECT COUNT(*) AS total_count FROM {$this->table_items} WHERE status = ".NEWSLETTER_SUBSCRIBER_UNSUBSCRIBED);
        
        $info = ['active' => $info_active['total_count'],
                 'unsubscribed' => $info_unsubscribed['total_count'],
                 'pending' => $info_pending['total_count']
                ];
        return $info;
    }
    //_________________________________________________________________________//
    public function ajaxSearchByCriteria($criteria = '', $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC')
    {
        global $SystemDB;
        //$keyword = strtolower($keyword);
        //$cleankeyword = sanitize_string("%{$keyword}%");
        //$criteria = "LOWER(email_address) LIKE {$cleankeyword}";
        if (empty($criteria))
            $criteria = '1';
        if ($this->isThereAnyCustomFields())
        {
            if (intval($start) > intval($end))
                return ajax_reply(400, 'Invalid Request');
            $sortby = $SystemDB->escapeString($sortby);
            $items_per_page = min(HARDCODE_MAX_ROWLIMIT, $end - $start);
            //$fields = implode(',', quote_array_of_field_names_for_query($this->getItemFieldNamesForAjaxListing()));
            $total_item_count = $this->app->getTotalItemSearchCountWithCustomFields($criteria);
            $result_array_files = $this->app->searchAllSubscribersWithCustomFields([],$criteria, 'id DESC', $items_per_page, $start);
            //$result_array_files = $this->app->getAllItems($fields, '', $start, $end, $sortby, $sortdirection, false);
            return ajax_datasource_reply(200, $result_array_files, $start, $end, $items_per_page, $total_item_count, $sortby, $sortdirection);
        } 
    }
    
    //_________________________________________________________________________//
    public function ajaxSearchObjects($keyword = '', $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC')
    {
        global $SystemDB;
        $keyword = strtolower($keyword);
        $cleankeyword = sanitize_string("%{$keyword}%");
        $criteria = "LOWER(email_address) LIKE {$cleankeyword}";
        $extra_criteria = [];
                /*if ($this->extra_custom_fields)
        {
            foreach ($this->extra_custom_fields as $cf)
            {
                $extra_criteria[] = "(fieldname_id = {$cf['id']} AND field_value LIKE {$cleankeyword})";
            }
            $str_extra_criteria = implode(' OR ', $extra_criteria);
            $criteria.= ' OR '.$str_extra_criteria;
        }*/

      
        if ($this->extra_custom_fields)
        {
            foreach ($this->extra_custom_fields as $cf)
            {
                $extra_criteria[] = "LOWER({$cf['virtual_filename']}) LIKE {$cleankeyword}";
            }
            $str_extra_criteria = implode(' OR ', $extra_criteria);
            $criteria.= ' OR '.$str_extra_criteria;
        }
        
        
        if ($this->isThereAnyCustomFields())
        {
            if (intval($start) > intval($end))
                return ajax_reply(400, 'Invalid Request');
            $sortby = $SystemDB->escapeString($sortby);
            $items_per_page = min(HARDCODE_MAX_ROWLIMIT, $end - $start);
            //$fields = implode(',', quote_array_of_field_names_for_query($this->getItemFieldNamesForAjaxListing()));
            $total_item_count = $this->app->getTotalItemSearchCountWithCustomFields($criteria);
            $result_array_files = $this->app->searchAllSubscribersWithCustomFields([],$criteria, 'id DESC', $items_per_page, $start);
            //$result_array_files = $this->app->getAllItems($fields, '', $start, $end, $sortby, $sortdirection, false);
            return ajax_datasource_reply(200, $result_array_files, $start, $end, $items_per_page, $total_item_count, $sortby, $sortdirection);
        } else
            return parent::ajaxSearchObjects ($keyword, $start, $end, $sortby, $sortdirection);
        
    }

    /**
     * Validation
     * @param type $datavalues
     */
    public function onGetAdminValidationErrorListBeforeSaveItem($datavalues) {

        $error_list = parent::onGetAdminValidationErrorListBeforeSaveItem($datavalues);
        if (filter_var($datavalues['email_address'], FILTER_VALIDATE_EMAIL) === false) {
            $error_list[] = ___('Invalid email address');
        }        
        $other_subscriber = $this->app->getSubscriberByRealEmailAddress($datavalues['email_address']);
        if ($other_subscriber && ($datavalues['id'] != $other_subscriber['id']))
            $error_list[] = ___('Email address already in use by another subscriber');
        if ($datavalues['id'] == 'new')
        {
            $subscribe_to = fpost_array_int('subscribe_to');
            if (empty($subscribe_to))
                $error_list[] = ___('You must choose at least one newsletter');
            $template_id = fpost_int('send_email_template_id');
            if ($template_id == 0)
                $error_list[] = ___('Invalid email template ID');
            
        }
        return $error_list;
    }
    /**
     * Modify data before save item
     * @global \App\Users $CurrentUser
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues) {
        global $CurrentUser;
        
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        
        $current_admin_id = $CurrentUser->getCurrentUserID();
        $field_admin_id = ($datavalues['id'] == 'new') ? 'created_by_id' : 'modified_by_id';
        $datavalues[$field_admin_id] = intval($datavalues[$field_admin_id]);
        if (!($datavalues[$field_admin_id] > 0 && $datavalues[$field_admin_id] != $current_admin_id))
            $datavalues[$field_admin_id] = $current_admin_id;
        if (empty($datavalues['real_email_address']))
            $datavalues['real_email_address'] = \App\Newsletters::getRealEmailAddress ($datavalues['email_address']);
        return $datavalues;
    }

    /**
     * Before save category
     * @param array $datavalues
     */
    public function onModifyDataBeforeSaveCategory($datavalues) {
        
        global $CurrentUser;

        $external_images_save_path = '/media/images/clippings/';        
        $datavalues = parent::onModifyDataBeforeSaveCategory($datavalues);
        $current_admin_id = $CurrentUser->getCurrentUserID();

        $field_admin_id = ($datavalues[$this->app->getFieldCategoryID()]) ? 'created_by_id' : 'modified_by_id';

        if (!($datavalues[$field_admin_id] > 0 && $datavalues[$field_admin_id] != $current_admin_id))
            $datavalues[$field_admin_id] = $current_admin_id;
        $datavalues['virtual_filename'] = convert_into_sef_friendly_title($datavalues['virtual_filename']);
        if (empty($datavalues['meta_description']))
            $datavalues['meta_description'] = $datavalues['title'];
        if (empty($datavalues['virtual_filename']) || ($datavalues['virtual_filename'] == 'index'))
            $datavalues['virtual_filename'] = 'cat' . $datavalues['cid'];
        
        $datavalues['virtual_filename'] = $this->app->preventDuplicateValueInCategory('virtual_filename', $datavalues['virtual_filename'], $datavalues[$this->field_category_id]);
        
        if ($datavalues['permission_read_everyone'])
            $datavalues['permission_read'] = 'everyone';
        $datavalues['permission_read'] = serialize($datavalues['permission_read']);
        $datavalues['permission_write'] = serialize($datavalues['permission_write']);
        $datavalues['options'] = serialize($datavalues['options']);
        $datavalues['summary'] = convert_pasted_png_images_from_html_text($datavalues['summary'],  CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
        $datavalues['description'] = convert_pasted_png_images_from_html_text($datavalues['description'], CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
        if ($this->app->getConfig('bool_convert_external_images')) {
            $datavalues['summary'] = move_static_external_images_from_html_text($datavalues['summary'], CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
            $datavalues['description'] = move_static_external_images_from_html_text($datavalues['description'], CURRENT_SUBSITE_PATH . $external_images_save_path, CURRENT_SUBSITE_URL_PATH . $external_images_save_path);
        }
        return $datavalues;        
    } 
    /**
     * You can customize the response schema field here
     * @param array $response_schema
     * @return array
     */
    public function modifyItemResponseSchemaFields(array $response_schema)
    {
        $response_schema = parent::modifyItemResponseSchemaFields($response_schema);
        
        $custom_fields = $this->app->getAllCustomFieldNames(true);
        foreach ($custom_fields as $cf)
        {
            if ($cf['status'] > 0)
            {
                $response_schema[] = $cf['virtual_filename']; // show table name in ajax listing
            }
        }
        
        return $response_schema;
    }

    /**
     * @global \App\Users $CurrentUser
     * @global \App\EmailQueue $SystemMail
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    public function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval) 
    {
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        if ($retval['status'] == SAVE_OK)
        {
            if ($original_datavalues['unsubscribe_from_all'])
            {
                $newsletter_app = new \App\Newsletters();
                $newsletter_app->completeUnsubscribe($retval['id'], 'By admin', NULL, true);
                $this->app->createSubClassInstance('EventLog')->recordAdminActivity("Admin ordered unsubscribe for subscriber ID #{$retval['id']}");
            } 
            if ($original_datavalues['id'] == 'new')
            {
                $newsletter_app = new \App\Newsletters();
                $email_templ = $this->app->createSubClassInstance('EmailTemplates')->getItemByID($original_datavalues['send_email_template_id']);
                $newsletter_app->createActivationRequest($retval['id'], NULL, $original_datavalues['subscribe_to'], $email_templ['virtual_filename'],___('Manually created by admin'));
                        
            }
            
        }
    }    
    
    protected function getTotalRelatedRecordForSubscriberByID($subscriber_id, $table)
    {
        global $SystemDB;

        $subscriber_id = (int) $subscriber_id;
        $sql = "SELECT count(id) AS total_count FROM `{$table}` WHERE subscriber_id = {$subscriber_id}";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result['total_count'];
    }

    public function ajaxGetRelatedRecordsForSubscriberID($table, $id,  $start, $end, $sortby, $sortdirection) 
    {
        global $SystemDB;
         $user_id = intval($id);
        $sortby = alpha_numeric_with_dash_underscore($sortby);
        $items_per_page = min(HARDCODE_MAX_ROWLIMIT, $end - $start);

        $total_item_count = $this->getTotalRelatedRecordForSubscriberByID($user_id, $table);

        $sql = $SystemDB->generateSelectSQLStatement($table, '*', "subscriber_id = {$user_id}", $start, $end, $sortby, $sortdirection, true, SCHLIX_SQL_ENFORCE_ROW_LIMIT);

        $result = $SystemDB->getQueryResultArray($sql);
        
        return ajax_datasource_reply(200, $result, $start, $end, $items_per_page, $total_item_count, $sortby, $sortdirection);
    }
    
    public function ajaxGetSubscribersHistorySchema() {
        $subscriber_history_admin = new Newsletters_SubscriberLog_Admin();
        return $subscriber_history_admin->ajaxGetAllDataResponseSchema();
    } 
    
    public function ajaxGetSubscribeRequestsSchema()
    {
        $subscriber_requests_admin = new Newsletters_SubscribeRequests_Admin();
        return $subscriber_requests_admin->ajaxGetAllDataResponseSchema();        
    }
    
    
    public function ajaxGetUnsubscribeRequestsSchema()
    {
        $subscriber_requests_admin = new Newsletters_UnsubscribeRequests_Admin();
        return $subscriber_requests_admin->ajaxGetAllDataResponseSchema();        
    }

    public function ajaxGetEmailChangeRequestsSchema()
    {
        $subscriber_requests_admin = new Newsletters_EmailChangeRequests_Admin();
        return $subscriber_requests_admin->ajaxGetAllDataResponseSchema();        
    }
    
    public function ajaxGetSubscribersSentMailsSchema()
    {
        $subscriber_requests_admin = new Newsletters_Queues_Admin();
        return $subscriber_requests_admin->ajaxGetAllDataResponseSchema();        
    }
    
    /**
     * Process the TSV import file
     * @param string $filename
     * @param int $email_template_id
     * @param array $newsletter_ids
     * @param string $separator
     * @return type
     */
    protected function processCSVImportFile($filename, $email_template_id, $newsletter_ids, $separator = "\t")
    {
        set_time_limit(0);
        $eventlog = new \App\Newsletters_EventLog();
        $newsletter_app = new \App\Newsletters();
        $newsletter_email_template_app = new \App\Newsletters_EmailTemplates();
        
        $newsletters_to_subscribe = $newsletter_app->sanitizeNewsletterIDsArray($newsletter_ids);
        $error_list = array();
        if (!file_exists($filename))
            $error_list[] = ___('File cannot be found');
        
        
        $row = 1;
        $firstrow_fields = [];
        if (($handle = fopen($filename, "r")) !== FALSE) {
            $firstrow_fields = fgetcsv($handle, 10000, $separator);
            for ($i = 0, $count = count($firstrow_fields); $i < $count; $i++)
            {
                $firstrow_fields[$i] = trim ($firstrow_fields[$i]);
            }
            $data = [];
            if ($firstrow_fields !== FALSE)
            {
                while (($line = fgetcsv($handle, 10000, $separator)) !== FALSE) {
                    $count = count ($line);
                    $nr = null;
                    $nr = [];
                    for ($i = 0; $i < $count; $i++)
                        $nr[$firstrow_fields[$i]] = trim($line[$i]);
                    if ($nr)
                        $data[] = $nr;
                    $row++;                    
                }
            } else $error_list[] = ___('Invalid TSV header');
            fclose($handle);
        }
        if (!in_array('email_address', $firstrow_fields))
        {
            $error_list[] = ___('Row header does not contain email_address field');
        }
        if (empty($newsletters_to_subscribe))
        {
            $error_list[] = ___('Newsletters to subscribe is empty');
        }
        $email_template = $newsletter_email_template_app->getItemByID($email_template_id);
        if (!$email_template)
        {
            $error_list[] = ___('Email template is unspecified');
        }
        if (empty($error_list))
        {
            $str_fields = implode(',', $firstrow_fields);
            $eventlog->recordAdminActivity("IMPORT - Begin to import {$row} records from {$filename}. Specified custom fields: [{$str_fields}]", 'import-tsv','info');
            $default_subscribe_page =  $this->app->createSubClassInstance('SubscribePages')->getDefaultSubscribePage();
            $initial_subscribe_page_id = $default_subscribe_page['id'];
            $i = 1;
            foreach ($data as $info)
            {
                $real_email_address = \App\Newsletters::getRealEmailAddress($info['email_address']);
                $existing_subscriber = $newsletter_app->getSubscriberByRealEmailAddress($real_email_address);
                if ($existing_subscriber)
                {
                    $eventlog->recordAdminActivity("IMPORT - Skipped row #{$i} - {$info['email_address']} - existing subscriber ID# {$existing_subscriber['id']}", 'import-tsv', 'warning');
                } else
                {
                    if (\App\Newsletters_Blacklist::isEmailAddressBlacklisted($info['email_address']))
                    {
                        $eventlog->recordAdminActivity("IMPORT - Blacklisted: {$info['email_address']} - ignored", 'import-tsv', 'info');
                    } else
                    {
                    $newsletter_app->setRequestEmailDeliveryInstantaneous(false); // do not deliver instantly, put into queue
                    $newsletter_app->addUnconfirmedNewSubscriber($info['email_address'], $info,$initial_subscribe_page_id, $newsletters_to_subscribe, $email_template['virtual_filename'],___('Imported'));
                    $eventlog->recordAdminActivity("IMPORT - Newly imported address: {$info['email_address']} - queued [{$email_template['id']}] email confirmation", 'import-tsv', 'info');
                    }
                    
                }
                $i++;
            }
            $row_count = $row > 0 ? $row-1 : 0;
            $eventlog->recordAdminActivity("IMPORT - End of import {$row_count} records from {$filename}", 'import-tsv','info');
        }
        $result = "{$row_count} records have been processed from {$filename}. Please click the Refresh button";
        return ['status' => empty($error_list), 'result' => $result, 'errors' => $error_list];
    }
    //_________________________________________________________________________// 	
    public function ajaxImportCSV() {

        $upload_failed = false;
        
        $error_list = $this->app->initializeDataDirectories();
        if ($error_list)
        {
            return ajax_reply(300,___('Error while trying to create data directories. Maybe you can check the permission on the /web folder'));
        }
        $uploaded_file = $_FILES['filedata'];
        $base_filename = strtolower(convert_into_sef_friendly_title(strip_tags(basename(pathinfo($uploaded_file['name'], PATHINFO_FILENAME)))) .date('Y-m-d_H-i-s'). '.' . pathinfo($uploaded_file['name'], PATHINFO_EXTENSION));
        $full_filename = $this->app->getDataFileFullPath('import', $base_filename);
        $move_result = move_uploaded_file($uploaded_file['tmp_name'], $full_filename);
        if ($move_result) {
            $upload_failed = false;
            $separator = fpost_int('separator') == 2 ? "," : "\t";
            $result = $this->processCSVImportFile($full_filename, fpost_int('send_email_template_id'), fpost_array_int('subscribe_to'),$separator);
            return ($result['status']) ? ajax_reply(200, $result['result']) : ajax_reply(500, implode(',',$result['errors']));
        }
        return ajax_reply(500, 'Upload Error - Cannot Upload');
    }
    
    
    public function downloadFile($id)
    {
        
        $item = $this->app->getITemByID($id);
        if ($item)
        {
            $filename_only = rawurlencode($item['extra_attachment']);
            $full_expected_file = $this->app->getDataFileFullPath('attachments', $item['guid']);
            if (file_exists($full_expected_file))
            {
                header('Content-Description: File Transfer');
                header('Content-Type: application/octet-stream');
                header('Content-Disposition: attachment; filename='.$filename_only);
                header('Content-Transfer-Encoding: binary');
                header('Expires: 0');
                header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
                header('Pragma: public');
                header('Content-Length: ' . filesize($full_expected_file));
                ob_clean();
                flush();
                readfile($full_expected_file);
                exit;               
            } else
            {
                die ("Cannot download {$id} - file not found in storage path");
            }
        } else
        {
            die("Cannot download file - not found");
        }
        
    }
    //_________________________________________________________________________//
    public function Run() {
        switch (fget_alphanumeric('action')) {
            case 'getitemsresponseschema' :
                return $this->getItemsResponseSchema();
                break; 
            case 'downloadattachment':
                return $this->downloadFile(fget_int('id'));
            case 'getemailchangerequestsresponseschema': 
                return ajax_echo($this->ajaxGetEmailChangeRequestsSchema());
                break;             
            case 'getunsubscriberequestsresponseschema':
                return ajax_echo($this->ajaxGetUnsubscribeRequestsSchema());
                break;                
            
            case 'getsubscriberequestsresponseschema':
                return ajax_echo($this->ajaxGetSubscribeRequestsSchema());
                break;                
            case 'getsubscriberhistoryresponseschema': 
                return ajax_echo($this->ajaxGetSubscribersHistorySchema());
                break;
            case 'getsubscribersentmailsresponseschema': 
                return ajax_echo($this->ajaxGetSubscribersSentMailsSchema());
                break;
            
            case 'gethistory':
                return ajax_echo($this->ajaxGetRelatedRecordsForSubscriberID('scxnl_log_subscribers',fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
            case 'getsentmails':
                return ajax_echo($this->ajaxGetRelatedRecordsForSubscriberID('scxnl_queues',fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
            
            
            case 'getsubscriberequests':
                return ajax_echo($this->ajaxGetRelatedRecordsForSubscriberID('scxnl_rq_subscribe',fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
            case 'getunsubscriberequests':
                return ajax_echo($this->ajaxGetRelatedRecordsForSubscriberID('scxnl_rq_unsubscribe',fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
            case 'getemailchangerequests':
                return ajax_echo($this->ajaxGetRelatedRecordsForSubscriberID('scxnl_rq_email_change',fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
            case 'importtsv':
                return ajax_echo ($this->ajaxImportCSV());
                break;
            
            
            default:return parent::Run();
        }
    }

    //_________________________________________________________________________//
    public function Uninstall() {
        return false;
    }

}