/* 
 * Copyright (C) 2015 SCHLIX WEB INC.
 *
 * This software is licensed under SHARED SOURCE; you can redistribute
 * but forking or creating a derivative is prohibited 
 *
 * Please read the LICENSE.html for details
 */

/**
 * View Controller class
 */
SCHLIX.CMS.NewslettersSubscribersAdmin = class extends SCHLIX.CMS.BaseController  {  
    /**
     * Constructor
     */
    constructor ()
    {
        super("newsletters.subscribers");
    };

    /**
     * Datatable row format: item status
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @returns {undefined}
     */
    static formatDataTableCell_SubscriberStatus (elCell, oRecord, oColumn, oData) {
        var button_name = '';
        var color = '';
        var text = '';
        switch (oData)
        {
            case -4:
                button_name = "question-circle";
                color = 'darkorange';
                text = 'invalidated';
                break;
            case -3:
                button_name = "exclamation-circle";
                color = 'red';
                text = 'blacklisted';
                break;
            case -2:
                button_name = "question-circle";
                color = 'darkorange';
                text = 'new';
                break;
                // more ...            
            case 1:
                button_name = "check-circle";
                color = 'green';
                text = 'active';
                break;
            case 0:
                button_name = "times-circle";
                color = 'red';
                text = 'unsubscribed';
                break;
            default:
                button_name = "exclamation-triangle";
                color = 'red';
                text = 'unknown';
                break;
        }
        elCell.innerHTML = '<i class="fa fa-' + button_name + '" style="color:' + color + '"></i> ' + text;
    }; // end func 

    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_ViewLog (elCell, oRecord, oColumn, oData) {

        var odata_id = oRecord.getData("id");
        if (oData === undefined)
        {
            elCell.innerHTML = "Invalid column!";
            return;
        }
        //  alert(this.parentControl.getTableFieldNameByRole('title'));
        var itemTitle = oData;// oRecord.getData("title");
        var itemLink = '';
        if (itemTitle && itemTitle.length > 60)
            itemTitle = itemTitle.substr(0, 60) + '...';
        if (!itemTitle) itemTitle = 'N/A';
        var app_name = this.parentControl.app_name;

        var the_id = oRecord.getData("id");
        var theValue = 'i' + the_id;
        if (itemTitle == '')
            itemTitle = '(Untitled)';
        //itemLink = _schlix_app_controller + oColumn.app_name + '&action=edititem&id=' + oRecord.getData("id");
        var the_icon;
        switch (oRecord.getData('type'))
        {
            case 'error' : 
                the_icon = 'fa-times-circle';
                break;
            case 'warn' : 
                the_icon = 'fa-exclamation-triangle';
                break;
                    
            case 'info' :
            default: the_icon = 'fa-info-circle';
        }
        var icon = "<i class =\"fa " + the_icon + " fa-2x\"></i>";
        // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            
        elCell.innerHTML = '<a class="open-viewlog" id="' + app_name + "-lnk-" + theValue + '" data-log-id="' + theValue + '" href="javascript:void(0)" data-log-message="' + oData + '">' + " " + icon + " " + itemTitle + '</a>';
    };
 
    
    /**
     * Datatable row format: item status
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @returns {undefined}
     */
    static formatDataTableCell_EmailChangeRequestStatus (elCell, oRecord, oColumn, oData) {
        var button_name = '';
        var color = '';
        var text = '';
        switch (oData)
        {
            case 0:
                button_name = "times-circle";
                color = 'darkorange';
                text = 'invalidated';
                break;
            case -4:
                button_name = "exclamation-circle";
                color = 'red';
                text = 'denied';
                break;
            
            
            case -2:
                button_name = "question-circle";
                color = 'darkorange';
                text = 'new';
                break;
                // more ...            
            case 1:
                button_name = "check-circle";
                color = 'green';
                text = 'done';
                break; 
                
                // more ...
            default:
                button_name = "exclamation-triangle";
                color = 'red';
                text = 'unknown';
                break;
        }
        elCell.innerHTML = '<i class="fa fa-' + button_name + '" style="color:' + color + '"></i> ' + text;
    }; // end func 

    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatHistoryDescription (elCell, oRecord, oColumn, oData) {

        var odata_id = oRecord.getData("id");
        if (oData === undefined)
        {
            elCell.innerHTML = "Invalid column!";
            return;
        }
        //  alert(this.parentControl.getTableFieldNameByRole('title'));
        var itemTitle = oData;// oRecord.getData("title");
        var itemLink = '';
        if (itemTitle.length > 60)
            itemTitle = itemTitle.substr(0, 60) + '...';
        var app_name = this.parentControl.app_name;

        var the_id = oRecord.getData("id");
        var theValue = 'i' + the_id;
        if (itemTitle == '')
            itemTitle = '(Untitled)';
        //itemLink = _schlix_app_controller + oColumn.app_name + '&action=edititem&id=' + oRecord.getData("id");
        var the_icon;
        switch (oRecord.getData('type'))
        {
            case 'error' : 
                the_icon = 'fa-times-circle';
                break;
            case 'warn' : 
                the_icon = 'fa-exclamation-triangle';
                break;
                    
            case 'info' :
            default: the_icon = 'fa-info-circle';
        }
        var icon = "<i class =\"fa " + the_icon + " fa-2x\"></i>";
        // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            
        itemLink = this.parentControl.schlix_application_url + 'action=edititem&id=' + the_id;
        var hist = oData ? SCHLIX.Util.escapeHTML(oData)  : '';
        elCell.innerHTML = '<a class="open-viewlog" id="' + app_name + "-lnk-" + theValue + '" data-log-id="' + theValue + '" href="javascript:void(0)" data-log-message="' + hist + '">' + " " + icon + " " + itemTitle + '</a>';
    };
    
    

    /**
     * Datatable row format: item status
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @returns {undefined}
     */
    static formatDataTableCell_SubscribeRequestStatus (elCell, oRecord, oColumn, oData) {
        var button_name = '';
        var color = '';
        var text = '';
        switch (oData)
        {
            case -4:
                button_name = "question-circle";
                color = 'darkorange';
                text = 'invalidated';
                break;
            case -3:
                button_name = "exclamation-circle";
                color = 'red';
                text = 'blacklisted';
                break;
            
            
            case -2:
                button_name = "question-circle";
                color = 'darkorange';
                text = 'new';
                break;
                // more ...            
            case 1:
                button_name = "check-circle";
                color = 'green';
                text = 'active';
                break;
            case 0:
                button_name = "check-circle";
                color = 'green';
                text = 'unsubscribed';
                break;
                // more ...
                
                // more ...
            default:
                button_name = "exclamation-triangle";
                color = 'red';
                text = 'unknown';
                break;
        }
        elCell.innerHTML = '<i class="fa fa-' + button_name + '" style="color:' + color + '"></i> ' + text;
    }; // end func 
 
    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_ViewID (elCell, oRecord, oColumn, oData) {

        var odata_id = oRecord.getData("id");
        if (oData === undefined)
        {
            elCell.innerHTML = "Invalid column!";
            return;
        }
        var app_name = this.parentControl.app_name;

        var the_id = oRecord.getData("id");
        var icon = "<i class =\"fa fa-envelope-open\"></i>";
        elCell.innerHTML = '<a class="open-viewrequest" href="javascript:void(0)"  id="' + app_name + "-lnk-" + the_id + '"' +
                'data-id="' + the_id + '"' + 
                'data-subscriber_id="' + oRecord.getData("subscriber_id") + '"' + 
                'data-date_created="' + oRecord.getData("date_created") + '"' + 
                'data-date_expiry="' + oRecord.getData("date_expiry") + '"' + 
                'data-date_activated="' + oRecord.getData("date_activated") + '"' + 
                'data-ip_address="' + oRecord.getData("ip_address") + '"' + 
                'data-user_agent="' + oRecord.getData("user_agent") + '"' + 
                'data-host_name="' + oRecord.getData("host_name") + '"' + 
                'data-confirmed_ip_address="' + oRecord.getData("confirmed_ip_address") + '"' + 
                'data-confirmed_user_agent="' + oRecord.getData("confirmed_user_agent") + '"' + 
                'data-confirmed_host_name="' + oRecord.getData("confirmed_host_name") + '"' + 
                'data-subscribe_page_id="' + oRecord.getData("subscribe_page_id") + '"' + 
                'data-request_newsletter_ids="' + oRecord.getData("request_newsletter_ids") + '"' + 
                'data-notes="' + SCHLIX.Util.escapeHTML(oRecord.getData("notes")) + '">' + 
                "#" + the_id + '</a>' ;
    };
    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_ViewMasked (elCell, oRecord, oColumn, oData) {

        var itemTitle = oData;
        var itemLink = '';
        if (itemTitle && itemTitle.length > 2)
            itemTitle = itemTitle[0] + itemTitle[1] + '*****';
        else
            itemTitle = '*';
        elCell.innerHTML = itemTitle;
    };
    

    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */

    static formatDataTableCell_ViewErrors  (elCell, oRecord, oColumn, oData) {

        //  alert(this.parentControl.getTableFieldNameByRole('title'));
        if (!oData || oData == 'false')
            oData = '';
        var txt = (oData.length > 50) ? oData.substr(0, 50) + '...' : oData;
            

        elCell.innerHTML = SCHLIX.Util.escapeHTML(txt);
    };
    
    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */

    static formatDataTableCell_ViewMail  (elCell, oRecord, oColumn, oData) {

        var odata_id = oRecord.getData("id");
        if (oData === undefined)
        {
            elCell.innerHTML = "Invalid column!";
            return;
        }
        //  alert(this.parentControl.getTableFieldNameByRole('title'));
        var itemTitle = oData;// oRecord.getData("title");
        var itemLink = site_httpbase;
        if (itemTitle.length > 40)
            itemTitle = itemTitle.substr(0, 40) + '...';
        var app_name = this.parentControl.app_name;

        var the_id = oRecord.getData("id");
        var theValue = 'i' + the_id;
        if (itemTitle == '')
            itemTitle = '(Untitled)';
        //itemLink = _schlix_app_controller + oColumn.app_name + '&action=edititem&id=' + oRecord.getData("id");
        var icon = "<i class =\"fa fa-file-text-o fa-2x\"></i>";
        // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            
        itemLink = site_httpbase+ this.parentControl.schlix_application_url + 'action=edititem&id=' + the_id;
        elCell.innerHTML = '<a class="open-viewmail" id="' + app_name + "-lnk-" + theValue + '"  href="javascript:void(0)" data-email-id="' + odata_id + '">' + " " + icon + " " + itemTitle + '</a>';
    };
    
    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_ViewEmail (elCell, oRecord, oColumn, oData) {

        var odata_id = oRecord.getData("id");
        if (oData === undefined)
        {
            elCell.innerHTML = "Invalid column!";
            return;
        }
        var itemTitle = oData;
        var itemLink = '';
        if (itemTitle.length > 40)
            itemTitle = itemTitle.substr(0, 40) + '...';
        var app_name = this.parentControl.app_name;

        var the_id = oRecord.getData("id");
        var theValue = 'i' + the_id;
        if (itemTitle == '')
            itemTitle = '(Untitled)';
        //itemLink = _schlix_app_controller + oColumn.app_name + '&action=edititem&id=' + oRecord.getData("id");
        var the_icon;
        switch (oRecord.getData('type'))
        {
            case 'error' : 
                the_icon = 'fa-times-circle';
                break;
            case 'warn' : 
                the_icon = 'fa-exclamation-triangle';
                break;
                    
            case 'info' :
            default: the_icon = 'fa-info-circle';
        }
        var icon = "<i class =\"fa fa-envelope-open\"></i>";
        // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            
        var __app_url  = this.parentControl.schlix_application_url.replace('.subscriberequests','.subscribers');
        var itemLink = site_httpbase + __app_url + 'action=edititem&id=' + oRecord.getData("subscriber_id");
        
        elCell.innerHTML = '<a class="open-viewmail" href="' + itemLink + '">' + " " + icon + " " + itemTitle + '</a>';
    };

    formatFileSize(size)
    {
        var result = size;
        if (size >= 1024 && size <= 1024*1024)
        {
            result = Math.round(size / 1024.0, 2) + ' Kb';
        } 
        else if (size > 1024*1024 && size <= 1024*1024*1024)
        {
            result = Math.round(size / (1024.0 * 1024.0), 2) + ' Mb';
        }
        else if (size > 1024*1024*1024)
        {
            result = Math.round(size / (1024.0*1024.0*1024.0), 2) + ' Gb';
        }
        return result;
    };    
////////////////////////////////////////////////////////////
    updateFileListToUpload ()
    {
        var input = document.getElementById('filedata');
        var ul = document.getElementById('filelist');
        var prefix_file_size = SCHLIX.Dom.get('filelist-prefix-totalsize');
        var display_file_size = SCHLIX.Dom.get('filelist-display-totalsize');
        var total_file_size_input = SCHLIX.Dom.get('__total_file_size');
        
        var total_size = 0;
        
        while (ul.hasChildNodes()) {
            ul.removeChild(ul.firstChild);
        }
        // You've selected input.files.length files
        for (var i = 0; i < input.files.length; i++) {
            // input.files[i] is a file object
            var li = document.createElement("li");
            li.innerHTML = '<i class="fa fa-file-text-o"></i> ' + input.files[i].name + ' (' + this.formatFileSize(input.files[i].size) + ')';
            total_size += input.files[i].size;            
            ul.appendChild(li);
        }
        total_file_size_input.value = total_size;
        prefix_file_size.innerHTML = "Total: ";
        display_file_size.innerHTML = this.formatFileSize(total_size);
        
    }; // end func

    
    onSubmitTSVUploadDialog (event)
    {
        var prefix_file_size = SCHLIX.Dom.get('filelist-prefix-totalsize');
        var upload_form = SCHLIX.Dom.get('frm_upload_file');
        var btn_upload = SCHLIX.Dom.get('btn-tsv-upload');
        var btn_cancel = SCHLIX.Dom.get('btn-tsv-cancel');
        var total_file_size = parseInt(SCHLIX.Dom.get('__total_file_size').value, 10);
        var max_file_size = parseInt(SCHLIX.Dom.get('__max_file_size').value, 10);
        //var chkselections = document.getElementsByName(this.selectionCheckBoxes);
        var total_newsletter_checked = 0;
        var total_emailtemplate_checked = 0;
        var chkselections = document.getElementsByName("subscribe_to[]");
        for (var i = 0; i < chkselections.length; i++)
            if (chkselections[i].checked)
                total_newsletter_checked ++;
        var radioemailtemplates = document.getElementsByName("send_email_template_id");
        
        for (var i = 0; i < radioemailtemplates.length; i++)
            if (radioemailtemplates[i].checked)
                total_emailtemplate_checked ++;
        if (total_emailtemplate_checked == 0)
        {
            SCHLIX.Alert.error('At least one email template must be selected');
            return false;            
        }
        if (total_newsletter_checked == 0)
        {
            SCHLIX.Alert.error('At least one newsletter must be initially assigned for the invite request');
            return false;
        }
        if (total_file_size > 0)
        {
            if (total_file_size > max_file_size)
            {
                SCHLIX.Alert.error('Total file size (' + this.formatFileSize(total_file_size) + ') is too large for this server. Max = ' + this.formatFileSize(max_file_size));
                event.preventDefault();
                event.stopPropagation();
                return false;
            }
            
            btn_upload.disabled = true;
            prefix_file_size.innerHTML = '<i class="fa fa-spinner fa-spin fa-2x fa-fw"></i>Uploading';
            btn_cancel.style.display = 'none';
        } else 
        {
            SCHLIX.Alert.error('You have not selected a file yet');
        }
    };    

    onShowTSVUploadDialog ()
    {
        var obj = SCHLIX.CMS.getObject('schlix-cms-upload-tsv-dialog');
        obj.show();
        
    }; // end func
     
    onDOMReady ()
    {
        SCHLIX.Event.delegate('schlix-userhistory-admin', 'click', this.onShowViewLogDialog, 'a.open-viewlog', this, true);
        SCHLIX.Event.delegate('schlix-sentmails-admin', 'click', this.onShowViewMailDialog, 'a.open-viewmail', this, true);
        SCHLIX.Event.on('btn-tsv-upload','click', this.onSubmitTSVUploadDialog, this, true);
        SCHLIX.Event.on('filedata', 'change', this.updateFileListToUpload, this, true);
        
    }; // end func
    
//// 
///////////////////////////////////////////////////////////
    onReceiveMail (o)
    {
        var email = ajax_parse(o.responseText);
        var _$ = SCHLIX.Dom.get;
        
        var obj = SCHLIX.CMS.getObject('schlix-cms-emailqueueviewer-dialog');
        var message_content = SCHLIX.Dom.get('message-content');
  //      message_content.innerHTML = tgt_message;

        _$('email_from').value = email.field_from_name + "<" +email.field_from_email + ">";
        _$('email_to').value = email.field_to_name + "<" +email.field_to_email + ">";
        _$('email_subject').value = email.field_subject;
        _$('email_datequeued').value = email.date_queued;
        _$('email_datesent').value = email.date_sent;
        _$('email_bodytext').innerHTML = email.body_text;
        _$('email_bodyhtml').innerHTML = email.body_html;
        _$('email_bodyhtml').style.maxHeight = '300px';
        _$('email_bodytext').style.maxWidth = '300px';
        _$('email_bodytext').style.overflow = 'scroll';
        /////////
        _$('email_queue_marker').value = email.queue_marker ;
        var link = site_httpbase + "/admin/app/" + 'newsletters.subscribers' + "?action=edititem&id=" + email.subscriber_id;
        _$('subscriber_link').innerHTML = '<a target="_blank" href="' + link + '">' + email.subscriber_id + '</a>';
        _$('email_errors').innerHTML = (email.errors) ? '<div class="alert alert-danger">' + SCHLIX.Util.escapeHTML(email.errors) + '</div>' : '';
        obj.show();        
/*
        var obj = SCHLIX.CMS.getObject('schlix-cms-logviewer-dialog');
        var message_content = SCHLIX.Dom.get('message-content');
        message_content.innerHTML = this.formatTable(log);
        message_content.style.maxHeight = '300px';
        message_content.style.overflow = 'scroll';
        obj.show();*/
    }; // end func
    
    onShowViewMailDialog (e)
    {
        var el = SCHLIX.Dom.get(e.target.id);
        var data_email_id = el.getAttribute('data-email-id');
        if (data_email_id)
        {
            //this.ajaxRequestGET(this.receiveLog, "action=getcronlog&id=" + tgt_cron_id);
            var request = SCHLIX.Ajax.GET( site_httpbase + "/admin/app/" + 'newsletters.queues' + "?action=getemail&id=" + data_email_id,
                    {success: this.onReceiveMail, failure: this.handleFailure, scope: this}
            );
        }
    }; // end func
    
    
    onShowViewLogDialog (e)
    {
        var obj = SCHLIX.CMS.getObject('schlix-cms-logviewer-dialog');
    var el = SCHLIX.Dom.get(e.target.id);
    var tgt_message = el.getAttribute('data-log-message');
        
        var message_content = SCHLIX.Dom.get('message-content');
        message_content.innerHTML = tgt_message;
        obj.show();
    }; // end func
    runCommand (command, evt)
    {
        switch (command)
        {
            case 'import-phplist':
                this.redirectToCMSCommand("importphplist",open_in_new_window);
                return true;
                break;
            
            
            case 'import-tsv':
                this.onShowTSVUploadDialog();
                break;
            
            default:
                return super.runCommand(command, evt);
        }
    }
};