<?php
namespace App;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
 

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
class Newsletters_SubscriberSegments_Admin extends \SCHLIX\cmsAdmin_List
{

//_________________________________________________________________________//    
    public function __construct()
    {
        // Data: Item
        parent::__construct( true, array());
        $this->search_field = 'description';
        
        $this->setItemFieldNamesForAjaxListing('id', 'title', 'snapshot_count');
        $this->extra_custom_fields = $this->app->createSubClassInstance('CustomFields')->getAllCustomFields();
        
    }
    
    public function getExtraCustomFieldsAsOptions()
    {
        $result = [];
        foreach ($this->extra_custom_fields as $field)
        {
            switch ($field['type'])
            {
                case 'textbox':
                default: $type = 'string'; $input_type = 'text';break;
            }
            $result[] = ['id' => $field['virtual_filename'], 'label' => $field['title'], 'type' => $type];
        }
        return json_encode($result);
    }
    

    public function ajaxGetSubscriberRequestsSchema()
    {
        $subscriber_requests_admin = new \App\Newsletters_Subscribers_Admin();
        return $subscriber_requests_admin->ajaxGetAllDataResponseSchema();        
    }
    
    public function ajaxGetSubscriberForSegmentID($id,  $start, $end, $sortby, $sortdirection) 
    {
        
        $item = $this->getItemByID($id);
        if ($item)
        {
            $sql_query = '('.$item['sql_query'].') AND (status > 0)';
            $ns_sub_adm = new \App\Newsletters_Subscribers_Admin();
            return $ns_sub_adm->ajaxSearchByCriteria($sql_query, $start, $end, $sortby, $sortdirection);
        }
    }
    
    public function ajaxGetSubscriberSegmentCount($id)
    {
        $item = $this->getItemByID($id);
        if ($item)
        {
            
            return ajax_reply(200, (int) $item['snapshot_count']);
        } else
        return ajax_reply(400, "Item not found");
    }
    
    /**
     * @global \App\Users $CurrentUser
     * @global \App\EmailQueue $SystemMail
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    public function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval) 
    {
        
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        if ($retval['status'] == SAVE_OK)
        {
            $ns = new \App\Newsletters_Subscribers();
            $sql = $datavalues['sql_query'] ? "(status = 1) AND ({$datavalues['sql_query']})" : 'status = 1';
            $datavalues['snapshot_count']= $ns->getTotalItemSearchCountWithCustomFields($sql);
            $this->table_items->quickUpdate($datavalues, "id = {$retval['id']}");
        }
    }
    
    //_________________________________________________________________________//
    public function Run() {
        switch (fget_alphanumeric('action')) {
            case 'getsubscriberresponseschema' :
                return ajax_echo($this->ajaxGetSubscriberRequestsSchema());
                break; 
            case 'getsubscribersegmentcount':
                return ajax_echo($this->ajaxGetSubscriberSegmentCount(fget_int('id')));
                break;                 
            case 'getbysegment':
                return ajax_echo($this->ajaxGetSubscriberForSegmentID(fget_int('id'), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break;
            default:return parent::Run();
        }
    }    
}