<?php
/**
 * Newsletters - Main admin view template
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */
if (!defined('SCHLIX_VERSION')) die();
?>
<?php

$newsletter_app = new \App\Newsletters();
$all_newsletters = $newsletter_app->getAllCategories('*', 'status > 0');

$newsletter_email_template_app = $this->app->createSubClassInstance('EmailTemplates');
$all_invite_templates = $newsletter_email_template_app->getAllItems('*', "virtual_filename = 'import-consent' OR is_system_message = 0 OR is_system_message IS NULL");
?>
<div class="schlix-explorer">
    <schlix-data-explorer-list id="schlix-<?= $this->app->getApplicationNameOnly() ?>-admin" class="schlix-data-explorer"  data-schlix-controller="SCHLIX.CMS.NewslettersSubscribersAdmin" data-main-app-description="<?= ___('Newsletters: Subscribers') ?>" data-app="<?= $this->app->getApplicationNameOnly() ?>" data-model="<?= $this->getDataModelURL() ?>" data-default-item-icon="fa fa-user fa-2x">
<?php $this->loadTemplateFile('view.main.admin.topenu', null) ?>
        <nav class="navbar navbar-default schlix-data-explorer-toolbar" role="navigation">
            <!-- for mobile -->
            <button type="button" class="navbar-toggle collapsed schlix-toolbar-toggle" data-toggle="collapse" data-target="#schlix-<?= $this->app->getApplicationNameOnly() ?>-admin-toolbar">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <!-- end for mobile -->

            <div class="collapse navbar-collapse" schlix-id="toolbar">
                <ul class="nav navbar-nav">
                    <li><a href="#" schlix-id="btn-new-item" data-schlix-command="new-item" data-schlix-app-action="newitem"  class="schlix-command-button"><i class="fa  fa-user-plus"></i> <?= ___('New Subscriber') ?></a></li>
                    <li><a href="#" schlix-id="btn-delete" data-schlix-command="delete" class="schlix-command-button require-selected-items"><i class="fa fa-user-times"></i> <?= ___('Delete') ?></a></li>
                    
                    
                    <li><a href="#" schlix-id="btn-refresh" data-schlix-command="refresh" class="schlix-command-button"><i class="fa fa-refresh"></i> <?= ___('Refresh') ?></a></li>
                    <li><a href="#" schlix-id="btn-import-tsv" data-schlix-command="import-tsv" class="schlix-command-button"><i class="fa fa-upload"></i> <?= ___('Import from a TSV/CSV file') ?></a></li>

                </ul>

                <div class="col-sm-3 col-md-3">
                    <form schlix-id="search-form" class="navbar-form" role="search">
                        <div class="btn-group">
                            <input schlix-id="search-keyword" type="search" class="form-control border-left-none search-with-clear"  placeholder="<?= ___('Search...') ?>">
                        </div>                    
                    </form>
                </div>
                <ul class="nav navbar-nav navbar-right">
                    <li class="dropdown">
                        <a href="#" class="dropdown-toggle" data-toggle="dropdown"><?= ___('Help') ?> <b class="caret"></b></a>
                        <ul class="dropdown-menu">
                            
                            <li class="divider"></li>
                            <li><a href="#" schlix-id="btn-help-about" data-schlix-command="help-about" data-schlix-app-action="helpabout" class="schlix-command-button"><?= ___('About') ?></a></li>
                        </ul>
                    </li>
                </ul>
            </div><!-- /.navbar-collapse -->
        </nav>

        <schlix-data-breadcrumb schlix-id="breadcrumb">
        </schlix-data-breadcrumb>
        <div style=" clear:both"></div>
        <!-- Begin Data Viewer -->
        <div class="schlix-data-container row" schlix-id="main-data-container" style="margin:0">
            <div class="col-sm-12" schlix-id="right-column" >
                
                <schlix-data-table schlix-id="datanav-datatable" data-default-sort-by="id" data-default-sort-direction="desc">
                    <schlix-data-table-columns>
                        <schlix-data-table-column key="check" label="this.getSelectAllCheckboxTableHeader()" formatter="this.formatDataTableCell_CheckBox" sortable="false" />
                        <schlix-data-table-column key="id" hidden="false" sortable="true" formatter="number" />
                        <schlix-data-table-column key="email_address" label= "<?= ___('Email Address') ?>" formatter="this.formatDataTableCell_DefaultTitleColumn" sortable="true" />                        
                        <?php foreach ($this->extra_custom_fields as $field) : ?>
                        <?php if (str_contains($field['virtual_filename'],'ame')): ?>
                        <schlix-data-table-column key="<?= $field['virtual_filename'] ?>" label="<?=  SAFE_HTML($field['title']) ?>" formatter="string" sortable="false"  />
                        <?php else: ?>
                        <schlix-data-table-column key="<?= $field['virtual_filename'] ?>" label="<?=  SAFE_HTML($field['title']) ?>" formatter="string" sortable="false"  />
                        <?php endif ?>
                        <?php endforeach ?>
                        <schlix-data-table-column key="status" label='<?= ___('Status') ?>' sortable="true" formatter="SCHLIX.CMS.NewslettersSubscribersAdmin.formatDataTableCell_SubscriberStatus" />
                        <schlix-data-table-column key="date_created" label='<?= ___('Date Created') ?>' sortable="true" formatter="this.formatDataTableCell_Date" />
                        
                        
                     </schlix-data-table-columns>
                </schlix-data-table>                
                <div schlix-id="datanav-paginator" class="schlixui-pg-container"></div>
            </div>
        </div>
        <!-- End Data Viewer -->
    </schlix-data-explorer-list>
    <div class="subscriber-count">
        <?php $subscriber_count_info = $this->getSubscriberCountArray(); ?>
        <p><?= ___('Active') ?>: <?= $subscriber_count_info['active'] ?> | <?= ___('New/Pending') ?>: <?= $subscriber_count_info['pending'] ?> | <?= ___('Unsubscribed') ?>: <?= $subscriber_count_info['unsubscribed'] ?></p>
    </div>
</div>
 


<!-- Import Dialog -->
<div class="schlix-cms-common-dialog" id="schlix-cms-upload-tsv-dialog" data-fixedcenter="true" data-visible="false" data-constraintoviewport="true" data-schlix-controller="SCHLIX.CMS.NewslettersSubscribersAdmin" data-model="<?= $this->getDataModelURL() ?>">    
    <div class="panel-header"><?= ___("Upload a TSV/CSV (tab or comma separated file)") ?></span></div>
    <div class="panel-body">
        <div id="container-upload" style="min-width:320px">
            <!-- form -->
                <div class="alert alert-warning ">
                    <p><?= ___('Imported email address should be collected in accordance to your local law') ?>.</p>
                    <p><?= ___('Warning - this may take hours if you have lots of records. Recommended maximum record: 1,000') ?></p>
                    <p><?= ___('Maximum file size').': '.' '.round(get_real_max_upload_size()/(1024.0*1024.0), 2).' '.___('Mb') ?></p>
                </div>
            <?php 
                if ($this->extra_custom_fields)
                {
                    $cols = array_column($this->extra_custom_fields,'virtual_filename'); 
                    $str_cols = implode(', ', $cols);
                } else $str_cols = NULL;
            ?>
            <p><?= ___('The first row of the file *MUST* contain the column name and you must at least include the [email_address] field.') ?></p>
                <?php if ($str_cols): ?>
            <p><?= ___('Available custom field columns') ?>: <strong><?= $str_cols ?></strong></p>
                <?php endif ?>
                
            
            <form method="post" id="frm_upload" class="schlix-form-install" action="/admin/app/<?= $app ?>?action=importtsv" data-action="action=importtsv" enctype="multipart/form-data">
                    <input type="hidden" id="__max_file_size" name="MAX_FILE_SIZE" value="<?= get_real_max_upload_size() ?>" />                     
                    <input type="hidden" id="__total_file_size" name="__total_file_size" value="0" />                     
                
                <?= \INPUT::CSRF() ?>
                <div class="row">
                    <div class="col-xs-6 col-md-6">
                        <span class="btn btn-default btn-file"><i class="fa fa-folder-open-o"></i> <?= ___("Select a CSV/TSV file") ?>
                            <input type="file"  id="filedata" class="form-control" name="filedata" required="required" />
                        </span>
                    </div>
                </div>
                    <div class="row">
                        <div class="col-xs-12">
                            <div class="radio">
                                <label>
                                    <input type="radio" name="separator" required="required" id="separator-comma" value="1" checked="checked"  /> <?= ___('Use TAB as the field separator (recommended)') ?>
                                </label>
                            </div>                            
                            <div class="radio">
                                <label>
                                    <input type="radio" name="separator" required="required" id="separator-tsv" value="2" /> <?= ___('Use COMMA as the field separator') ?>
                                </label>
                            </div>
                            
                        <p><?= ___('Automatically assign the following newsletter') ?>:</p>
                        <?php if ($all_newsletters): ?>
                            <?php foreach ($all_newsletters as $nz): ?>
                                <div class="checkbox"><label><input type="checkbox" name="subscribe_to[]" value="<?= $nz['cid'] ?>" /> <?= SAFE_HTML($nz['title']) ?><label></div>
                            <?php endforeach ?>
                        <?php endif ?>
                        <p><?= ___('Once imported, immediately ask for consent with the following email template') ?>:</p>
                        <?php if ($all_invite_templates): ?>
                            <?php foreach ($all_invite_templates as $tmpl): ?>
                        <div class="radio"><label><input type="radio" name="send_email_template_id" required="required" value="<?= $tmpl['id'] ?>" /> <?= '#'.$tmpl['id'].': '.SAFE_HTML($tmpl['title'].' - '.$tmpl['purpose']) ?></label></div>
                            <?php endforeach ?>
                        <?php endif ?>
                                    <p class="small"><?= ___('You can create your own custom invite email asking for their consent') ?></p>
                        </div>
                    </div>
                    <div style="max-height:100px;overflow: auto">
                    <ul id="filelist" class="list-unstyled" ></ul>
                    </div>
                    <p><span id="filelist-prefix-totalsize"></span>&nbsp;<span id="filelist-display-totalsize"></span></p>
                    <button type="submit" name="upload" id="btn-tsv-upload" data-schlix-command="dialog-submit" class="btn btn-primary"><i class="fa fa-upload"></i>&nbsp;<?= ___("Go") ?></button>
                    <button type="submit" name="cancel" id="btn-tsv-cancel"data-schlix-command="dialog-close" class="btn btn-default"><i class="fa fa-times"></i>&nbsp;<?= ___("Cancel") ?></button>

            </form>        
            <!-- end form -->
        </div>
    </div>
</div>
